-- 岗位最低补偿表
CREATE TABLE `income_compensation` (
   `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
   `post_id` bigint(20) NOT NULL COMMENT '岗位id',
   `post_name` varchar(255) DEFAULT NULL COMMENT '岗位名称',
   `shop_ids` varchar(255) NOT NULL COMMENT '适用门店ids',
   `shop_names` varchar(255) NOT NULL COMMENT '适用门店名称',
   `months` varchar(255) NOT NULL COMMENT '多个生效月份',
   `money` decimal(10,4) DEFAULT NULL COMMENT '单人补偿金额',
   `max_money` decimal(10,4) DEFAULT NULL COMMENT '单人最高补偿金额',
   `status` tinyint(3) NOT NULL COMMENT '状态;1:审批中 3:生效中 4:已失效 ',
   `group_id` bigint(20) NOT NULL COMMENT '集团id',
   `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
   `create_by` bigint(20) DEFAULT NULL COMMENT '创建人',
   `create_time` datetime DEFAULT NULL COMMENT '创建时间',
   `update_by` bigint(20) DEFAULT NULL COMMENT '更新人',
   `update_time` datetime DEFAULT NULL COMMENT '更新时间',
   PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='收入补偿';

-- 新增SalaryTypeEnum 岗位最低收入补偿,新增计算器计算



-- 业务系统需要监听所有启用中的绩效指标，通过系统id获取到指标，上报所有人的绩效指标数据。考评、薪酬、绩效需要指标数据就到这里搜
-- 数据类型 当选择综合指标（只能相同类型的指标能一起计算）
-- 适用范围类型 用于筛选指标，比如工龄只能在薪酬组中使用，考评组、绩效组中不会展示工龄指标
ALTER TABLE `fw_morax`.`indicators`
    ADD COLUMN `data_type` tinyint(3) NOT NULL COMMENT '数据类型；1：台数  2：百分比 3：金额' AFTER `target_type`,
ADD COLUMN `apply_types` varchar(255) NOT NULL COMMENT '适用范围类型；1：绩效组 2：薪酬组 3：考评组' AFTER `data_type`;
-- 保存 当月离职员工、调岗员工 最后在绩效组的日期，用于查询详情
ALTER TABLE `fw_morax`.`kpi_pool`
    ADD COLUMN `data_date` date NOT NULL COMMENT '数据日期' AFTER `inclusion`;

CREATE TABLE `kpi_indicator_value` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
    `user_id` bigint(20) NOT NULL COMMENT '人员id',
    `indicator_code` varchar(255) NOT NULL COMMENT '指标编码',
    `indicator_name` varchar(255) DEFAULT NULL COMMENT '指标库名称',
    `indicator_value` varchar(2000) NOT NULL COMMENT '指标业务值;jons字符串 map类型',
    `value_type` tinyint(3) NOT NULL COMMENT '类型;1：台数  2：百分比 3：金额 4：条件值',
    `data_date` date DEFAULT NULL COMMENT '数据日期',
    `group_id` bigint(20) NOT NULL COMMENT '集团id',
    `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
    `create_time` datetime DEFAULT NULL COMMENT '创建时间',
    `update_time` datetime DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`id`),
    KEY `kpi_indicator_value_data_date_index` (`data_date`),
    KEY `kpi_indicator_value_user_id_index` (`user_id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效人员指标业务值';

-- 绩效
-- 1. 绩效组指标：kpi_group_indicator	新增字段：名称 目标计算类型
ALTER TABLE `fw_morax`.`kpi_group_indicator`
DROP COLUMN `indicator_code`,
DROP COLUMN `indicator_name`,
DROP COLUMN `target_type`,
DROP COLUMN `target_value`,
ADD COLUMN `name` varchar(255) NOT NULL COMMENT '指标名称' AFTER `id`;
-- 修改后
CREATE TABLE `kpi_group_indicator` (
   `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
   `kpi_group_id` bigint(20) NOT NULL COMMENT '绩效组配置id',
   `name` varchar(255) DEFAULT NULL COMMENT '指标名称',
   `base_score` int(11) NOT NULL COMMENT '基础绩效分',
   `score_way` tinyint(3) DEFAULT NULL COMMENT '得分方式;1: 阶梯得分 2:普通得分',
   `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
   `create_time` datetime DEFAULT NULL COMMENT '创建时间',
   `update_time` datetime DEFAULT NULL COMMENT '更新时间',
   PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效组指标项';
-- 2. 绩效组新增关联表   绩效组指标参数（一对多）：
CREATE TABLE `kpi_group_indicator_param` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
    `kpi_group_id` bigint(20) NOT NULL COMMENT '绩效组配置id',
    `kpi_group_indicator_id` bigint(20) NOT NULL COMMENT '绩效组指标id',
    `cond_type` tinyint(3) NOT NULL COMMENT '条件类型（1.台阶条件，2，提成条件）',
    `indicator_code` varchar(255) NOT NULL COMMENT '指标编码',
    `indicator_name` varchar(255) DEFAULT NULL COMMENT '指标库名称',
    `proportion` decimal(10,4) DEFAULT NULL COMMENT '占比',
    `cap` tinyint(1) DEFAULT '1' COMMENT '是否封顶',
    `target_calc_type` tinyint(3) DEFAULT NULL COMMENT '目标计算类型;1: 目标值计算 2:最低要求计算',
    `target_type` tinyint(3) DEFAULT '1' COMMENT '绩效目标值类型; 1:无 2:百分比 3:金额 4:台数',
    `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
    `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
    `create_time` datetime DEFAULT NULL COMMENT '创建时间',
    `update_time` datetime DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效组指标参数';

-- 前置条件表（指标、指标目标、顺序）
CREATE TABLE `kpi_group_indicator_cond` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
    `kpi_group_id` bigint(20) NOT NULL COMMENT '绩效组配置id',
    `kpi_group_indicator_id` bigint(20) NOT NULL COMMENT '绩效组指标id',
    `indicator_code` varchar(255) NOT NULL COMMENT '指标编码',
    `indicator_name` varchar(255) DEFAULT NULL COMMENT '指标库名称',
    `target_calc_type` tinyint(3) DEFAULT NULL COMMENT '目标计算类型;1: 目标值计算 2:最低要求计算',
    `target_type` tinyint(3) DEFAULT '1' COMMENT '绩效目标值类型; 1:无 2:百分比 3:金额 4:台数',
    `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
    `sort` int(11) NOT NULL COMMENT '排序',
    `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
    `create_time` datetime DEFAULT NULL COMMENT '创建时间',
    `update_time` datetime DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效组指标条件';
-- 前置条件阶梯表（阶梯值、分值）
CREATE TABLE `kpi_group_indicator_cond_ladders` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
    `kpi_group_indicator_cond_id` bigint(20) NOT NULL COMMENT '绩效组指标条件id',
    `lower` decimal(10,4) NOT NULL COMMENT '阶梯下限;大于等于',
    `upper` decimal(10,4) NOT NULL DEFAULT '65536.0000' COMMENT '阶梯上限;小于; 无上限的情况值为 2<<15',
    `score_percent` decimal(10,4) DEFAULT NULL COMMENT '得分百分比',
    `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
    `create_time` datetime DEFAULT NULL COMMENT '创建时间',
    `update_time` datetime DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效组指标条件得分阶梯';

CREATE TABLE `kpi_group_indicator_target_log` (
     `id` bigint(20) NOT NULL AUTO_INCREMENT,
     `kpi_group_indicator_cond_id` bigint(20) DEFAULT NULL COMMENT '绩效组指标条件id',
     `kpi_group_indicator_param_id` bigint(20) COMMENT DEFAULT NULL '绩效组指标参数id',
     `value` decimal(18,2) DEFAULT NULL COMMENT '原始值',
     `reach_value` decimal(18,2) DEFAULT NULL COMMENT '达成目标',
     `data_date` date NOT NULL COMMENT '数据日期',
     `group_id` bigint(20) NOT NULL COMMENT '集团id',
     `create_time` datetime DEFAULT NULL,
     `update_time` datetime DEFAULT NULL,
     `yn` tinyint(1) DEFAULT '1',
     PRIMARY KEY (`id`),
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效指标达成目标记录';

CREATE TABLE `kpi_group_indicator_cond_log` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT,
    `kpi_group_indicator_cond_id` bigint(20) DEFAULT NULL COMMENT '绩效组指标条件id',
    `kpi_group_indicator_cond_ladders_id` bigint(20) DEFAULT NULL COMMENT '命中绩效组指标条件得分阶梯id',
    `value` decimal(18,2) DEFAULT NULL COMMENT '值',
    `data_date` date NOT NULL COMMENT '数据日期',
    `group_id` bigint(20) NOT NULL COMMENT '集团id',
    `create_time` datetime DEFAULT NULL,
    `update_time` datetime DEFAULT NULL,
    `yn` tinyint(1) DEFAULT '1',
    PRIMARY KEY (`id`),
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效指标命中条件记录';

CREATE TABLE `kpi_group_indicator_log` (
   `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
   `kpi_group_indicator_id` bigint(20) NOT NULL COMMENT '绩效组指标项id',
   `score` int(11) NOT NULL COMMENT '得分',
   `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
   `create_time` datetime DEFAULT NULL COMMENT '创建时间',
   `update_time` datetime DEFAULT NULL COMMENT '更新时间',
   PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效组指标项得分记录';


-- 薪酬 各业务系统将各系统所有人的薪酬指标每天凌晨上报，有几种计算方式就上报几种
-- 删除salary_raw_data表
DROP TABLE salary_raw_data;
-- 保存 当月离职员工、调岗员工 最后在绩效组的日期，用于查询详情
ALTER TABLE `fw_morax`.`kpi_pool`
    ADD COLUMN `data_date` date NULL COMMENT '数据日期' AFTER `inclusion`;

ALTER TABLE `fw_morax`.`salary_project`
DROP COLUMN `salary_code`,
DROP COLUMN `salary_name`,
ADD COLUMN `name` varchar(255) NOT NULL COMMENT '名称' AFTER `id`;

CREATE TABLE `salary_group_project_param` (
     `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
     `salary_group_id` bigint(20) NOT NULL COMMENT '薪酬组id',
     `salary_group_project_id` bigint(20) NOT NULL COMMENT '薪酬组项目id',
     `param_type` tinyint(3) NOT NULL COMMENT '参数类型（1.台阶，2，提成）',
     `code_type` tinyint(3) NOT NULL COMMENT '编码类型（1.指标，2.综合指标组合）',
     `indicator_code` varchar(255) NOT NULL COMMENT '指标编码',
     `indicator_name` varchar(255) DEFAULT NULL COMMENT '指标库名称',
     `target_type` tinyint(3) DEFAULT '1' COMMENT '目标值类型; 1:无 2:百分比 3:金额 4:台数',
     `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
     `proportion` decimal(10,4) DEFAULT NULL COMMENT '占比',
     `cap` tinyint(1) DEFAULT '1' COMMENT '是否封顶',
     `sort` int(11) NULL COMMENT '排序',
     `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
     `create_time` datetime DEFAULT NULL COMMENT '创建时间',
     `update_time` datetime DEFAULT NULL COMMENT '更新时间',
     PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='薪酬组薪酬项目参数';

CREATE TABLE `salary_group_project_cond` (
     `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
     `salary_group_id` bigint(20) NOT NULL COMMENT '薪酬组id',
     `salary_group_project_id` bigint(20) NOT NULL COMMENT '薪酬组项目id',
     `code_type` tinyint(3) NOT NULL COMMENT '编码类型（1. 指标，2.综合指标组合，3.薪酬指标）',
     `indicator_code` varchar(255) NOT NULL COMMENT '指标编码',
     `indicator_name` varchar(255) DEFAULT NULL COMMENT '指标库名称',
     `target_type` tinyint(3) DEFAULT '1' COMMENT '条件值类型; 1:无 2:百分比 3:金额 4:台数',
     `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
     `cond_value` decimal(10,4) DEFAULT NULL COMMENT '条件值',
     `sort` int(11) NULL COMMENT '排序',
     `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
     `create_time` datetime DEFAULT NULL COMMENT '创建时间',
     `update_time` datetime DEFAULT NULL COMMENT '更新时间',
     PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='薪酬组薪酬项目前置条件';

CREATE TABLE `salary_group_project_target_log` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `salary_group_project_id` bigint(20) NOT NULL COMMENT '薪酬组项目id',
  `salary_group_project_cond_id` bigint(20) DEFAULT NULL COMMENT '薪酬组薪酬项目前置条件id',
  `salary_group_project_param_id` bigint(20) DEFAULT NULL COMMENT '薪酬组薪酬项目参数id',
  `value` decimal(18,2) DEFAULT NULL COMMENT '原始值',
  `reach_value` decimal(18,2) DEFAULT NULL COMMENT '达成目标',
  `data_date` date NOT NULL COMMENT '数据日期',
  `create_time` datetime DEFAULT NULL,
  `update_time` datetime DEFAULT NULL,
  `yn` tinyint(1) DEFAULT '1',
  PRIMARY KEY (`id`),
  UNIQUE KEY `salary_project_cond_log_index` (`salary_group_project_id`,`salary_pool_id`,`data_date`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='薪酬指标达成目标记录';

CREATE TABLE `salary_group_project_cond_log` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `salary_group_project_id` bigint(20) NOT NULL COMMENT '薪酬组项目id',
  `salary_group_project_cond_id` bigint(20) DEFAULT NULL COMMENT '薪酬组薪酬项目前置条件id',
  `value` decimal(18,2) DEFAULT NULL COMMENT '值',
  `data_date` date NOT NULL COMMENT '数据日期',
  `create_time` datetime DEFAULT NULL,
  `update_time` datetime DEFAULT NULL,
  `yn` tinyint(1) DEFAULT '1',
  PRIMARY KEY (`id`),
  UNIQUE KEY `salary_project_cond_log_index` (`salary_group_project_id`,`salary_pool_id`,`data_date`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='薪酬指标条件记录';

CREATE TABLE `salary_group_project_hit_log` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `salary_project_id` bigint(20) NOT NULL COMMENT '薪酬项id',
  `salary_pool_id` bigint(20) NOT NULL COMMENT '薪酬池id',
  `salary_group_project_settin_id` bigint(20) DEFAULT NULL COMMENT '命中具体配置项id',
  `salary_value` decimal(18,2) DEFAULT NULL COMMENT '薪酬值',
  `data_date` date NOT NULL COMMENT '数据日期',
  `group_id` bigint(20) NOT NULL COMMENT '集团id',
  `create_time` datetime DEFAULT NULL,
  `update_time` datetime DEFAULT NULL,
  `yn` tinyint(1) DEFAULT '1',
  PRIMARY KEY (`id`),
  UNIQUE KEY `salary_project_hit_log_index` (`salary_project_id`,`salary_pool_id`,`data_date`,`salary_group_project_settin_id`) USING BTREE
) ENGINE=InnoDB AUTO_INCREMENT=2866 DEFAULT CHARSET=utf8mb4 COMMENT='用户薪酬项命中记录';



-- 考评
CREATE TABLE `eval_indicator` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
  `code` varchar(255) NOT NULL COMMENT '考评指标编码',
  `name` varchar(255) DEFAULT NULL COMMENT '考评指标名称',
  `sys_id` bigint(20) DEFAULT NULL COMMENT '归属系统id',
  `sys_prefix` varchar(64) DEFAULT NULL COMMENT '归属系统前缀',
  `sys_name` varchar(32) DEFAULT NULL COMMENT '归属系统名称',
  `role_code` varchar(64) DEFAULT NULL COMMENT '导入操作人员角色码',
  `role_name` varchar(64) DEFAULT NULL COMMENT '导入操作人员角色名称',
  `rule_type` tinyint(3) DEFAULT NULL COMMENT '频率类型; 1:按滚动天 2:按星期 3:按月指定日期 4：按滚动月',
  `rule_values` varchar(256) DEFAULT NULL COMMENT '频率值 ,号分隔（按星期、按月指定日期使用）',
  `roll_value` int(11) DEFAULT NULL COMMENT '滚动月数/滚动天数',
  `start_time` date DEFAULT NULL COMMENT '开始日期',
  `month_day_type` tinyint(3) DEFAULT NULL COMMENT '月天; 1:月第几天 2:月倒数第几天',
  `month_day` tinyint(3) DEFAULT NULL COMMENT '天',
  `target_type` tinyint(3) DEFAULT '1' COMMENT '目标值类型; 1:无 2:百分比 3:金额 4:台数',
  `enable` tinyint(1) NOT NULL COMMENT '启用/禁用;0:禁用 1:启用',
  `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
  `create_time` datetime DEFAULT NULL COMMENT '创建时间',
  `update_time` datetime DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评指标';

CREATE TABLE `eval_group` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
  `egc` varchar(60) NOT NULL COMMENT '考评组编码',
  `name` varchar(255) DEFAULT NULL COMMENT '考评，活动名称',
  `type` tinyint(3) DEFAULT '1' COMMENT '考评类型; 1:活动考评 2:其他考评',
  `post_id` bigint(20) NOT NULL COMMENT '岗位id',
  `post_name` varchar(255) DEFAULT NULL COMMENT '岗位名称',
  `shop_ids` varchar(255) NOT NULL COMMENT '适用门店ids',
  `shop_names` varchar(255) NOT NULL COMMENT '适用门店名称',
  `reason` varchar(512) DEFAULT NULL COMMENT '调整原因',
  `attachment` varchar(512) DEFAULT NULL COMMENT '附件;fid逗号隔开',
  `status` tinyint(3) NOT NULL COMMENT '状态;1:审批中 2:审批拒绝 3:生效中 4:草稿 ',
  `indicator_num` int(5) DEFAULT NULL COMMENT '包含指标数量',
  `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
  `create_time` datetime DEFAULT NULL COMMENT '创建时间',
  `update_time` datetime DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评组';

CREATE TABLE `draft` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
  `type` tinyint(3) DEFAULT '1' COMMENT '类型; 1:考评组 ',
  `union_id` bigint(20) NOT NULL COMMENT '关联id',
  `content` text COMMENT '内容【json】',
  `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
  `create_time` datetime DEFAULT NULL COMMENT '创建时间',
  `update_time` datetime DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='草稿';


-- 排名正负激励          阶梯排名（按固定值、按比例）
-- 阶梯奖励（累加）       阶梯翻倍
-- 总金额X达成率占比      固定金额占比
-- 固定值X达成率         固定金额
-- 固定值+超额部分追加奖励 超额奖励

-- 固定金额占比、固定金额、超额奖励  使用
CREATE TABLE `eval_group_reward` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
    `name` varchar(255) DEFAULT NULL COMMENT '考评，活动名称',
    `cal_method` int(3) NOT NULL COMMENT '计算方式; 1: 排名正负激励 2:阶梯奖励(翻倍) 3:总金额X达成率占比 4:固定值X达成率 5: 固定值+超额部分追加奖励',
    `rank_type` tinyint(3) NOT NULL COMMENT '排名方式;1:排名序号 2:排名人数百分比',
    `money` decimal(10,4) DEFAULT NULL COMMENT '金额',
    `extra_money` decimal(10,4) DEFAULT NULL COMMENT '追加奖励 XX 元/台，超额奖励用',
    `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
    `create_time` datetime DEFAULT NULL COMMENT '创建时间',
    `update_time` datetime DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评奖惩';

-- 阶梯排名、阶梯翻倍  使用
CREATE TABLE `eval_group_reward_ladders` (
     `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
     `eval_group_reward_id` bigint(20) NOT NULL COMMENT '绩效组指标条件id',
     `lower` decimal(10,4) NOT NULL COMMENT '阶梯下限;大于等于',
     `upper` decimal(10,4) NOT NULL DEFAULT '65536.0000' COMMENT '阶梯上限;小于; 无上限的情况值为 2<<15',
     `money` decimal(10,4) DEFAULT NULL COMMENT '金额',
     `cap_money` decimal(10,4) DEFAULT NULL COMMENT '封顶金额/台',
     `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
     `create_time` datetime DEFAULT NULL COMMENT '创建时间',
     `update_time` datetime DEFAULT NULL COMMENT '更新时间',
     PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评奖惩阶梯';

CREATE TABLE `eval_group_reward_param` (
      `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
      `eval_group_id` bigint(20) NOT NULL COMMENT '考评组id',
      `eval_group_reward_id` bigint(20) NOT NULL COMMENT '考评奖惩id',
      `param_type` tinyint(3) NOT NULL COMMENT '参数类型（1.台阶，2，提成）',
      `code_type` tinyint(3) NOT NULL COMMENT '编码类型（1.指标，2.考评指标）',
      `code` varchar(255) NOT NULL COMMENT '项目编码',
      `name` varchar(255) DEFAULT NULL COMMENT '项目名称',
      `proportion` decimal(10,4) DEFAULT NULL COMMENT '占比',
      `cap` tinyint(1) DEFAULT '1' COMMENT '是否封顶',
      `target_type` tinyint(3) DEFAULT '1' COMMENT '目标值类型; 1:无 2:百分比 3:金额 4:台数',
      `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
      `money` decimal(10,4) DEFAULT NULL COMMENT '金额',
      `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
      `create_time` datetime DEFAULT NULL COMMENT '创建时间',
      `update_time` datetime DEFAULT NULL COMMENT '更新时间',
      PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评奖惩参数';

CREATE TABLE `eval_group_reward_cond` (
     `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
     `eval_group_id` bigint(20) NOT NULL COMMENT '考评组id',
     `eval_group_reward_id` bigint(20) NOT NULL COMMENT '考评奖惩id',
     `cond_type` tinyint(3) DEFAULT '1' COMMENT '条件值类型; 1:满足目标 2:满足排名率 3:金额',
     `code_type` tinyint(3) NOT NULL COMMENT '编码类型（1. 指标，2.考评指标）',
     `code` varchar(255) NOT NULL COMMENT '项目编码',
     `name` varchar(255) DEFAULT NULL COMMENT '项目名称',
     `target_type` tinyint(3) DEFAULT '1' COMMENT '条件值类型; 1:无 2:百分比 3:金额',
     `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
     `cond_value` decimal(10,4) DEFAULT NULL COMMENT '条件值',
     `sort` int(11) NULL COMMENT '排序',
     `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
     `create_time` datetime DEFAULT NULL COMMENT '创建时间',
     `update_time` datetime DEFAULT NULL COMMENT '更新时间',
     PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评奖惩条件';


CREATE TABLE `eval_group_indicator` (
   `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
   `eval_group_id` bigint(20) NOT NULL COMMENT '考评组配置id',
   `name` varchar(255) DEFAULT NULL COMMENT '指标名称',
   `base_score` int(11) NOT NULL COMMENT '基础绩效分',
   `score_way` tinyint(3) DEFAULT NULL COMMENT '得分方式;1: 阶梯得分 2:普通得分',
   `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
   `create_time` datetime DEFAULT NULL COMMENT '创建时间',
   `update_time` datetime DEFAULT NULL COMMENT '更新时间',
   PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='绩效组指标项';
-- 2. 绩效组新增关联表   绩效组指标参数（一对多）：
CREATE TABLE `eval_group_indicator_param` (
     `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
     `eval_group_id` bigint(20) NOT NULL COMMENT '考评组配置id',
     `eval_group_indicator_id` bigint(20) NOT NULL COMMENT '考评组指标id',
     `cond_type` tinyint(3) NOT NULL COMMENT '条件类型（1.台阶条件，2，提成条件）',
     `code_type` tinyint(3) NOT NULL COMMENT '指标类型（1.指标，2，考评指标）',
     `code` varchar(255) NOT NULL COMMENT '指标编码',
     `name` varchar(255) DEFAULT NULL COMMENT '指标库名称',
     `proportion` decimal(10,4) DEFAULT NULL COMMENT '占比',
     `cap` tinyint(1) DEFAULT '1' COMMENT '是否封顶',
     `target_calc_type` tinyint(3) DEFAULT NULL COMMENT '目标计算类型;1: 目标值计算 2:最低要求计算',
     `target_type` tinyint(3) DEFAULT '1' COMMENT '绩效目标值类型; 1:无 2:百分比 3:金额 4:台数',
     `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
     `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
     `create_time` datetime DEFAULT NULL COMMENT '创建时间',
     `update_time` datetime DEFAULT NULL COMMENT '更新时间',
     PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评组指标参数';

-- 前置条件表（指标、指标目标、顺序）
CREATE TABLE `eval_group_indicator_cond` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
    `eval_group_id` bigint(20) NOT NULL COMMENT '考评组配置id',
    `eval_group_indicator_id` bigint(20) NOT NULL COMMENT '考评组指标id',
    `indicator_code` varchar(255) NOT NULL COMMENT '指标编码',
    `indicator_name` varchar(255) DEFAULT NULL COMMENT '指标库名称',
    `target_calc_type` tinyint(3) DEFAULT NULL COMMENT '目标计算类型;1: 目标值计算 2:最低要求计算',
    `target_type` tinyint(3) DEFAULT '1' COMMENT '绩效目标值类型; 1:无 2:百分比 3:金额 4:台数',
    `target_value` decimal(10,4) DEFAULT NULL COMMENT '目标值',
    `sort` int(11) NOT NULL COMMENT '排序',
    `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
    `create_time` datetime DEFAULT NULL COMMENT '创建时间',
    `update_time` datetime DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评组指标条件';
-- 前置条件阶梯表（阶梯值、分值）
CREATE TABLE `eval_group_indicator_cond_ladders` (
    `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
    `eval_group_indicator_cond_id` bigint(20) NOT NULL COMMENT '考评组指标条件id',
    `lower` decimal(10,4) NOT NULL COMMENT '阶梯下限;大于等于',
    `upper` decimal(10,4) NOT NULL DEFAULT '65536.0000' COMMENT '阶梯上限;小于; 无上限的情况值为 2<<15',
    `score_percent` decimal(10,4) DEFAULT NULL COMMENT '得分百分比',
    `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
    `create_time` datetime DEFAULT NULL COMMENT '创建时间',
    `update_time` datetime DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评组指标条件得分阶梯';

CREATE TABLE `eval_group_indicator_target_log` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `eval_group_id` bigint(20) NOT NULL COMMENT '考评组配置id',
  `eval_group_indicator_param_id` bigint(20) NOT NULL COMMENT '考评组指标参数id',
  `eval_group_indicator_cond_id` bigint(20) NOT NULL COMMENT '考评组指标条件id',
  `value` decimal(18,2) DEFAULT NULL COMMENT '原始值',
  `reach_value` decimal(18,2) DEFAULT NULL COMMENT '达成目标',
  `data_date` date NOT NULL COMMENT '数据日期',
  `group_id` bigint(20) NOT NULL COMMENT '集团id',
  `create_time` datetime DEFAULT NULL,
  `update_time` datetime DEFAULT NULL,
  `yn` tinyint(1) DEFAULT '1',
  PRIMARY KEY (`id`),
  UNIQUE KEY `salary_project_hit_log_index` (`kpi_group_indicator_id`,`kpi_pool_id`,`data_date`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评指标达成目标记录';

CREATE TABLE `eval_group_indicator_cond_log` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `eval_group_id` bigint(20) NOT NULL COMMENT '考评组配置id',
  `eval_group_indicator_cond_id` bigint(20) NOT NULL COMMENT '考评组指标条件id',
  `eval_group_indicator_cond_ladders_id` bigint(20) NOT NULL COMMENT '考评组指标条件阶梯id',
  `value` decimal(18,2) DEFAULT NULL COMMENT '原始值',
  `data_date` date NOT NULL COMMENT '数据日期',
  `group_id` bigint(20) NOT NULL COMMENT '集团id',
  `create_time` datetime DEFAULT NULL,
  `update_time` datetime DEFAULT NULL,
  `yn` tinyint(1) DEFAULT '1',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评指标条件记录';

CREATE TABLE `eval_group_reward_cond_log` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `eval_group_id` bigint(20) NOT NULL COMMENT '考评组配置id',
  `eval_group_reward_id` bigint(20) NOT NULL COMMENT '考评组奖惩id',
  `eval_group_reward_cond_id` bigint(20) NOT NULL COMMENT '考评组奖惩条件id',
  `value` decimal(18,2) DEFAULT NULL COMMENT '原始值',
  `reach_value` decimal(18,2) DEFAULT NULL COMMENT '达成目标',
  `data_date` date NOT NULL COMMENT '数据日期',
  `group_id` bigint(20) NOT NULL COMMENT '集团id',
  `create_time` datetime DEFAULT NULL,
  `update_time` datetime DEFAULT NULL,
  `yn` tinyint(1) DEFAULT '1',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评奖惩条件记录';

CREATE TABLE `eval_pool` (
   `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
   `egc` varchar(60) NOT NULL COMMENT '考评组编码',
   `eval_group_id` bigint(20) NOT NULL COMMENT '考评组id',
   `user_id` bigint(20) NOT NULL COMMENT '用户id',
   `user_name` varchar(255) DEFAULT NULL COMMENT '用户名称',
   `post_id` bigint(20) NOT NULL COMMENT '岗位id',
   `post_name` varchar(255) DEFAULT NULL COMMENT '岗位名称',
   `shop_id` bigint(20) NOT NULL COMMENT '门店id',
   `shop_name` varchar(255) DEFAULT NULL COMMENT '门店名称',
   `fund_shop_id` bigint(20) DEFAULT NULL COMMENT '社保公积金门店id',
   `fund_shop_name` varchar(255) DEFAULT NULL COMMENT '社保公积金门店名称',
   `reward` decimal(18,2) NOT NULL COMMENT '考评奖惩',
   `score` decimal(10,2) NOT NULL COMMENT '考评得分',
   `score_ratio` decimal(10,2) NOT NULL COMMENT '考评得分率',
   `monthly` varchar(8) NOT NULL COMMENT '月度',
   `regular` tinyint(1) NOT NULL COMMENT '固定的',
   `group_id` bigint(20) NOT NULL COMMENT '集团id',
   `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
   `create_time` datetime DEFAULT NULL COMMENT '创建时间',
   `update_time` datetime DEFAULT NULL COMMENT '更新时间',
   PRIMARY KEY (`id`),
   KEY `eval_pool_monthly_index` (`monthly`),
   KEY `eval_pool_salary_group_id_index` (`eval_group_id`),
   KEY `eval_pool_user_id_index` (`user_id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COMMENT='考评池';

CREATE TABLE `eval_pool_indicator_detail` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
  `eval_pool_id` bigint(20) NOT NULL COMMENT '考评池id',
  `eval_group_indicator_id` bigint(20) DEFAULT NULL COMMENT '考评项id',
  `name` varchar(255) DEFAULT NULL COMMENT '指标名称',
  `base_score` int(11) NOT NULL COMMENT '基础绩效分',
  `score` decimal(10,2) NOT NULL COMMENT '考评得分',
  `score_ratio` decimal(10,2) NOT NULL COMMENT '考评得分率',
  `monthly` varchar(8) NOT NULL COMMENT '月度',
  `group_id` bigint(20) NOT NULL COMMENT '集团id',
  `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
  `create_time` datetime DEFAULT NULL COMMENT '创建时间',
  `update_time` datetime DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3804686 DEFAULT CHARSET=utf8mb4 COMMENT='考评指标明细';

CREATE TABLE `eval_pool_reward_detail` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'ID',
  `eval_pool_id` bigint(20) NOT NULL COMMENT '考评池id',
  `eval_group_reward_id` bigint(20) DEFAULT NULL COMMENT '考评项id',
  `name` varchar(255) DEFAULT NULL COMMENT '指标名称',
  `money` decimal(10,4) DEFAULT NULL COMMENT '金额',
  `monthly` varchar(8) NOT NULL COMMENT '月度',
  `group_id` bigint(20) NOT NULL COMMENT '集团id',
  `yn` tinyint(1) DEFAULT '1' COMMENT '逻辑删除',
  `create_time` datetime DEFAULT NULL COMMENT '创建时间',
  `update_time` datetime DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3804686 DEFAULT CHARSET=utf8mb4 COMMENT='考评奖惩明细';

