package cn.fw.morax.dao.kpi;

import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.dto.query.KpiGroupRepeatQueryDTO;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * @author : kurisu
 * @className : KpiGroupDao
 * @description : 绩效组配置
 * @date: 2022-04-06 14:29
 */
@Repository
public interface KpiGroupDao extends BaseMapper<KpiGroup> {
    /**
     * 根据指标编码和岗位码查询集团所有绩效组id
     *
     * @param indicatorCode 指标编码
     * @param postId        岗位Id
     * @param groupId       集团id
     * @param dataDate      数据日期
     * @return List
     */
    List<Long> queryKgiGroupIdsByCode(@Param("indicatorCode") final String indicatorCode,
                                      @Param("postId") final Long postId,
                                      @Param("groupId") final Long groupId,
                                      @Param("dataDate") final Date dataDate
    );

    /**
     * 根据指标编码和岗位码查询集团所有绩效组
     *
     * @param indicatorCode 指标编码
     * @param postId        岗位Id
     * @param groupId       集团id
     * @param dataDate      数据日期
     * @return List
     */
    List<KpiGroup> queryKgiGroupByCode(@Param("indicatorCode") final String indicatorCode,
                                      @Param("postId") final Long postId,
                                      @Param("groupId") final Long groupId,
                                      @Param("dataDate") final Date dataDate
    );


    /**
     * 查询重复的绩效组配置
     *
     * @param dto
     * @return
     */
    List<KpiGroup> queryRepeatKpiGroup(KpiGroupRepeatQueryDTO dto);

    /**
     * 根据时间查询当时生效的绩效组id
     *
     * @param day
     * @return
     */
    Set<Long> queryKgiGroupIdsByDay(@Param("day") Date day);

    /**
     * 通过id获取，包括已删除数据
     *
     * @param id
     * @return
     */
    KpiGroup getAllDataById(@Param("id") Long id);

    /**
     * 获取生效中数据
     *
     * @param shopId
     * @param postId
     * @param day
     * @return
     */
    KpiGroup getEffectGroup(@Param("shopId") Long shopId, @Param("postId") Long postId, @Param("day") Date day);

    /**
     * 获取所有生效中数据
     *
     * @param day
     * @return
     */
    List<KpiGroup> getAllEffectGroups(@Param("day") Date day);

    /**
     * 门店岗位重复的绩效组
     *
     * @param postId
     * @param shopIds
     * @param preId
     * @return
     */
    List<KpiGroup> getRepeatKpis(@Param("postId") Long postId,
                                 @Param("shopIds") List<Long> shopIds,
                                 @Param("preId") Long preId,
                                 @Param("beginTime") LocalDate beginTime);

    /**
     * 根据时间查询当时生效的绩效组
     *
     * @param day
     * @return
     */
    List<KpiGroup> queryKgiGroupsByDay(@Param("day") Date day);

    /**
     * 通过岗位、门店、时间查询绩效组
     *
     * @param postId
     * @param shopIds
     * @param date
     * @return
     */
    List<KpiGroup> getKpis(@Param("postId") Long postId,
                                 @Param("shopIds") List<Long> shopIds,
                                 @Param("day") LocalDate date);
}
