package cn.fw.morax.domain.db.eval;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.common.data.mybatis.handler.LongListTypeHandler;
import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.RankTypeEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 考评奖惩
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName(autoResultMap = true)
public class EvalGroupReward extends BaseAuditableTimeEntity<EvalGroupReward, Long> {

    private static final long serialVersionUID = 1L;

    /**
     * 考评组配置id
     */
    private Long evalGroupId;

    /**
     * 阶段id
     */
    private Long evalGroupRankStageId;

    /**
     * 维度id
     */
    private Long evalGroupDimId;

    /**
     * 岗位id
     */
    @TableField(typeHandler = LongListTypeHandler.class)
    private List<Long> postIds;

    /**
     * 岗位名称
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> postNames;

    /**
     * 台阶参数别名
     */
    public String ladderParamAlias;

    /**
     * 提成参数别名
     */
    public String commissionParamAlias;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum laddersType;

    /**
     * 前置条件别名
     */
    public String preconditionAlias;

    /**
     * 考评，活动名称
     */
    private String name;

    /**
     * 计算方式; 1: 排名正负激励 2:阶梯奖励(翻倍) 3:总金额X达成率占比 4:固定值X达成率 5: 固定值+超额部分追加奖励
     */
    private EvalRewardCalMethodEnum calMethod;

    /**
     * 排名方式;1:排名序号 2:排名人数百分比
     */
    private RankTypeEnum rankType;

    /**
     * 金额
     */
    private BigDecimal money;

    /**
     * 追加奖励 XX 元/台，超额奖励用
     */
    private BigDecimal extraMoney;

    /**
     * 台阶指标编码
     */
    public String ladderCode;
    /**
     * 指标类型（1.指标，2，考评指标 3，组合指标）
     */
    public IndicatorCodeTypeEnum ladderCodeType;
    /**
     * 提成指标编码
     */
    public String commissionCode;
    /**
     * 指标类型（1.指标，2，考评指标 3，组合指标）
     */
    public IndicatorCodeTypeEnum commissionCodeType;

    /**
     * 逻辑删除
     */
    @TableLogic
    private Boolean yn;

    /**
     * 台阶得分对应的组合指标编码[codeType=3，指标组合必传]
     */
    private String code;

    /**
     * 台阶得分对应的 指标类型（1.指标，2，考评指标 3，指标组合[包含组合指标或考评组合指标]））
     */
    private IndicatorCodeTypeEnum codeType;


}
