package cn.fw.morax.domain.db.eval;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.common.data.mybatis.handler.IntegerListTypeHandler;
import cn.fw.common.data.mybatis.handler.LongListTypeHandler;
import cn.fw.morax.domain.enums.*;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.*;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * 考评指标
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName(autoResultMap = true)
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class EvalIndicator extends BaseAuditableTimeEntity<EvalIndicator, Long> {

    private static final long serialVersionUID = 1L;

    /**
     * 考评指标编码
     */
    private String code;

    /**
     * 考评指标名称
     */
    private String name;

    /**
     * 归属系统id
     */
    private Long sysId;

    /**
     * 归属系统前缀
     */
    private String sysPrefix;

    /**
     * 归属系统名称
     */
    private String sysName;

    /**
     * 导入操作人员角色码
     */
    private String roleCode;

    /**
     * 导入操作人员角色名称
     */
    private String roleName;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum dataType;

    /**
     * 频率类型; 1:按滚动天 2:按星期 3:按月指定日期 4：按滚动月
     */
    private EvalRuleTypeEnum ruleType;

    /**
     * 滚动天数 1:按滚动天 4.按滚动月
     */
    private Integer rollValue;

    /**
     * 频率值 ,号分隔（按星期、按月指定日期使用）
     */
    @TableField(typeHandler = IntegerListTypeHandler.class)
    private List<Integer> ruleValues;

    /**
     * 按滚动月计算顺序; 1:正序 2:倒序
     */
    private RollMonthOrderTypeEnum orderType;

    /**
     * 开始日期
     */
    private LocalDate startTime;

    /**
     * 目标值类型; 1:无 2:百分比 3:金额 4:台数
     */
    private TargetTypeEnum targetType;

    /**
     * 启用/禁用;0:禁用 1:启用
     */
    private Boolean enable;

    /**
     * 数据范围范围类型; 1. 人员, 2.门店
     */
    @TableField(typeHandler = IntegerListTypeHandler.class)
    private List<Integer> dataScopeTypes;

    /**
     * 逻辑删除
     */
    @TableLogic
    private Boolean yn;


}
