package cn.fw.morax.domain.db.kpi;

import cn.fw.common.data.entity.BaseAuditableEntity;
import cn.fw.common.data.mybatis.handler.LongListTypeHandler;
import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.morax.domain.enums.EffectMonthEnum;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.domain.enums.StarEvaluationEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * 绩效组排名配置
 * </p>
 *
 * @author jiangchao
 * @since 2022-09-26
 */
@Data
@TableName(autoResultMap = true)
@EqualsAndHashCode(callSuper = true)
public class KpiGroupRank extends BaseAuditableEntity<KpiGroupRank, Long> {

    /**
     * 岗位id
     */
    @TableField(typeHandler = LongListTypeHandler.class)
    private List<Long> postIds;
    /**
     * 门店id
     */
    @TableField(typeHandler = LongListTypeHandler.class)
    private List<Long> shopIds;

    /**
     * 组合名称
     */
    private String name;

    /**
     * 星级评定方式;1:绩效得分率 2:人员百分比
     */
    private StarEvaluationEnum starEvaluationType;

    /**
     * 撤销D级得分率
     */
    private BigDecimal revokedScoreRatio;

    /**
     * 绩效组编码
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> kgcs;

    /**
     * 绩效组编码
     */
    @TableField(typeHandler = LongListTypeHandler.class)
    private List<Long> kpiGroupIds;

    /**
     * 状态;1:审批中 2:待生效 3:生效中 4:已失效
     */
    private SettingStatusEnum status;

    /**
     * 生效时间
     */
    private LocalDate beginTime;
    /**
     * 失效时间
     */
    private LocalDate endTime;

    /**
     * 调整原因
     */
    @NotBlank(message = "调整原因不能为空")
    @Length(max = 512, message = "调整原因超出最大限制")
    private String reason;
    /**
     * 附件;fid逗号隔开
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> attachment;

    /**
     * 集团id
     */
    private Long groupId;

    /**
     * 逻辑删除
     */
    @TableLogic
    private Boolean yn;

    /**
     * 失效日期
     */
    private LocalDate ineffectiveDate;

}
