package cn.fw.morax.domain.db.salary;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.common.data.mybatis.handler.IntegerListTypeHandler;
import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.morax.domain.enums.ApplicableRoleEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.*;

import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-2
 * @desc : 薪酬项
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@TableName(autoResultMap = true)
public class SalaryProject extends BaseAuditableTimeEntity<SalaryProject, Long> {
    /**
     * 薪酬项编码
     */
    private String salaryCode;
    /**
     * 薪酬项名称
     */
    private String salaryName;

    /**
     * 适用角色类型;1: 全部角色 2:全部管理角色 3:自定义
     */
//    @NotNull(message = "适用角色类型不能为空")
    private ApplicableRoleEnum roleType;

    /**
     * 适用角色编码
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleCodes;


    /**
     * 适用角色名称
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleNames;

    /**
     * 可选计算方式; 1: 星级 2:固定金额 3:阶梯(单) 4:按量 5: 固定提成 6:阶梯(总)
     * 逗号隔开
     */
    @TableField(typeHandler = IntegerListTypeHandler.class)
    private List<Integer> optionalMethod;
    /**
     * 可见的;对用户可见 只有对用户可见的才能配置薪酬项，否则为系统使用的项目
     */
    private Boolean visible;
    /**
     * 启用/禁用;
     * 0:禁用 1:启用
     */
    private Boolean enable;
    /**
     * 逻辑删除
     */
    @TableLogic
    private Boolean yn;
}