package cn.fw.morax.domain.dto;

import cn.fw.common.data.mybatis.handler.IntegerListTypeHandler;
import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.enums.DataScopeTypeEnum;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.vo.CombineIndicatorParamVO;
import com.baomidou.mybatisplus.annotation.TableField;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.List;

/**
 * <p>
 * 组合指标
 * </p>
 *
 * @author jiangchao
 * @since 2023-03-27
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class CombineIndicatorDTO {

    private static final long serialVersionUID = 1L;

    /**
     * id
     */
    private Long id;

    /**
     * 指标编号
     */
    private String combineCode;

    /**
     * 组合指标名称
     */
    @NotBlank(message = "组合指标名称不能为空")
    @Length(max = 255, message = "组合指标名称超出最大长度限制")
    private String name;

    /**
     * 台阶数据类型；1:数量 2:百分比 3:金额
     */
    @NotNull(message = "台阶数据类型不能为空")
    private DataTypeEnum laddersType;

    /**
     * 数据范围范围类型; 1. 人员, 2.门店
     */
    @NotNull(message = "数据范围范围类型不能为空")
    @Size(min = 1, message = "数据范围范围类型不能为空")
    private List<DataScopeTypeEnum> dataScopeTypes;

//    /**
//     * 适用范围类型；1：绩效组 2：薪酬组 3：考评组
//     */
//    @NotNull(message = "适用范围类型不能为空")
//    @Size(min = 1, message = "适用范围类型不能为空")
//    private List<Integer> applyTypes;


//    /**
//     * 组合指标类型（1.组合指标，2，考评组合指标）
//     */
//    @NotNull(message = "组合指标类型不能为空")
//    private IndicatorCodeTypeEnum combineType;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

    /**
     * 启用/禁用;
     * 0:禁用 1:启用
     */
//    @NotNull(message = "启用/禁用不能为空")
    private Boolean enable;

    /**
     * 指标
     */
    @Valid
    @Size(min = 1, message = "指标不能为空")
    @NotNull(message = "指标不能为空")
    private List<CombineIndicatorParamDTO> indicators;


}
