package cn.fw.morax.domain.dto.eval;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.morax.domain.db.eval.EvalGroupIndicator;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorPreconditionLadders;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import cn.fw.morax.domain.vo.eval.EvalGroupIndicatorTargetVO;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * <p>
 * 绩效组指标项
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupIndicatorDTO {

    private static final long serialVersionUID = 1L;

    /**
     * 指标名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String name;

    /**
     * 基础绩效分
     */
    @NotNull(message = "基础绩效分不能为空")
    private Integer baseScore;

    /**
     * 得分方式;1: 阶梯得分 2:普通得分
     */
    @NotNull(message = "得分方式不能为空")
    private ScoreWayEnum scoreWay;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum laddersType;

    /**
     * 台阶参数别名
     */
    public String ladderParamAlias;

    /**
     * 提成参数别名
     */
    public String commissionParamAlias;

    /**
     * 前置条件别名
     */
    public String preconditionAlias;
    /**
     * 台阶得分对应的组合指标编码[codeType=3，指标组合必传]
     */
    @NotNull(message = "编码不能为空")
    @Length(max = 255, message = "台阶得分对应的组合指标编码超出最大长度限制")
    private String code;

    /**
     * 台阶得分对应的 指标类型（1.指标，2，考评指标 3，指标组合[包含组合指标或考评组合指标]））
     */
    @NotNull(message = "编码类型不能为空")
    private IndicatorCodeTypeEnum codeType;

    /**
     * 提成参数
     */
    @Valid
    private List<EvalGroupIndicatorParamDTO> commissionParams;

    /**
     * 台阶参数
     */
    @Valid
    private List<EvalGroupIndicatorParamDTO> ladderParams;

    /**
     * 条件
     */
    @Valid
    private List<EvalGroupIndicatorPreconditionDTO> conds;

    /**
     * 指标阶梯
     */
    @Valid
    private List<EvalGroupIndicatorLaddersDTO> ladders;

    /**
     * 目标
     */
    private List<EvalGroupIndicatorTargetVO> targets;


}
