package cn.fw.morax.domain.dto.eval;

import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.enums.SettingDraftStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.Future;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * 考评组排名配置
 * </p>
 *
 * @author jiangchao
 * @since 2022-09-26
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class EvalGroupRankDTO {

    /**
     * id
     */
    private Long id;

    /**
     * 草稿id
     */
    private Long draftId;

    /**
     * 考评排名组名称
     */
    @NotBlank(message = "考评排名组名称不能为空")
    @Length(max = 255, message = "考评排名组名称超出最大限制")
    private String name;

    /**
     * 多阶段
     */
    @NotNull(message = "多阶段不能为空")
    private Boolean multiStage;

    /**
     * 生效时间
     */
//    @Future(message = "生效时间不能在当前时间之前")
    @NotNull(message = "生效时间不能为空")
    private LocalDate beginTime;

    /**
     * 结束时间
     */
    @NotNull(message = "结束时间不能为空")
    @Future(message = "结束时间不能在当前时间之前")
    private LocalDate overTime;

    /**
     * 考评阶段
     */
    @NotNull(message = "考评阶段不能为空")
    @Size(min = 1, message = "考评阶段不能为空")
    @Valid
    private List<EvalGroupRankStageDTO> stages;

    /**
     * 审批单号
     */
    private String approvalNo;

    /**
     * 草稿状态 1:未发布 2:发布待审批 3:审批拒绝 4:审批同意 5:撤销审批
     */
    private SettingDraftStatusEnum draftStatus;

    /**
     * 调整原因
     */
    @NotBlank(message = "调整原因不能为空")
    @Length(max = 512, message = "调整原因超出最大限制")
    private String reason;

    /**
     * 附件;fid逗号隔开
     */
    private List<String> attachment;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

//    /**
//     * 考评组编码
//     */
//    @NotNull(message = "考评组编码不能为空")
//    @Size(min = 1, message = "考评组编码不能为空")
//    private List<String> egcs;
//
//    /**
//     * 考评范围; 1:门店考评 2:人员考评
//     */
//    @NotNull(message = "考评范围不能为空")
//    private EvalScopeEnum scopeType;

}
