package cn.fw.morax.domain.dto.eval;

import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.RankTypeEnum;
import cn.fw.morax.domain.vo.eval.EvalGroupIndicatorTargetVO;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 考评奖惩
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupRewardDTO {

    private static final long serialVersionUID = 1L;

//    /**
//     * 岗位id
//     */
//    @NotNull(message = "岗位id不能为空")
//    @Size(min = 1, message = "岗位id不能为空")
//    private List<Long> postIds;
//
//    /**
//     * 岗位名称
//     */
//    @NotNull(message = "岗位名称不能为空")
//    @Size(min = 1, message = "岗位名称不能为空")
//    private List<String> postNames;

    /**
     * 考评奖惩指标名称
     */
//    @NotBlank(message = "考评奖惩指标名称不能为空")
//    @Length(max = 255, message = "考评奖惩指标名称超出最大限制")
    private String name;

    /**
     * 计算方式；1: 排名正负激励 2:固定值 3:阶梯 4:总金额占比 5.阶梯翻倍
     */
//    @NotNull(message = "计算方式不能为空")
    private EvalRewardCalMethodEnum calMethod;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum laddersType;

    /**
     * 排名方式;1:排名序号 2:排名人数百分比
     */
    private RankTypeEnum rankType;

    /**
     * 金额
     */
//    @NotNull(message = "金额不能为空")
    private BigDecimal money;

    /**
     * 追加奖励 XX 元/台，超额奖励用
     */
    private BigDecimal extraMoney;

    /**
     * 台阶参数别名
     */
    public String ladderParamAlias;

    /**
     * 提成参数别名
     */
    public String commissionParamAlias;

    /**
     * 前置条件别名
     */
    public String preconditionAlias;

    /**
     * 台阶指标编码
     */
    public String ladderCode;
    /**
     * 指标类型（1.指标，2，考评指标 3，组合指标）
     */
    public IndicatorCodeTypeEnum ladderCodeType;
    /**
     * 提成指标编码
     */
    public String commissionCode;
    /**
     * 指标类型（1.指标，2，考评指标 3，组合指标）
     */
    public IndicatorCodeTypeEnum commissionCodeType;

    /**
     * 提成参数
     */
    @Valid
    private List<EvalGroupRewardParamDTO> commissionParams;

    /**
     * 台阶参数
     */
    @Valid
    private List<EvalGroupRewardParamDTO> ladderParams;

    /**
     * 阶梯
     */
    @Valid
    private List<EvalGroupRewardLaddersDTO> ladders;

    /**
     * 条件
     */
    @Valid
    private List<EvalGroupRewardPreconditionDTO> conds;

    /**
     * 目标
     */
    private List<EvalGroupIndicatorTargetVO> targets;

//    /**
//     * 台阶得分对应的组合指标编码[codeType=3，指标组合必传]
//     */
//    @NotNull(message = "编码不能为空")
//    @Length(max = 255, message = "台阶得分对应的组合指标编码超出最大长度限制")
//    private String code;
//
//    /**
//     * 台阶得分对应的 指标类型（1.指标，2，考评指标 3，指标组合[包含组合指标或考评组合指标]））
//     */
//    @NotNull(message = "编码类型不能为空")
//    private IndicatorCodeTypeEnum codeType;

}
