package cn.fw.morax.domain.dto.kpi;

import cn.fw.common.data.entity.BaseAuditableEntity;
import cn.fw.common.data.mybatis.handler.LongListTypeHandler;
import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.enums.KpiAssessTypeEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.List;

/**
 * <p>
 * 人员质量考核配置
 * </p>
 *
 * @author jiangchao
 * @since 2023-03-17
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class KpiAssessDTO {

    private static final long serialVersionUID = 1L;

    /**
     * id
     */
    private Long id;

    /**
     * 人员质量考核类型; 1:D级员工 2:转正后保护期员工
     */
    @NotNull(message = "人员质量考核类型不能为空")
    private KpiAssessTypeEnum type;

    /**
     * 岗位id
     */
    @NotNull(message = "岗位id不能为空")
    private Long postId;

    /**
     * 岗位名称
     */
    @NotBlank(message = "岗位名称不能为空")
    private String postName;

    /**
     * 适用门店ids
     */
    @NotNull(message = "适用门店id不能为空")
    @Size(min = 1, message = "适用门店id不能为空")
    private List<Long> shopIds;

    /**
     * 适用门店名称
     */
    @NotNull(message = "适用门店名称不能为空")
    @Size(min = 1, message = "适用门店名称不能为空")
    private List<String> shopNames;

    /**
     * 排名
     */
    private Integer rank;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

    /**
     * 考核详情
     */
    @NotNull(message = "考核详情不能为空")
    @Size(min = 1, message = "考核详情不能为空")
    @Valid
    private List<KpiAssessDetailDTO> details;

}
