package cn.fw.morax.domain.dto.kpi;

import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import cn.fw.morax.domain.vo.kpi.KpiGroupIndicatorTargetVO;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 绩效组指标项
 */
@Data
public class KpiGroupIndicatorDTO {

    /**
     * 指标库名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String name;

    /**
     * 指标库名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String paramAlias;

    /**
     * 基础绩效分
     */
    @NotNull(message = "基础绩效分不能为空")
    private Integer baseScore;

    /**
     * 得分方式;1: 阶梯得分 2:普通得分
     */
    @NotNull(message = "得分方式不能为空")
    private ScoreWayEnum scoreWay;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum laddersType;

    /**
     * 台阶参数
     */
    @Valid
    private List<KpiGroupIndicatorParamDTO> ladderParams;

    /**
     * 提成参数（非台阶使用）
     */
//    @NotNull(message = "提成参数不能为空")
//    @Size(min = 1, message = "提成参数不能为空")
    @Valid
    private List<KpiGroupIndicatorParamDTO> commissionParams;

    /**
     * 前置条件配置
     */
    @Valid
    private List<KpiGroupIndicatorPreconditionDTO> conds;


    /**
     * 得分阶梯
     */
    @Valid
    private List<KpiGroupIndicatorLaddersDTO> indicatorLadders;

    /**
     * 指标目标
     */
    private List<KpiGroupIndicatorTargetVO> targets;



 //*    新增字段针对   台阶参数(上面的 ladderParams)-> 有组合指标选择的情况,如果`台阶参数`选择了组合指标,这里要进行一个记录
 //*
 //*   `code` varchar(255) DEFAULT NULL COMMENT '组合指标编码',
 //*   `code_type` tinyint(4) DEFAULT NULL COMMENT '指标类型（1.指标，2，考评指标 3，组合指标）',

    /**
     * 台阶得分对应的组合指标编码[codeType=3，指标组合必传]
     */
    @NotNull(message = "编码不能为空")
    @Length(max = 255, message = "台阶得分对应的组合指标编码超出最大长度限制")
    private String code;

    /**
     * 台阶得分对应的 指标类型（1.指标，2，考评指标 3，指标组合[包含组合指标或考评组合指标]））
     */
    @NotNull(message = "编码类型不能为空")
    private IndicatorCodeTypeEnum codeType;


}