package cn.fw.morax.domain.dto.kpi;

import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.enums.EffectMonthEnum;
import cn.fw.morax.domain.enums.SettingDraftStatusEnum;
import cn.fw.morax.domain.enums.StarEvaluationEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import lombok.*;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 绩效组排名配置
 * </p>
 *
 * @author jiangchao
 * @since 2022-09-26
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class KpiGroupRankDTO {

    /**
     * id
     */
    private Long id;

    /**
     * 草稿id
     */
    private Long draftId;

    /**
     * 组合名称
     */
    @NotBlank(message = "组合名称不能为空")
    @Length(max = 255, message = "组合名称超出最大限制")
    private String name;

    /**
     * 星级评定方式;1:绩效得分率 2:人员百分比
     */
    @NotNull(message = "星级评定方式不能为空")
    private StarEvaluationEnum starEvaluationType;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

    /**
     * 撤销D级得分率
     */
    private BigDecimal revokedScoreRatio;

    /**
     * 绩效组
     */
    @Valid
    @NotNull(message = "绩效组不能为空")
    @Size(min = 1, message = "绩效组不能为空")
    private List<KpiGroupDTO> kpiGroups;

    /**
     * 星级评定阶梯
     */
    @Valid
    @Size(min = 1, message = "星级评定阶梯不能为空")
    @NotNull(message = "星级评定阶梯不能为空")
    private List<KpiGroupRankStarLaddersDTO> starLadders;


    /**
     * 生效时间
     */
    private LocalDate beginTime;

    /**
     * 调整原因
     */
    @NotBlank(message = "调整原因不能为空")
    @Length(max = 512, message = "调整原因超出最大限制")
    private String reason;
    /**
     * 附件;fid逗号隔开
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> attachment;

    @NotNull(message = "生效时间类型不能为空")
    private EffectMonthEnum beginTimeType;

    /**
     * 审批单号
     */
    private String approvalNo;

    /**
     * 草稿状态 1:未发布 2:发布待审批 3:审批拒绝 4:审批同意 5:撤销审批
     */
    private SettingDraftStatusEnum draftStatus;

    /**
     * 岗位
     */
    List<String> postNames;

    /**
     * 门店
     */
    List<String> shopNames;

}
