package cn.fw.morax.domain.dto.query;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.dto.kpi.KpiGroupDTO;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 绩效组配置
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class KpiGroupRepeatQueryDTO {

    /**
     * 岗位id
     */
    private Long postId;

    /**
     * 适用门店ids
     */
    private List<Long> shopIds;

    /**
     * 生效时间
     */
    private LocalDate beginTime;

    /**
     * 状态;
     */
    private SettingStatusEnum status;

    /**
     * 集团id
     */
    private Long groupId;

    /**
     * 排除ids
     */
    private List<Long> excludeIds;

    /**
     * 查询生效中的重复数据
     *
     * @param kpiGroup
     * @return
     */
    public static KpiGroupRepeatQueryDTO convertToEffectiveDTO(KpiGroup kpiGroup){
        KpiGroupRepeatQueryDTO repeatQueryDTO = KpiGroupRepeatQueryDTO.builder()
                .shopIds(kpiGroup.getShopIds())
                .groupId(kpiGroup.getGroupId())
                .postId(kpiGroup.getPostId())
                .status(SettingStatusEnum.EFFECTIVE)
                .build();
        List<Long> excludeIds = new ArrayList(){{
            add(kpiGroup.getId());
        }};
        repeatQueryDTO.setExcludeIds(excludeIds);
        return repeatQueryDTO;
    }

    /**
     * 查询待生效的重复数据
     *
     * @param kpiGroup
     * @return
     */
    public static KpiGroupRepeatQueryDTO convertToBeEffectiveDTO(KpiGroup kpiGroup){
        KpiGroupRepeatQueryDTO repeatQueryDTO = KpiGroupRepeatQueryDTO.builder()
                .shopIds(kpiGroup.getShopIds())
                .groupId(kpiGroup.getGroupId())
                .postId(kpiGroup.getPostId())
                .status(SettingStatusEnum.BE_EFFECTIVE)
                .beginTime(kpiGroup.getBeginTime())
                .build();
        List<Long> excludeIds = new ArrayList(){{
            add(kpiGroup.getId());
        }};
//        if (PublicUtil.isNotEmpty(kpiGroup.getPreId())) {
//            excludeIds.add(kpiGroup.getPreId());
//        }
        repeatQueryDTO.setExcludeIds(excludeIds);
        return repeatQueryDTO;
    }

    /**
     * 构建查询参数
     * @param kpiGroup
     * @param status
     * @return
     */
    public static KpiGroupRepeatQueryDTO convertDto(KpiGroupDTO kpiGroup, SettingStatusEnum status){
        KpiGroupRepeatQueryDTO repeatQueryDTO = KpiGroupRepeatQueryDTO.builder()
                .shopIds(kpiGroup.getShopIds())
                .groupId(kpiGroup.getGroupId())
                .postId(kpiGroup.getPostId())
                .status(status)
                .build();
        List<Long> excludeIds = new ArrayList();
        if (SettingStatusEnum.BE_EFFECTIVE.equals(status)) {
            repeatQueryDTO.setBeginTime(kpiGroup.getBeginTime());
        }
        if (PublicUtil.isNotEmpty(kpiGroup.getId())) {
            excludeIds.add(kpiGroup.getId());
        }
        repeatQueryDTO.setExcludeIds(excludeIds);
        return repeatQueryDTO;
    }


}