package cn.fw.morax.domain.dto.query;

import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 薪酬组配置
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class SalaryGroupRepeatQueryDTO {

    /**
     * 岗位id
     */
    private Long postId;

    /**
     * 适用门店ids
     */
    private List<Long> shopIds;

    /**
     * 生效时间
     */
    private LocalDate beginTime;

    /**
     * 状态;
     */
    private SettingStatusEnum status;

    /**
     * 集团id
     */
    private Long groupId;

    /**
     * 排除ids
     */
    private List<Long> excludeIds;

    /**
     * 查询生效中的重复数据
     *
     * @param salaryGroup
     * @return
     */
    public static SalaryGroupRepeatQueryDTO convertToEffectiveDTO(SalaryGroup salaryGroup){
        SalaryGroupRepeatQueryDTO repeatQueryDTO = SalaryGroupRepeatQueryDTO.builder()
                .shopIds(salaryGroup.getShopIds())
                .groupId(salaryGroup.getGroupId())
                .postId(salaryGroup.getPostId())
                .status(SettingStatusEnum.EFFECTIVE)
                .build();
        List<Long> excludeIds = new ArrayList(){{
            add(salaryGroup.getId());
        }};
        repeatQueryDTO.setExcludeIds(excludeIds);
        return repeatQueryDTO;
    }

    /**
     * 查询待生效的重复数据
     *
     * @param salaryGroup
     * @return
     */
    public static SalaryGroupRepeatQueryDTO convertToBeEffectiveDTO(SalaryGroup salaryGroup){
        SalaryGroupRepeatQueryDTO repeatQueryDTO = SalaryGroupRepeatQueryDTO.builder()
                .shopIds(salaryGroup.getShopIds())
                .groupId(salaryGroup.getGroupId())
                .postId(salaryGroup.getPostId())
                .beginTime(salaryGroup.getBeginTime())
                .status(SettingStatusEnum.BE_EFFECTIVE)
                .build();
        List<Long> excludeIds = new ArrayList(){{
            add(salaryGroup.getId());
        }};
//        if (PublicUtil.isNotEmpty(salaryGroup.getPreId())) {
//            excludeIds.add(salaryGroup.getPreId());
//        }
        repeatQueryDTO.setExcludeIds(excludeIds);
        return repeatQueryDTO;
    }


}