package cn.fw.morax.domain.dto.salary;

import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.TargetCalcTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

/**
 * <p>
 * 薪酬组薪酬项目前置条件
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-05
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class SalaryGroupProjectPreconditionDTO {

    /**
     * 指标编码
     */
    @NotBlank(message = "指标编码不能为空")
    private String indicatorCode;

    /**
     * 指标库名称
     */
    @NotBlank(message = "指标名称不能为空")
    private String indicatorName;

    /**
     * 目标值类型; 1:无 2:百分比 3:金额 4:台数
     */
    @NotNull(message = "绩效目标值类型不能为空")
    private TargetTypeEnum targetType;

    /**
     * 目标计算类型;1: 目标值计算 2:最低要求计算
     */
    private TargetCalcTypeEnum targetCalcType;

    /**
     * 目标值
     */
    private BigDecimal targetValue;

    /**
     * 条件值
     */
    @NotNull(message = "条件值不能为空")
    private BigDecimal condValue;

    /**
     * 排序
     */
    @NotNull(message = "排序值不能为空")
    private Integer sort;


    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    private DataTypeEnum dataType;


}
