package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 指标参数类型
 *
 * @author kurisu
 */
public enum ConditionTypeEnum implements IEnum<Integer> {
    /**
     * 条件值类型; 1:满足目标 2:满足排名率 3:金额
     */
    TARGET(1, "满足目标"),
    RANK(2, "满足排名率"),
    MONEY(3, "金额"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ConditionTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ConditionTypeEnum ofValue(final Integer value) {
        for (final ConditionTypeEnum _enum : ConditionTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ConditionTypeEnum _enum : ConditionTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }
}
