package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 错误类型
 *
 * @author kurisu
 */
public enum ImportErrorReasonEnum implements IEnum<Integer> {

    /**
     * 错误类型: 1:未匹配到人员 2:未匹配到门店 3:未匹配到指标
     */
    NO_MATCH_STAFF(1, "未匹配到人员"),
    NO_MATCH_SHOP(2, "未匹配到门店"),
    NO_MATCH_INDICATOR(3, "未匹配到指标"),
    REPETITION(4, "重复"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ImportErrorReasonEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ImportErrorReasonEnum ofValue(final Integer value) {
        for (final ImportErrorReasonEnum _enum : ImportErrorReasonEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ImportErrorReasonEnum _enum : ImportErrorReasonEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }
}
