package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 指标参数类型
 *
 * @author kurisu
 */
public enum IndicatorCodeTypeEnum implements IEnum<Integer> {
    /**
     * 指标类型（1.指标，2，考评指标 3，组合指标）
     */
    INDICATOR(1, "指标"),
    EVAL_INDICATOR(2, "考评指标"),
    COMBINE_INDICATOR(3, "组合指标"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    IndicatorCodeTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static IndicatorCodeTypeEnum ofValue(final Integer value) {
        for (final IndicatorCodeTypeEnum _enum : IndicatorCodeTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final IndicatorCodeTypeEnum _enum : IndicatorCodeTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }
}
