package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;


/**
 * @author : kurisu
 * @className : StarLevelEnum
 * @description : 绩效星级枚举
 * @date: 2022-04-06 11:01
 */
public enum ReportDimensionEnum implements IEnum<Integer> {

    /**
     * 门店
     */
    SHOP(10, "门店"),
    /**
     * 门店-绩效组
     */
    SHOP_KPI_GROUP(11, "门店-绩效组"),
    /**
     * 门店-岗位
     */
    SHOP_POST(12, "门店-绩效组"),
    /**
     * 门店-人员考评组
     */
    SHOP_EVAL_GROUP(13, "门店-人员考评组"),
    /**
     * 人员
     */
    STAFF(20, "人员"),
    /**
     * 绩效排名组
     */
    KPI_RANK(30, "绩效排名组"),
    /**
     * 绩效排名组-门店
     */
    KPI_RANK_SHOP(31, "绩效排名组-门店"),
    /**
     * 岗位
     */
    POST(40, "岗位"),
    /**
     * 岗位门店
     */
    POST_SHOP(41, "岗位门店"),
    /**
     * 管理层
     */
    MANAGER(50, "管理层"),
    /**
     * 管理层-门店
     */
    MANAGER_SHOP(51, "管理层-门店"),
    /**
     * 管理层-岗位
     */
    MANAGER_POST(52, "管理层-岗位"),
    /**
     * 管理层-绩效组
     */
    MANAGER_KPI_GROUP(53, "管理层-绩效组"),
    /**
     * 考评组
     */
    EVAL_GROUP(60, "考评组"),
    /**
     * 考评组-门店
     */
    EVAL_GROUP_SHOP(61, "考评组-门店"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ReportDimensionEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ReportDimensionEnum ofValue(final Integer value) {
        for (final ReportDimensionEnum _enum : ReportDimensionEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static Integer getNameByVale(final Integer value) {
        for (final ReportDimensionEnum _enum : ReportDimensionEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getValue();
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
