package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 离职校验的处理状态
 *
 * @author : kurisu
 * @version : 2.0
 * @className : ResignExamineStatusEnum
 * @description : 离职校验的处理状态
 * @date : 2022-12-29 16:52
 */
public enum ResignExamineStatusEnum implements IEnum<Integer> {
    /**
     * 离职校验的处理状态；1:待处理 2:已处理 3:无需处理
     */
    WAITING(1, "待处理"),
    COMPLETE(2, "已处理"),
    IGNORE(2, "无需处理"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ResignExamineStatusEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ResignExamineStatusEnum ofValue(final Integer value) {
        for (final ResignExamineStatusEnum _enum : ResignExamineStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ResignExamineStatusEnum _enum : ResignExamineStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
