package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : SalaryCalMethodEnum
 * @description : 薪酬计算方式
 * @date: 2022-04-06 11:44
 */
public enum SalaryCalMethodEnum implements IEnum<Integer> {
    /**
     * 计算方式;1: 星级 2:固定金额 3:阶梯提成 4:车系提成 99:动态
     */
    STAR(1, "员工星级"),
    FIXATION(2, "固定金额"),
    LADDER(3, "台阶提成"),
    CAR_SERIES(4, "车系提成"),
    DYNAMIC(99, "动态计算"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    SalaryCalMethodEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static SalaryCalMethodEnum ofValue(final Integer value) {
        for (final SalaryCalMethodEnum _enum : SalaryCalMethodEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final SalaryCalMethodEnum _enum : SalaryCalMethodEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
