package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 草稿类型
 *
 * @author kurisu
 */
public enum SettingDraftStatusEnum implements IEnum<Integer> {
    /**
     * 草稿状态 1:未发布 2:发布待审批 3:审批拒绝 4:审批同意 5:撤销审批
     */
    NO_RELEASE(1, "未发布"),
    RELEASE_APPROVAL(2, "发布待审批"),
    RELEASE_APPROVAL_REJECT(3, "审批拒绝"),
    RELEASE_APPROVAL_AGREE(4, "审批同意"),
    RELEASE_APPROVAL_CANCEL(5, "撤销审批"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    SettingDraftStatusEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static SettingDraftStatusEnum ofValue(final Integer value) {
        for (final SettingDraftStatusEnum _enum : SettingDraftStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final SettingDraftStatusEnum _enum : SettingDraftStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }
}
