package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 目标类型
 *
 * @author kurisu
 */
public enum TargetTypeEnum implements IEnum<Integer> {

    /**
     * 目标类型: 1:无 2:百分比 3:金额
     */
    /**
     * 无
     */
    NO(1, "无"),
    /**
     * 百分比
     */
    RATIO(2, "百分比"),
    /**
     * 金额
     */
    MONEY(3, "金额"),

    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    TargetTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static TargetTypeEnum ofValue(final Integer value) {
        for (final TargetTypeEnum _enum : TargetTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final TargetTypeEnum _enum : TargetTypeEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }
}
