package cn.fw.morax.domain.vo.eval;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.enums.*;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * <p>
 * 考评奖惩前置条件
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupRewardPreconditionVO{

    /**
     * id
     */
    private Long id;
    /**
     * 考评组id
     */
    private Long evalGroupId;

    /**
     * 考评奖惩id
     */
    private Long evalGroupRewardId;

    /**
     * 条件值类型; 1:满足目标 2:满足排名率 3:金额
     */
    private ConditionTypeEnum condType;

    /**
     * 编码类型（1. 指标，2.考评指标）
     */
    private IndicatorCodeTypeEnum codeType;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    private DataTypeEnum dataType;

    /**
     * 条件值
     */
    private BigDecimal rankCondValue;

    /**
     * 排名
     */
    private Integer rank;

    /**
     * 项目编码
     */
    private String code;

    /**
     * 项目名称
     */
    private String name;

    /**
     * 目标计算类型;1: 目标值计算 2:最低要求计算
     */
    private TargetCalcTypeEnum targetCalcType;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额 4:台数
     */
    private TargetTypeEnum targetType;

    /**
     * 目标值
     */
    private BigDecimal targetValue;

    /**
     * 条件值
     */
    private BigDecimal condValue;

    /**
     * 原始值
     */
    private BigDecimal indicatorValue;

    /**
     * 达成目标
     */
    private BigDecimal reachValue;

    /**
     * 排序
     */
    private Integer sort;

    /**
     * 命中
     */
    private Boolean hit;

    /**
     * 转换为百分数展示
     */
    public void convertToPercent(){
        if (TargetTypeEnum.RATIO.equals(this.getTargetType())) {
            this.setTargetValue(this.getTargetValue().multiply(Constant.ONE_HUNDRED));
        }
        if (ConditionTypeEnum.RANK.equals(this.getCondType())) {
            this.setCondValue(this.getCondValue().multiply(Constant.ONE_HUNDRED));
        } else {
            if (PublicUtil.isNotEmpty(this.getDataType()) && DataTypeEnum.RATIO.equals(this.getDataType())) {
                this.setCondValue(this.getCondValue().multiply(Constant.ONE_HUNDRED));
            }
        }
    }

    /**
     * 转换为百分数展示
     */
    public void convertValueToPercent(){
        //无目标 根据数据类型转换
        if (TargetTypeEnum.NO.equals(this.getTargetType())) {
            if (PublicUtil.isNotEmpty(this.getDataType()) && DataTypeEnum.RATIO.equals(this.getDataType())) {
                this.setCondValue(this.getCondValue().multiply(Constant.ONE_HUNDRED));
            }
        } else {
            //有目标
            this.setCondValue(this.getCondValue().multiply(Constant.ONE_HUNDRED));
        }
        if (TargetTypeEnum.RATIO.equals(this.getTargetType())) {
            this.setTargetValue(this.getTargetValue().multiply(Constant.ONE_HUNDRED));
        }
        if (DataTypeEnum.RATIO.equals(this.getDataType())) {
            this.setIndicatorValue(this.getIndicatorValue().multiply(Constant.ONE_HUNDRED));
        }
        //有目标
        if (! TargetTypeEnum.NO.equals(this.getTargetType())) {
            this.setReachValue(this.getReachValue().multiply(Constant.ONE_HUNDRED));
        }
    }

    /**
     * 检查是否命中
     */
    public void checkHit() {
        if (TargetTypeEnum.NO.equals(this.getTargetType())) {
            this.setHit(this.getIndicatorValue().compareTo(this.getCondValue()) >= 0);
            return;
        }
        this.setHit(this.getReachValue().compareTo(this.getCondValue()) >= 0);
    }

}
