package cn.fw.morax.domain.vo.eval;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.morax.domain.dto.eval.EvalGroupRewardLaddersDTO;
import cn.fw.morax.domain.dto.eval.EvalGroupRewardParamDTO;
import cn.fw.morax.domain.dto.eval.EvalGroupRewardPreconditionDTO;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.RankTypeEnum;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 考评奖惩
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupRewardVO {

    /**
     * id
     */
    private Long id;
    /**
     * 考评组配置id
     */
    private Long evalGroupId;

    /**
     * 考评，活动名称
     */
    private String name;

    /**
     * 计算方式; 1: 排名正负激励 2:阶梯奖励(翻倍) 3:总金额X达成率占比 4:固定值X达成率 5: 固定值+超额部分追加奖励
     */
    private EvalRewardCalMethodEnum calMethod;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum laddersType;

    /**
     * 排名方式;1:排名序号 2:排名人数百分比
     */
    private RankTypeEnum rankType;

    /**
     * 金额
     */
    private BigDecimal money;

    /**
     * 追加奖励 XX 元/台，超额奖励用
     */
    private BigDecimal extraMoney;

    /**
     * 台阶参数别名
     */
    public String ladderParamAlias;

    /**
     * 提成参数别名
     */
    public String commissionParamAlias;

    /**
     * 前置条件别名
     */
    public String preconditionAlias;

    /**
     * 提成参数
     */
    @Valid
    private List<EvalGroupRewardParamVO> commissionParams;

    /**
     * 台阶参数
     */
    @Valid
    private List<EvalGroupRewardParamVO> ladderParams;

    /**
     * 阶梯
     */
    @Valid
    private List<EvalGroupRewardLaddersVO> ladders;

    /**
     * 条件
     */
    @Valid
    private List<EvalGroupRewardPreconditionVO> conds;

    /**
     * 目标
     */
    private List<EvalGroupIndicatorTargetVO> targets;

    /**
     * 台阶得分对应的组合指标编码[codeType=3，指标组合必传]
     */
    private String code;

    /**
     * 台阶得分对应的 指标类型（1.指标，2，考评指标 3，指标组合[包含组合指标或考评组合指标]））
     */
    private IndicatorCodeTypeEnum codeType;
}
