package cn.fw.morax.domain.vo.kpi;

import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.morax.domain.enums.ApplicableRoleEnum;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import lombok.Data;

import java.util.Date;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-2
 * @desc : 指标 indicators  和 考评指标 eval_indicator 组合数据
 */
@Data
public class IndicatorsAndEvalIndicatorsVO {


    //  `code_type` tinyint(4) NOT NULL COMMENT '指标类型（1.指标，2，考评指标）',
    private IndicatorCodeTypeEnum codeType;

    //两个表共有字段
    /**
     * 指标id
     */
    private Long id;
    /**
     * 指标编号 [统一返回code命名]
     */
    private String code;
    /**
     * 指标名称 [统一返回name命名]
     */
    private String name;

    /**
     * 归属系统id
     */
    private Long sysId;

    /**
     * 归属系统名称
     */
    private String sysName;

    /**
     * 归属系统前缀
     */
    private String sysPrefix;

    /**
     * 启用/禁用;
     * 0:禁用 1:启用
     */
    private Boolean enable;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额
     */
    private TargetTypeEnum targetType;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    private DataTypeEnum dataType;

    /**
     * 数据范围类型; 1. 人员, 2.门店
     */
    private List<Integer> dataScopeTypes;





    //indicators表特有
    /**
     * 适用角色类型;1: 全部角色 2:全部管理角色 3:自定义
     */
    private ApplicableRoleEnum roleType;
    /**
     * 适用角色编码
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleCodes;
    /**
     * 适用角色名称
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> roleNames;

    /**
     * 适用范围类型; 1. 绩效组, 2.薪酬组, 3. 考评组
     */
    private List<Integer> applyTypes;

    /**
     * 内部指标
     * 0:否 1:是
     */
    private Boolean innerIndicator;

    /**
     * 报表路径
     */
    private String reportPath;





    //eval_indicator表特有
    /**
     * 导入操作人员角色码
     */
    private String roleCode;
    /**
     * 导入操作人员角色名称
     */
    private String roleName;
    /**
     * 频率类型; 1:按滚动天 2:按星期 3:按月指定日期 4：按滚动月
     */
    private Integer ruleType;
    /**
     * 滚动天
     */
    private Integer rollValue;
    /**
     * 频率值 ,号分隔（按星期、按月指定日期使用）
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<Integer> ruleValues;

    /**
     * 按滚动月计算顺序; 1:正序 2:倒序
     */
    private Integer orderType;

    /**
     * 开始日期
     */
    private Date startTime;

}