package cn.fw.morax.domain.vo.kpi;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.math.BigDecimal;

/**
 * <p>
 * 绩效组指标条件得分阶梯
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-05
 */
@Data
public class KpiGroupIndicatorPreconditionLaddersVO implements Comparable<KpiGroupIndicatorCondLaddersVO>{

    /**
     * id
     */
    private Long id;

    /**
     * 绩效组指标条件id
     */
    private Long preconditionId;

    /**
     * 阶梯下限;大于等于
     */
    private BigDecimal lower;

    /**
     * 阶梯上限;小于; 无上限的情况值为 2<<15
     */
    private BigDecimal upper;

    /**
     * 得分百分比
     */
    private BigDecimal scorePercent;

    /**
     * 是否命中
     */
    private Boolean hit;

    /**
     * 正序排列
     *
     * @param other
     * @return
     */
    @Override
    public int compareTo(KpiGroupIndicatorCondLaddersVO other) {
        //优先通过下限比较，上限与下限
        if (PublicUtil.isEmpty(this.upper) || PublicUtil.isEmpty(other.getUpper())) {
            return this.lower.compareTo(other.getLower());
        } else {
            return this.upper.compareTo(other.getUpper());
        }
    }

    /**
     * 转换为百分数展示
     */
    public void convertLaddersToPercent(TargetTypeEnum targetType, DataTypeEnum dataType){
        this.setScorePercent(this.getScorePercent().multiply(Constant.ONE_HUNDRED));
        //有目标一定是 百分比，无目标通过 dataType判断
        if (TargetTypeEnum.RATIO.equals(targetType) || TargetTypeEnum.MONEY.equals(targetType)) {
            this.setUpper(this.getUpper().multiply(Constant.ONE_HUNDRED));
            this.setLower(this.getLower().multiply(Constant.ONE_HUNDRED));
            return;
        }
        //无目标
        if (PublicUtil.isNotEmpty(dataType) && DataTypeEnum.RATIO.equals(dataType)) {
            this.setUpper(this.getUpper().multiply(Constant.ONE_HUNDRED));
            this.setLower(this.getLower().multiply(Constant.ONE_HUNDRED));
        }
    }

}
