package cn.fw.morax.domain.vo.kpi;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorPrecondition;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorTargetHitLog;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.TargetCalcTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 绩效组指标条件
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-05
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class KpiGroupIndicatorPreconditionVO {

    /**
     * id
     */
    private Long id;

    /**
     * 绩效组配置id
     */
    private Long kpiGroupId;

    /**
     * 绩效组指标id
     */
    private Long kpiGroupIndicatorId;

    /**
     * 指标库id
     */
    private Long indicatorId;

    /**
     * 指标编码
     */
    private String indicatorCode;

    /**
     * 指标库名称
     */
    private String indicatorName;

    /**
     * 目标计算类型;1: 目标值计算 2:最低要求计算
     */
    private TargetCalcTypeEnum targetCalcType;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额 4:台数
     */
    private TargetTypeEnum targetType;

    /**
     * 目标值
     */
    private BigDecimal targetValue;

    /**
     * 排序
     */
    private Integer sort;

    /**
     * 命中阶梯
     */

    private Long hitLaddersId;

//    /**
//     * 条件值
//     */
//    private BigDecimal condValue;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    private DataTypeEnum dataType;

    /**
     * 指标值
     */
    private BigDecimal indicatorValue;

    /**
     * 达成值
     */
    private BigDecimal reachValue;

    /**
     * 前置条件台阶
     */
    private List<KpiGroupIndicatorPreconditionLaddersVO> condLadders;


    /**
     * 处理百分数
     */
    public void processPercent() {
        if (TargetTypeEnum.RATIO.equals(this.getTargetType())) {
            this.setTargetValue(this.getTargetValue().multiply(Constant.ONE_HUNDRED));
        }
    }

    /**
     * 处理指标百分数
     */
    public void processTargetPercent(KpiGroupIndicatorTargetHitLog kpiGroupIndicatorTargetHitLog) {
        if (PublicUtil.isEmpty(kpiGroupIndicatorTargetHitLog)) {
            this.setIndicatorValue(BigDecimal.ZERO);
            this.setReachValue(BigDecimal.ZERO);
            return;
        }
        if (DataTypeEnum.RATIO.equals(this.getDataType())) {
            this.setIndicatorValue(kpiGroupIndicatorTargetHitLog.getValue().multiply(Constant.ONE_HUNDRED));
            this.setReachValue(kpiGroupIndicatorTargetHitLog.getReachValue().multiply(Constant.ONE_HUNDRED));
        } else {
            this.setIndicatorValue(kpiGroupIndicatorTargetHitLog.getValue());
            this.setReachValue(kpiGroupIndicatorTargetHitLog.getReachValue());
        }
    }

    /**
     * 处理指标百分数
     */
    public void processNoTargetPercent() {
        if (PublicUtil.isEmpty(this.getIndicatorValue())) {
            return;
        }
        if (DataTypeEnum.RATIO.equals(this.getDataType())) {
            this.setIndicatorValue(this.getIndicatorValue().multiply(Constant.ONE_HUNDRED));
        } else {
            this.setIndicatorValue(this.getIndicatorValue());
        }
    }

}
