package cn.fw.morax.domain.vo.kpi;

import cn.fw.common.data.mybatis.handler.LongListTypeHandler;
import cn.fw.common.data.mybatis.handler.StringListTypeHandler;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.domain.enums.StarEvaluationEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.Set;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 绩效组配置
 */
@Data
public class KpiGroupRankVO {
    /**
     * 主键id
     */
    private Long id;

    /**
     * 组合名称
     */
    private String name;

    /**
     * 岗位id
     */
    private List<Long> postIds;
    /**
     * 岗位名称
     */
    private Set<String> postNames;
    /**
     * 适用门店ids
     */
    private List<Long> shopIds;
    /**
     * 适用门店名称
     */
    private Set<String> shopNames;
    /**
     * 绩效组编码
     */
    private List<String> kgcs;

    /**
     * 绩效组名称
     */
    private List<String> kpiGroupNames;

    /**
     * 星级评定方式;1:绩效得分率 2:人员百分比
     */
    private StarEvaluationEnum starEvaluationType;

    /**
     * 撤销D级得分率
     */
    private BigDecimal revokedScoreRatio;

    /**
     * 集团id
     */
    private Long groupId;

    /**
     * 绩效组
     */
    private List<KpiGroupVO> kpiGroups;

    /**
     * 绩效组星级排名
     */
    private List<KpiGroupRankStarLaddersVO> starLadders;

    /**
     * 状态;1:审批中 2:待生效 3:生效中 4:已失效
     */
    private SettingStatusEnum status;

    /**
     * 生效时间
     */
    private LocalDate beginTime;
    /**
     * 失效时间
     */
    private LocalDate endTime;

    /**
     * 绩效组编码
     */
    private List<Long> kpiGroupIds;

    /**
     * 调整原因
     */
    @NotBlank(message = "调整原因不能为空")
    @Length(max = 512, message = "调整原因超出最大限制")
    private String reason;
    /**
     * 附件;fid逗号隔开
     */
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> attachment;

}