package cn.fw.morax.domain.vo.salary;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.TargetCalcTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.math.BigDecimal;
import java.util.Optional;

/**
 * <p>
 * 薪酬组薪酬项目前置条件
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-05
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class SalaryGroupProjectPreconditionVO {

    private static final long serialVersionUID = 1L;

    /**
     * id
     */
    private Long id;

    /**
     * 薪酬组id
     */
    private Long salaryGroupId;

    /**
     * 薪酬组项目id
     */
    private Long salaryGroupProjectId;

    /**
     * 指标编码
     */
    private String indicatorCode;

    /**
     * 指标库名称
     */
    private String indicatorName;


    /**
     * 目标值类型; 1:无 2:百分比 3:金额
     */
    private TargetTypeEnum targetType;

    /**
     * 目标计算类型;1: 目标值计算 2:最低要求计算
     */
    private TargetCalcTypeEnum targetCalcType;

    /**
     * 目标值
     */
    private BigDecimal targetValue;

    /**
     * 数据类型；1:数量 2:百分比 3:金额
     */
    private DataTypeEnum dataType;

    /**
     * 条件值
     */
    private BigDecimal condValue;

    /**
     * 指标值
     */
    private BigDecimal indicatorValue;

    /**
     * 达成值
     */
    private BigDecimal reachValue;

    /**
     * 排序
     */
    private Integer sort;

    /**
     * 是否命中
     */
    private Boolean hit;

    /**
     * 处理百分数
     */
    public void processPercent() {
        if (TargetTypeEnum.RATIO.equals(this.getTargetType())) {
            this.setTargetValue(this.getTargetValue().multiply(Constant.ONE_HUNDRED));
        }
        if (PublicUtil.isNotEmpty(dataType) && DataTypeEnum.RATIO.equals(dataType)) {
            this.setIndicatorValue(this.getIndicatorValue().multiply(Constant.ONE_HUNDRED));
            this.setCondValue(this.getCondValue().multiply(Constant.ONE_HUNDRED));
        }
    }

    /**
     * 处理上报数据百分数
     */
    public void processReportPercent() {
        if (TargetTypeEnum.RATIO.equals(this.getTargetType())) {
            this.setTargetValue(this.getTargetValue().multiply(Constant.ONE_HUNDRED));
        }
        if (PublicUtil.isNotEmpty(dataType) && DataTypeEnum.RATIO.equals(dataType)) {
            this.setIndicatorValue(this.getIndicatorValue().multiply(Constant.ONE_HUNDRED));
            this.setCondValue(this.getCondValue().multiply(Constant.ONE_HUNDRED));
        }
    }


    /**
     * 检查是否命中
     */
    public void checkHit() {
        if (TargetTypeEnum.NO.equals(this.getTargetType())) {
            this.setHit(Optional.ofNullable(this.getIndicatorValue()).orElse(BigDecimal.ZERO).compareTo(this.getCondValue()) >= 0);
            return;
        }
        this.setHit(Optional.ofNullable(this.getReachValue()).orElse(BigDecimal.ZERO).compareTo(this.getCondValue()) >= 0);
    }

}
