package cn.fw.morax.rpc.backlog;

import cn.fw.backlog.sdk.api.BacklogItemApi;
import cn.fw.backlog.sdk.api.param.*;
import cn.fw.backlog.sdk.api.result.FailBacklogItem;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.morax.rpc.backlog.dto.BacklogBatchPlanItemReqDTO;
import cn.fw.morax.rpc.backlog.dto.BacklogBatchPlanReqDTO;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : TodoRpcService
 * @description : erp待办rpc
 * @date: 2020-08-18 15:53
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class TodoRpcService {
    private final BacklogItemApi backlogItemApi;

    /**
     * 生成待办
     *
     * @param dto
     */
    public boolean push(BackLogItemDTO dto) {
        final String func = "调用ERP系统生成待办";
        try {
            log.info("{}：params: [{}]", func, dto);
            BacklogItemPlanReq req = new BacklogItemPlanReq();
            BeanUtils.copyProperties(dto, req);
            req.setPlanTime(dto.getDataTime());
            Message<Void> message = backlogItemApi.planBacklogItem(req);
            log.info("{} 结果：{} desc: {}", func, message.isSuccess(), message.getResult());
            return message.isSuccess();
        } catch (Exception e) {
            log.error("{}失败，params：{}", func, dto, e);
            return false;
        }
    }

    /**
     * 生成待办
     *
     * @param dto
     */
    public List<FailBacklogItem> batchPush(BacklogBatchPlanReqDTO dto) {
        final String func = "调用ERP系统生成批量待办";
        try {
            log.info("{}：params: [todoCode:{}, items:{}]", func, dto.getItemCode(), dto.getItemList());
            List<BacklogBatchPlanItemReq> itemList = PublicUtil.copyList(dto.getItemList(), BacklogBatchPlanItemReq.class);
            BacklogBatchPlanReq req = new BacklogBatchPlanReq(dto.getItemCode(), itemList);
            Message<List<FailBacklogItem>> message = backlogItemApi.batchPlanBacklogItem(req);
            log.info("{} backlogItemApi.batchPlanBacklogItem 传入参数:{} 返回结果：{}", func, JSON.toJSONString(req), JSON.toJSONString(message));
            BV.isTrue(message.isSuccess(), message.getResult());
            return message.getData();
        } catch (Exception e) {
            log.error("{}失败，params：{}", func, dto, e);
            return new ArrayList<>();
        }
    }

    /**
     * 完成待办
     *
     * @param dto
     */
    public boolean complete(BackLogItemDTO dto) {
        final String func = "调用ERP系统完成待办";
        try {
            log.info("{}：params: [{}]", func, dto);
            BacklogItemCommitReq req = new BacklogItemCommitReq();
            BeanUtils.copyProperties(dto, req);
            req.setCommitTime(dto.getDataTime());
            Message<Void> message = backlogItemApi.commitBacklogItem(req);
            log.info("{} 结果：{} desc: {}", func, message.isSuccess(), message.getResult());

            return message.isSuccess();
        } catch (Exception e) {
            log.error("{}失败，params：{}", func, dto, e);
            return false;
        }
    }

    /**
     * 取消待办
     *
     * @param dto
     */
    public boolean cancel(BackLogItemDTO dto) {
        final String func = "调用ERP系统取消待办";
        try {
            log.info("{}：params: [{}]", func, dto);

            BacklogItemCancelReq req = new BacklogItemCancelReq();
            BeanUtils.copyProperties(dto, req);
            Message<Void> message = backlogItemApi.cancelBacklogItem(req);
            log.info("{} 结果：{} desc: {}", func, message.isSuccess(), message.getResult());

            return message.isSuccess();
        } catch (Exception e) {
            log.error("{}失败，params：{}", func, dto, e);
            return false;
        }
    }
}
