package cn.fw.morax.rpc.finance;

import cn.fw.data.base.domain.common.Message;
import cn.fw.finance.sdk.FinanceApi;
import cn.fw.finance.sdk.enums.SubjectTypeEnum;
import cn.fw.finance.sdk.param.RpParam;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.rpc.finance.dto.FinanceParam;
import cn.fw.morax.rpc.finance.dto.FundParam;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.util.Collections;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : FinanceRpcService
 * @description : 财务rpc服务
 * @date : 2022-05-06 17:44
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class FinanceRpcService {
    private final FinanceApi financeApi;

    /**
     * 发放工资/奖励等
     *
     * @param financeParam
     * @return
     */
    @Nullable
    public String paySomething(@NonNull FinanceParam financeParam) {
        RpParam rpParam = transferRpParam(financeParam);
        try {
            Message<Void> msg = financeApi.addPay(rpParam);
            log.info("FinanceApi.addPay: msg.code={}, msg.result={}", msg.getCode(), msg.getResult());
            if (msg.isSuccess()) {
                return financeParam.getToken();
            }
        } catch (Exception e) {
            log.error("发放工资/奖励失败", e);
        }
        return null;
    }

    /**
     * 扣款
     *
     * @param financeParam
     * @return
     */
    @Nullable
    public String deductSomething(@NonNull FinanceParam financeParam) {
        RpParam rpParam = transferRpParam(financeParam);
        try {
            Message<Void> msg = financeApi.addRcv(rpParam);
            log.info("FinanceApi.addRcv: msg.code={}, msg.result={}", msg.getCode(), msg.getResult());
            if (msg.isSuccess()) {
                return financeParam.getToken();
            }
        } catch (Exception e) {
            log.error("发起扣款失败", e);
        }
        return null;
    }


    private RpParam transferRpParam(FinanceParam financeParam) {
        RpParam rp = new RpParam();
        rp.setBusinessType(financeParam.getBusinessType());
        rp.setBusinessNo(financeParam.getBusinessNo());
        rp.setShopId(financeParam.getShopId());
        rp.setDealerId(financeParam.getDealerId());
        rp.setGroupId(financeParam.getGroupId());
        FundParam fundParam = financeParam.getFundParam();
        RpParam.Item item = new RpParam.Item();
        item.setAmount(fundParam.getAmount());
        item.setSubjectType(SubjectTypeEnum.EMPLOYEE);
        item.setSubjectId(fundParam.getUserId());
        item.setSubjectName(fundParam.getUserName());
        if (PublicUtil.isNotEmpty(fundParam.getMobile())) {
            item.setSubjectPhone(fundParam.getMobile());
        }
        item.setType(fundParam.getType());
        item.setDate(DateUtil.localDate2Date(fundParam.getDate()));
        item.setCorporate(fundParam.isCorporate());
        rp.setItems(Collections.singletonList(item));
        financeParam.setToken(item.getToken());
        return rp;
    }
}
