package cn.fw.morax.rpc.office.dto;

import cn.fw.office.sdk.dto.AttachDto;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * 处罚员工
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder(toBuilder = true)
public class PunishParam {

    /**
     * 处罚类型key
     */
    @NotNull(message = "处罚类型key不能为空")
    private String key;

    /**
     * 集团id
     */
    @NotNull(message = "集团id不能为空")
    private Long groupId;

    /**
     * 执行人id
     */
//    @NotNull(message = "员工id不能为空")
    private Long executorId;

    /**
     * 员工id
     */
    @NotNull(message = "员工id不能为空")
    private Long staffId;

    /**
     * 门店id
     */
    @NotNull(message = "门店id不能为空")
    private Long shopId;

    /**
     * 门店名
     */
    @NotBlank(message = "门店名不能为空")
    @Length(max = 32, message = "门店名超出长度最大限制")
    private String shopName;

    /**
     * 处罚绩效
     */
    @NotNull(message = "绩效不能为空")
    @Min(value = 0, message = "绩效不能小于0")
    private BigDecimal performance;

    /**
     * 罚款金额（元）
     */
    @NotNull(message = "罚款不能为空")
    @Min(value = 0, message = "罚款不能小于0元")
    private BigDecimal fine;

    /**
     * 处罚原因
     */
    @NotNull(message = "处罚原因不能为空")
    @Length(max = 512, message = "原因超出长度最大限制")
    private String reason;

    /**
     * 是否展示报销配置
     * <span>true：申诉将会选择是否允许报销 false：不展示</span>
     */
    private Boolean showReimburse;

    /**
     * 业务单号
     */
    @Length(max = 32,message = "业务单号超出长度")
    @NotNull(message = "业务单号不能为空")
    private String businessToken;

    /**
     * 处罚附件
     */
    @Valid
    private List<AttachParam> attaches;
}
