package cn.fw.morax.sdk.api;

import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.sdk.dto.kpi.KpiGroupUserQuery;
import cn.fw.morax.sdk.dto.kpi.KpiGroupUserResult;
import cn.fw.morax.sdk.dto.kpi.KpiPointRewardReq;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @author : kurisu
 * @version : 2.0
 * @className : KpiReportApi
 * @description : 绩效处理接口
 * @date: 2022-04-07 15:35
 */
@FeignClient(value = "fw-morax", path = "/api/morax/kpi")
public interface IKpiReportService {
    /**
     * 查询绩效组人员
     *
     * @param kpiGroupUserQuery 查询参数
     * @return 分组人员
     * @Deprecated 已过时 此方法不在维护，将在2.1.0移除
     */
    @Deprecated
    @GetMapping("/query/user")
    Message<List<KpiGroupUserResult>> queryKpiGroupUser(@Validated({KpiGroupUserQuery.Query.class}) @SpringQueryMap KpiGroupUserQuery kpiGroupUserQuery);

    /**
     * 实时查询绩效组人员
     *
     * @param kpiGroupUserQuery 查询参数
     * @return 分组人员
     * @Deprecated 已过时 此方法不在维护，将在2.1.0移除
     */
    @Deprecated
    @GetMapping("/real-time/query/user")
    Message<List<KpiGroupUserResult>> realTimeQueryKpiGroupUser(@Validated({KpiGroupUserQuery.RealTimeQuery.class}) @SpringQueryMap KpiGroupUserQuery kpiGroupUserQuery);

    /**
     * 绩效分奖惩上报
     *
     * @param kpiPointRewardReq 绩效分奖惩上报数据
     * @return void
     */
    @PostMapping("/point/reward")
    Message<Void> kpiPointReward(@Valid @RequestBody KpiPointRewardReq kpiPointRewardReq);
}
