package cn.fw.morax.sdk.api;

import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.sdk.dto.CustomList;
import cn.fw.morax.sdk.dto.salary.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @author : kurisu
 * @className : ISalaryReportService
 * @description : 薪酬上报处理类
 * @date : 2022-04-07 15:41
 */
@FeignClient(value = "fw-morax", path = "/api/morax/salary")
public interface ISalaryReportService {
    /**
     * 奖惩薪资
     *
     * @param salaryRewardReq 薪资项
     */
    @PostMapping("/bonus-penalty/report")
    Message<Void> salaryRewardReport(@Valid @RequestBody SalaryRewardReq salaryRewardReq);

    /**
     * 社保扣款项上报
     *
     * @param generalSalaryProjectReq 社保扣款
     */
    @PostMapping("/social-security/report")
    Message<Void> socialSecurityProject(@Valid @RequestBody GeneralSalaryProjectReq generalSalaryProjectReq);

    /**
     * 福利补贴项
     *
     * @param generalSalaryProjectReq 福利补贴项
     * @return
     */
    @PostMapping("/subsidy/report")
    Message<Void> subsidyProject(@Valid @RequestBody GeneralSalaryProjectReq generalSalaryProjectReq);

    /**
     * 福利补贴项上报
     *
     * @param generalSalaryProjectReqs 福利补贴项
     */
    @PostMapping("/subsidy/reports")
    Message<Void> subsidyProjects(@Valid @RequestBody CustomList<GeneralSalaryProjectReq> generalSalaryProjectReqs);

    /**
     * 查询薪酬组人员
     *
     * @param salaryGroupUserQuery
     * @return 分组人员
     */
    @GetMapping("/query/user")
    Message<List<SalaryGroupUserResult>> querySalaryGroupUser(@Validated({SalaryGroupUserQuery.Query.class}) @SpringQueryMap SalaryGroupUserQuery salaryGroupUserQuery);

    /**
     * 查询集团所有薪酬组人员
     *
     * @param salaryGroupUserQuery
     * @return 集团所有薪酬组人员
     */
    @GetMapping("/query/group-user")
    Message<SalaryGroupUserResult> queryGroupSalaryUser(@Validated({SalaryGroupUserQuery.GroupQuery.class}) @SpringQueryMap SalaryGroupUserQuery salaryGroupUserQuery);

    /**
     * 集团薪酬配置
     *
     * @param groupIds 集团id
     * @return void
     */
    @PostMapping("/groups-settin")
    Message<List<SalaryGeneralSettinResult>> queryGroupSettins(@RequestBody CustomList<Long> groupIds);

    /**
     * 薪酬关单
     *
     * @param salaryClosureReq 薪酬关单
     * @return
     */
    @PostMapping("/salary-closure")
    Message<Void> salaryClosure(@Valid @RequestBody SalaryClosureReq salaryClosureReq);
}
