package cn.fw.morax.sdk.dto.common;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : IndicatorValueEnum
 * @description : 指标值类型
 * @date : 2022-04-24 14:37
 */
public enum IndicatorValueEnum {
    /**
     * 百分比
     */
    RATIO(1, "百分比"),
    /**
     * 数量/金额
     */
    QUANTITY(2, "数量/金额"),
    /**
     * 条件值
     */
    CONDITION(3, "条件值"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    IndicatorValueEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static IndicatorValueEnum ofValue(final Integer value) {
        for (final IndicatorValueEnum salaryValueEnum : IndicatorValueEnum.values()) {
            if (salaryValueEnum.value.equals(value)) {
                return salaryValueEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final IndicatorValueEnum salaryValueEnum : IndicatorValueEnum.values()) {
            if (salaryValueEnum.value.equals(value)) {
                return salaryValueEnum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    public Integer getValue() {
        return value;
    }
}
