package cn.fw.morax.sdk.dto.common;

import cn.fw.common.validator.EnumValue;
import lombok.Data;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;

/**
 * @author : kurisu
 * @version : 2.0
 * @className : IndicatorValueReq
 * @description : 人员绩效指标值
 * @date : 2022-12-22 09:45
 */
@Data
@ScriptAssert.List({
        @ScriptAssert(lang = "javascript", script = "_this.checkConditionValue()", message = "指标项key不正确"),
})
public class IndicatorValueReq {
    private static final String regNo = "^[-\\+]?[\\d]*$";
    /**
     * 人员id
     */
    @NotNull(message = "人员id不能为空")
    private Long userId;
    /**
     * 指标编码
     */
    @NotBlank(message = "指标编码不能为空")
    private String indicatorCode;
    /**
     * 指标值类型
     *
     * @see IndicatorValueEnum
     */
    @NotNull(message = "指标值类型不能为空")
    @EnumValue(enumClass = IndicatorValueEnum.class, message = "指标值枚举值不正确")
    private Integer valueType;
    /**
     * 指标项
     */
    @NotEmpty(message = "指标项不能为空")
    @Valid
    private List<IndicatorValueDTO> indicatorValue;
    /**
     * 数据日期
     */
    @NotNull(message = "数据日期不能为空")
    @Past(message = "数据日期必须为过去的时间")
    private Date dataDate;
    /**
     * 集团id
     */
    @NotNull(message = "集团id不能为空")
    private Long groupId;

    public boolean checkConditionValue() {
        if (IndicatorValueEnum.CONDITION.getValue().equals(valueType)) {
            return indicatorValue.stream().allMatch(r -> isNumber(r.getKey()));
        }
        return true;
    }

    private boolean isNumber(String str) {
        if (str == null) {
            return false;
        }
        Pattern compile = Pattern.compile(regNo);
        return compile.matcher(str).matches();
    }
}

