package cn.fw.morax.sdk.dto.salary;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryRewardReq
 * @description : 奖惩薪酬数据
 * @date : 2022-04-24 15:05
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class SalaryRewardReq {
    /**
     * 用户id
     */
    @NotNull(message = "用户id不能为空")
    private Long userId;
    /**
     * 岗位id
     */
    @NotNull(message = "岗位不能为空")
    private Long postId;
    /**
     * 在职门店id
     */
    @NotNull(message = "在职门店不能为空")
    private Long shopId;
    /**
     * 数据产生时间
     */
    @NotNull(message = "数据产生时间不能为空")
    @Past(message = "数据产生时间必须为过去的时间")
    private Date dataTime;
    /**
     * 金额
     */
    @NotNull(message = "金额不能为空")
    @Min(value = 0, message = "金额不能小于0")
    private BigDecimal amount;
    /**
     * 是否需要及时处理
     * 为'真'时会立即推送给财务，否则会并入工资里结算
     */
    private Boolean timely;

    /**
     * 是否是奖励
     * 为'真'时为奖励，否则为处罚
     * 为空按'假'处理
     */
    private Boolean award;

    /**
     * 业务id
     * 查询详情是使用
     */
    @NotNull
    private String dataId;

    /**
     * 款项描述
     */
    @NotNull
    private String dataName;

    /**
     * 类型id
     */
    @NotNull(message = "类型id不能为空")
    private Long typeId;

    /**
     * 类型名称
     */
    @NotBlank(message = "类型名称不能为空")
    private String typeName;
}
