package cn.fw.morax.server.consumer;

import cn.fw.approval.sdk.mq.ApprovalResult;
import cn.fw.morax.common.pojo.event.ApprovalResultEvent;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.SettingDraft;
import cn.fw.morax.service.biz.ApprovalBizService;
import cn.fw.morax.service.biz.SettingDraftBizService;
import cn.fw.morax.service.biz.eval.EvalGroupBizService;
import cn.fw.morax.service.biz.eval.EvalGroupDataService;
import cn.fw.morax.service.biz.eval.EvalRewardService;
import cn.fw.morax.service.biz.kpi.KpiGroupBizService;
import cn.fw.morax.service.biz.kpi.KpiStarSpecialRuleBizService;
import cn.fw.morax.service.biz.salary.IncomeCompensationBizService;
import cn.fw.morax.service.biz.salary.SalaryGroupBizService;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author : Devin
 * Date: 2020/10/22
 * Time: 17:34
 * Description: 审批消费
 */
@Slf4j
@Service
@RocketMQMessageListener(topic = ApprovalResult.TOPIC, consumerGroup = "${spring.application.name}_" + ApprovalResult.TOPIC)
@RequiredArgsConstructor
public class FlowConsumer implements RocketMQListener<ApprovalResult> {

    private final KpiStarSpecialRuleBizService kpiStarSpecialRuleBizService;
    private final IncomeCompensationBizService incomeCompensationBizService;
    private final SalaryGroupBizService salaryGroupBizService;
    private final EvalGroupDataService evalGroupDataService;
    private final EvalGroupBizService evalGroupBizService;
    private final KpiGroupBizService kpiGroupBizService;
    private final ApprovalBizService approvalBizService;
    private final EvalRewardService evalRewardService;
    private final SettingDraftBizService settingDraftBizService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void onMessage(ApprovalResult result) {
        log.info("审批系统MQ消息回调：{}", JSONObject.toJSONString(result));
        try {
            Boolean agree = result.getAgree();
            ApprovalRecord approvalRecord = approvalBizService.closeApprove(result.getOrderNo(), agree);
            if (PublicUtil.isEmpty(approvalRecord)) {
                return;
            }
            ApprovalResultEvent approvalResult = PublicUtil.copy(result, ApprovalResultEvent.class);
            this.doChangeCustomer(approvalRecord, approvalResult);
        } catch (Exception e) {
            log.error("审批系统回调处理异常", e);
        }
    }

    /**
     * 处理数据
     * @param approvalRecord
     */
    @Transactional(rollbackFor = Exception.class)
    public void doChangeCustomer(ApprovalRecord approvalRecord, ApprovalResultEvent result) {
        switch (approvalRecord.getApprovalType()) {
           case KPI: {
               kpiGroupBizService.approvalKpiGroupDraft(approvalRecord, result);
               break;
           }
           case SALARY: {
               salaryGroupBizService.approvalSalaryDraft(approvalRecord, result);
               break;
           }
           case COMPLAIN: {
               kpiStarSpecialRuleBizService.approvalKpiStarAdjustment(approvalRecord, result);
               break;
           }
           case INCOME_COMPENSATION: {
               incomeCompensationBizService.approvalCompensationConfig(approvalRecord, result);
               break;
           }
            case EVAL_GROUP_RANK: {
                SettingDraft settingDraft = settingDraftBizService.approvalDraft(approvalRecord, result);
                if (result.getAgree()) {
                    evalGroupBizService.approvalEvalConfig(approvalRecord, settingDraft);
                }
               break;
           }
            case KPI_GROUP_RANK: {
                SettingDraft settingDraft = settingDraftBizService.approvalDraft(approvalRecord, result);
                if (result.getAgree()) {
                    kpiGroupBizService.saveKpiGroupRankDraft(approvalRecord, settingDraft);
                }
               break;
           }
       }
    }


}
