package cn.fw.morax.server.consumer;

import cn.fw.morax.common.utils.MessageFormatUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.db.kpi.KpiGroupUser;
import cn.fw.morax.domain.db.kpi.KpiPool;
import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.db.salary.SalaryGroupUser;
import cn.fw.morax.domain.db.salary.SalaryPool;
import cn.fw.morax.rpc.oop.OopRpcService;
import cn.fw.morax.rpc.oop.dto.ShopDTO;
import cn.fw.morax.service.data.kpi.KpiGroupService;
import cn.fw.morax.service.data.kpi.KpiGroupUserService;
import cn.fw.morax.service.data.kpi.KpiPoolService;
import cn.fw.morax.service.data.salary.SalaryGroupService;
import cn.fw.morax.service.data.salary.SalaryGroupUserService;
import cn.fw.morax.service.data.salary.SalaryPoolService;
import cn.fw.oop.sdk.dto.AlterShopEvent;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author jiangchao
 * @des: 门店名称变更
 * @date 2022/11/23 19:27
 */
@Slf4j
@Service
@RocketMQMessageListener(topic = AlterShopEvent.TOPIC, consumerGroup = "${spring.application.name}_" + AlterShopEvent.TOPIC)
@RequiredArgsConstructor
public class OopAlterShopConsumer implements RocketMQListener<AlterShopEvent> {

    private final SalaryGroupUserService salaryGroupUserService;
    private final SalaryPoolService salaryPoolService;
    private final KpiGroupUserService kpiGroupUserService;
    private final KpiPoolService kpiPoolService;
    private final SalaryGroupService salaryGroupService;
    private final KpiGroupService kpiGroupService;
    private final OopRpcService oopRpcService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void onMessage(AlterShopEvent message) {
        log.info("门店名称变更MQ消息：{}", JSONObject.toJSONString(message));

        Long shopId = message.getShopId();
        String newShopShortName = message.getShortName();
//        List<SettingStatusEnum> status = new ArrayList<SettingStatusEnum>(){{
//            add(SettingStatusEnum.EFFECTIVE);
//            add(SettingStatusEnum.BE_EFFECTIVE);
//            add(SettingStatusEnum.BE_EFFECTIVE);
//        }};
        List<SalaryGroup> salaryGroups = salaryGroupService.list(Wrappers.<SalaryGroup>lambdaQuery()
                .eq(SalaryGroup::getYn, Boolean.TRUE)
                .apply(MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, shop_ids)", shopId))
        );
        if (PublicUtil.isNotEmpty(salaryGroups)) {
            for (SalaryGroup salaryGroup : salaryGroups) {
                List<ShopDTO> shopDTOS = oopRpcService.queryShops(salaryGroup.getShopIds());
                salaryGroup.setShopNames(shopDTOS.stream().map(ShopDTO::getShortName).collect(Collectors.toList()));
            }
            salaryGroupService.updateBatchById(salaryGroups);
        }


        List<KpiGroup> kpiGroups = kpiGroupService.list(Wrappers.<KpiGroup>lambdaQuery()
                .eq(KpiGroup::getYn, Boolean.TRUE)
                .apply(MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, shop_ids)", shopId))
        );
        if (PublicUtil.isNotEmpty(kpiGroups)) {
            for (KpiGroup kpiGroup : kpiGroups) {
                List<ShopDTO> shopDTOS = oopRpcService.queryShops(kpiGroup.getShopIds());
                kpiGroup.setShopNames(shopDTOS.stream().map(ShopDTO::getShortName).collect(Collectors.toList()));
            }
            kpiGroupService.updateBatchById(kpiGroups);
        }


        salaryGroupUserService.update(Wrappers.<SalaryGroupUser>lambdaUpdate()
                .set(SalaryGroupUser::getShopName, newShopShortName)
                .eq(SalaryGroupUser::getShopId, shopId)
        );
        salaryPoolService.update(Wrappers.<SalaryPool>lambdaUpdate()
                .set(SalaryPool::getShopName, newShopShortName)
                .eq(SalaryPool::getShopId, shopId)
        );
        kpiGroupUserService.update(Wrappers.<KpiGroupUser>lambdaUpdate()
                .set(KpiGroupUser::getShopName, newShopShortName)
                .eq(KpiGroupUser::getShopId, shopId)
        );
        kpiPoolService.update(Wrappers.<KpiPool>lambdaUpdate()
                .set(KpiPool::getShopName, newShopShortName)
                .eq(KpiPool::getShopId, shopId)
        );




    }
}
