package cn.fw.morax.server.controller.erp;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.CombineIndicatorAbleDTO;
import cn.fw.morax.domain.dto.CombineIndicatorDTO;
import cn.fw.morax.domain.dto.kpi.IndicatorAbleDTO;
import cn.fw.morax.domain.dto.kpi.KpiAssessDTO;
import cn.fw.morax.domain.dto.kpi.KpiAssessStaffDTO;
import cn.fw.morax.domain.dto.query.CombineIndicatorQueryDTO;
import cn.fw.morax.domain.dto.query.KpiAssessQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupStaffQueryDTO;
import cn.fw.morax.domain.enums.ApplicableTypeEnum;
import cn.fw.morax.domain.vo.CombineAndEvalIndicatorVO;
import cn.fw.morax.domain.vo.CombineIndicatorVO;
import cn.fw.morax.domain.vo.RoleInfoVO;
import cn.fw.morax.domain.vo.eval.CompositeIndicatorVO;
import cn.fw.morax.domain.vo.kpi.KpiAssessVO;
import cn.fw.morax.domain.vo.kpi.KpiGroupUserVO;
import cn.fw.morax.service.biz.kpi.CombineIndicatorBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 组合指标
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/erp/combination-indicator")
public class CombineIndicatorController {

    private final CombineIndicatorBizService combineIndicatorBizService;

    /**
     * 自定义指标（组合指标、考评指标）
     *
     * @param dto
     * @return
     */
    @GetMapping("/eval-indicator")
    @ControllerMethod("自定义指标指标列表")
    public Message<List<CombineAndEvalIndicatorVO>> combineAndEvalIndicators(@Validated CombineIndicatorQueryDTO dto) {
        return success(combineIndicatorBizService.combineAndEvalIndicators(dto));
    }

    /**
     * 组合指标列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/page")
    @ControllerMethod("组合指标列表")
    public Message<AppPage<CombineIndicatorVO>> page(@Validated CombineIndicatorQueryDTO dto) {
        return success(combineIndicatorBizService.page(dto));
    }

    /**
     * 组合指标详情
     *
     * @param id
     * @return
     */
    @GetMapping("/detail")
    @ControllerMethod("组合指标详情")
    public Message<CombineIndicatorVO> detail(@NotNull(message = "组合指标id不能为空") Long id,
                                      @CurrentUser LoginAuthBean currentUser) {
        return success(combineIndicatorBizService.detail(id));
    }

    /**
     * 组合指标保存
     *
     * @param dto
     * @return
     */
    @PostMapping("/save")
    @ControllerMethod("组合指标保存")
    public Message<Void> save(@Validated @RequestBody CombineIndicatorDTO dto,
                                              @CurrentUser LoginAuthBean currentUser) {
        combineIndicatorBizService.save(dto);
        return success();
    }

    /**
     * 组合指标删除
     *
     * @param id
     * @return
     */
    @GetMapping("/del")
    @ControllerMethod("组合指标删除")
    public Message<Void> delKpiAssess(@NotNull(message = "组合指标id不能为空") Long id,
                                                      @CurrentUser LoginAuthBean currentUser) {
        combineIndicatorBizService.removeById(id);
        return success();
    }

    /**
     * 组合指标库禁用启用
     *
     * @return
     */
    @PutMapping("/enable")
    @ControllerMethod("组合指标库禁用启用")
    public Message<Void> enableIndicator(@Validated @RequestBody CombineIndicatorAbleDTO dto) {
        combineIndicatorBizService.enableCombineIndicator(dto.getId(), dto.getEnable());
        return success();
    }

}
