package cn.fw.morax.server.controller.erp;


import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.EvalIndicatorDTO;
import cn.fw.morax.domain.dto.eval.EvalConfigPostIndicatorsDTO;
import cn.fw.morax.domain.dto.eval.EvalGroupRankDTO;
import cn.fw.morax.domain.dto.eval.EvalShopIndicatorsDTO;
import cn.fw.morax.domain.dto.kpi.IndicatorAbleDTO;
import cn.fw.morax.domain.dto.query.EvalIndicatorQueryDTO;
import cn.fw.morax.domain.dto.query.QueryConditionEvalIndicatorsListDTO;
import cn.fw.morax.domain.enums.ApplicableTypeEnum;
import cn.fw.morax.domain.vo.eval.*;
import cn.fw.morax.service.biz.eval.EvalGroupBizService;
import cn.fw.morax.service.biz.eval.EvalIndicatorBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考评指标
 *
 * @author jiangchao
 * @since 2022-11-09
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/erp/eval")
public class EvalConfigController {

    private final EvalGroupBizService evalGroupBizService;
    private final EvalIndicatorBizService evalIndicatorBizService;

    /**
     * 考评指标列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/indicator/page")
    @ControllerMethod("考评指标列表")
    public Message<AppPage<EvalIndicatorVO>> evalIndicatorPage(@Validated EvalIndicatorQueryDTO dto) {
        return success(evalIndicatorBizService.evalIndicatorPage(dto));
    }

    /**
     * 考评指标详情
     *
     * @return
     */
    @GetMapping("/indicator")
    @ControllerMethod("考评指标详情")
    public Message<EvalIndicatorVO> evalIndicatorDetail(@NotNull(message = "考评指标id不能为空") Long id) {
        return success(evalIndicatorBizService.evalIndicatorDetail(id));
    }

    /**
     * 考评指标禁用启用
     *
     * @return
     */
    @PutMapping("/indicator/enable")
    @ControllerMethod("考评指标禁用启用")
    public Message<Void> enableIndicator(@Validated @RequestBody IndicatorAbleDTO dto) {
        evalIndicatorBizService.enableIndicator(dto.getIndicatorId(), dto.getEnable());
        return success();
    }

    /**
     * 保存考评指标
     *
     * @param dto 考评指标
     * @return
     */
    @PostMapping("/indicator")
    @ControllerMethod("保存考评指标")
    public Message<Void> saveEvalIndicator(@Validated @RequestBody EvalIndicatorDTO dto,
                                                                @CurrentUser LoginAuthBean user) {
        evalIndicatorBizService.saveEvalIndicator(dto, user);
        return success();
    }

    /**
     * 获取指标
     *
     * @param dto
     * @return
     */
    @GetMapping("/indicators")
    @ControllerMethod("获取指标")
    public Message<List<CompositeIndicatorVO>> getIndicators(@Validated EvalConfigPostIndicatorsDTO dto) {
        return success(evalIndicatorBizService.getIndicators(dto, ApplicableTypeEnum.EVAL));
    }

    /**
     * 获取门店的指标
     *
     * @return
     */
    @GetMapping("/shop/indicator")
    @ControllerMethod("获取门店的指标")
    public Message<List<CompositeIndicatorVO>> getShopIndicators(@Validated EvalShopIndicatorsDTO dto) {
        return success(evalIndicatorBizService.getShopIndicators(dto,ApplicableTypeEnum.EVAL));
    }

    /**
     * 禁用考评
     *
     * @param id 绩效组id
     * @return
     */
    @GetMapping("/group-disable")
    @ControllerMethod("禁用考评")
    public Message<Void> disableEval(@NotNull(message = "绩效组id不能为空") Long id,
                                      @CurrentUser LoginAuthBean currentUser) {
        evalGroupBizService.disableEval(id, currentUser);
        return success();
    }


    /**
     * 获取带条件的考评指标
     *
     * @param dto
     * @return
     */
    @GetMapping("/indicator/queryConditionEvalIndicatorsList")
    @ControllerMethod("获取带条件的考评指标")
    public Message<List<QueryConditionEvalIndicatorsListVO>> queryConditionEvalIndicatorsList(@Validated QueryConditionEvalIndicatorsListDTO dto) {
        return success(evalIndicatorBizService.queryConditionEvalIndicatorsList(dto));
    }

}
