package cn.fw.morax.server.controller.erp;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.eval.EvalGroupRankDTO;
import cn.fw.morax.domain.dto.query.EvalGroupRankQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupRankStaffQueryDTO;
import cn.fw.morax.domain.dto.query.ShopsPostsQueryDTO;
import cn.fw.morax.domain.vo.eval.EvalGroupRankVO;
import cn.fw.morax.domain.vo.eval.EvalGroupUserVO;
import cn.fw.morax.domain.vo.eval.EvalGroupVO;
import cn.fw.morax.service.biz.eval.EvalGroupBizService;
import cn.fw.morax.service.biz.eval.EvalGroupRankBizService;
import cn.fw.morax.service.biz.eval.EvalGroupUserBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考评排名配置
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@RequestMapping("/erp/eval-rank")
@IgnoreAuth
public class EvalGroupRankController {

    private final EvalGroupRankBizService evalGroupRankBizService;
    private final EvalGroupUserBizService evalGroupUserBizService;
    private final EvalGroupBizService evalGroupBizService;

    /**
     * 考评排名列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/page")
    @ControllerMethod("考评排名列表")
    public Message<AppPage<EvalGroupRankVO>> evalGroupRankPage(@Validated EvalGroupRankQueryDTO dto) {
        return success(evalGroupRankBizService.evalGroupRankPage(dto));
    }

    /**
     * 保存考评排名组
     *
     * @param dto 考评
     * @return
     */
    @PostMapping("/group-rank/{submit}")
    @ControllerMethod("保存考评排名组")
    public Message<Void> saveEvalGroupRank(@Validated @RequestBody EvalGroupRankDTO dto,
                                           @PathVariable("submit") Integer submit,
                                           @CurrentUser LoginAuthBean user) {
        Boolean isSubmit = (submit == 1);
        evalGroupRankBizService.saveEvalGroupRank(dto, user, isSubmit);
        return success();
    }

    /**
     * 考评排名组详情
     *
     * @return
     */
    @GetMapping("/rank-group")
    @ControllerMethod("考评排名组详情")
    public Message<EvalGroupRankVO> evalGroupRankDetail(@NotNull(message = "考评排名组id不能为空") Long id) {
        return success(evalGroupBizService.evalGroupRankDetail(id));
    }


    /**
     * 删除考评组排名组合包
     *
     * @param id 考评组排名组合包id
     * @return
     */
    @GetMapping("/del")
    @ControllerMethod("删除考评组排名组合包")
    public Message<Void> removeById(@NotNull(message = "考评组排名组合包id不能为空") Long id) {
        evalGroupRankBizService.removeById(id);
        return success();
    }

    /**
     * 查询考评组排名所有人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/all-staffs")
    @ControllerMethod("查询考评组排名所有人员")
    public Message<List<EvalGroupUserVO>> queryRankStaffs(@Validated({KpiGroupRankStaffQueryDTO.EvalGroupQuery.class}) KpiGroupRankStaffQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getEvalGroupIds())) {
            throw new BusinessException("参数错误");
        }
        return success(evalGroupUserBizService.queryRankStaffs(dto.getEvalGroupIds()));
    }

    /**
     * 查询门店岗位人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/shop-post-staffs")
    @ControllerMethod("查询门店岗位人员")
    public Message<List<EvalGroupUserVO>> queryRankStaffs(@Validated ShopsPostsQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getPostIds())) {
            throw new BusinessException("参数错误");
        }
        return success(evalGroupUserBizService.queryShopPostStaffs(dto));
    }

    /**
     * 查询考评组人员
     *
     * @param evalGroupId
     * @return
     */
    @GetMapping("/group-staffs")
    @ControllerMethod("查询考评组人员")
    public Message<List<EvalGroupUserVO>> queryEvalGroupStaffs(@NotNull(message = "考评组id不能为空") Long evalGroupId) {
        return success(evalGroupUserBizService.queryEvalGroupStaffs(evalGroupId));
    }

}
