package cn.fw.morax.server.controller.erp;


import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.IncomeCompensationDTO;
import cn.fw.morax.domain.dto.query.IncomeCompensationQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupQueryDTO;
import cn.fw.morax.domain.vo.kpi.KpiGroupIndicatorVO;
import cn.fw.morax.domain.vo.kpi.KpiGroupVO;
import cn.fw.morax.domain.vo.salary.IncomeCompensationVO;
import cn.fw.morax.service.biz.kpi.KpiGroupIndicatorBizService;
import cn.fw.morax.service.biz.salary.IncomeCompensationBizService;
import cn.fw.morax.service.data.salary.IncomeCompensationService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 岗位收入补偿
 *
 * @author jiangchao
 * @since 2022-11-09
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/erp/compensation")
public class IncomeCompensationController {

    private final IncomeCompensationBizService incomeCompensationBizService;
    private final IncomeCompensationService incomeCompensationService;

    /**
     * 岗位最低收入补偿
     *
     * @param dto
     * @return
     */
    @GetMapping("/income-compensation/page")
    @ControllerMethod("岗位最低收入补偿列表")
    public Message<AppPage<IncomeCompensationVO>> incomeCompensationPage(@Validated IncomeCompensationQueryDTO dto) {
        return success(incomeCompensationBizService.incomeCompensationPage(dto));
    }

    /**
     * 岗位最低收入补偿详情
     *
     * @return
     */
    @GetMapping("/income-compensation")
    @ControllerMethod("绩效组详情")
    public Message<IncomeCompensationVO> incomeCompensationDetail(@NotNull(message = "岗位最低收入补偿id不能为空") Long id) {
        return success(incomeCompensationBizService.incomeCompensationDetail(id));
    }

    /**
     * 保存岗位最低收入补偿
     *
     * @param dto 岗位最低收入补偿
     * @return
     */
    @PostMapping("/income-compensation")
    @ControllerMethod("保存岗位最低收入补偿")
    public Message<IncomeCompensationVO> saveIncomeCompensation(@Validated @RequestBody IncomeCompensationDTO dto,
                                                                @CurrentUser LoginAuthBean user) {
        incomeCompensationBizService.saveIncomeCompensation(dto, user);
        return success();
    }


}
