package cn.fw.morax.server.controller.erp;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.kpi.KpiAssessDTO;
import cn.fw.morax.domain.dto.kpi.KpiAssessStaffDTO;
import cn.fw.morax.domain.dto.query.KpiAssessQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupStaffQueryDTO;
import cn.fw.morax.domain.vo.RoleInfoVO;
import cn.fw.morax.domain.vo.kpi.KpiAssessVO;
import cn.fw.morax.domain.vo.kpi.KpiGroupUserVO;
import cn.fw.morax.service.biz.kpi.KpiAssessBizService;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

import java.time.LocalDate;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 人员质量评测
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/erp/kpi/assess")
public class KpiAssessController {

    private final KpiAssessBizService kpiAssessBizService;
    private final KpiGroupUserBizService kpiGroupUserBizService;

    /**
     * 人员质量考核列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/page")
    @ControllerMethod("人员质量考核列表")
    public Message<AppPage<KpiAssessVO>> kpiAssessPage(@Validated KpiAssessQueryDTO dto) {
        return success(kpiAssessBizService.kpiAssessPage(dto));
    }

    /**
     * 人员质量考核详情
     *
     * @param id
     * @return
     */
    @GetMapping("/detail")
    @ControllerMethod("人员质量考核详情")
    public Message<KpiAssessVO> kpiAssessDetail(@NotNull(message = "人员质量考核id不能为空") Long id,
                                      @CurrentUser LoginAuthBean currentUser) {
        return success(kpiAssessBizService.kpiAssessDetail(id));
    }

    /**
     * 人员质量考核保存
     *
     * @param dto
     * @return
     */
    @PostMapping("/save")
    @ControllerMethod("人员质量考核保存")
    public Message<Void> saveKpiAssess(@Validated @RequestBody KpiAssessDTO dto,
                                              @CurrentUser LoginAuthBean currentUser) {
        kpiAssessBizService.saveKpiAssess(dto);
        return success();
    }

    /**
     * 人员质量考核删除
     *
     * @param id
     * @return
     */
    @GetMapping("/del")
    @ControllerMethod("人员质量考核删除")
    public Message<Void> delKpiAssess(@NotNull(message = "人员质量考核id不能为空") Long id,
                                                      @CurrentUser LoginAuthBean currentUser) {
        kpiAssessBizService.removeById(id);
        return success();
    }

    /**
     * 绩效转正后保护期员工
     *
     * @param dto
     * @return
     */
    @GetMapping("/regular-protection/staffs")
    @ControllerMethod("绩效转正后保护期员工")
    public Message<List<KpiGroupUserVO>> kpiRegularProtectionStaffs(@Validated KpiGroupStaffQueryDTO dto) {
        return success(kpiAssessBizService.kpiRegularProtectionStaffs(dto.getPostId(), dto.getShopIds(), dto.getStaffName(), LocalDate.now()));
    }

    /**
     * 岗位、门店所有员工（星级为D级员工使用）
     *
     * @param dto
     * @return
     */
    @GetMapping("/real-time/staffs")
    @ControllerMethod("岗位、门店所有员工（星级为D级员工使用）")
    public Message<List<KpiGroupUserVO>> queryShopPostRealTimeStaffs(@Validated KpiGroupStaffQueryDTO dto) {
        return success(kpiGroupUserBizService.queryShopPostRealTimeStaffs(dto));
    }

    /**
     * 考核员工（考核详情使用）
     *
     * @param dto
     * @return
     */
    @GetMapping("/assess/staffs")
    @ControllerMethod("考核员工（考核详情使用））")
    public Message<List<KpiGroupUserVO>> queryAssessStaffs(@Validated KpiAssessStaffDTO dto) {
        return success(kpiAssessBizService.queryAssessStaffs(dto));
    }

    /**
     * 人员质量考核角色
     *
     * @return
     */
    @GetMapping("/roles")
    @ControllerMethod("人员质量考核角色")
    public Message<List<RoleInfoVO>> assessRoleInfos() {
        return success(kpiAssessBizService.assessRoleInfos());
    }



}
