package cn.fw.morax.server.controller.erp;

import cn.fw.approval.sdk.mq.ApprovalResult;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.query.KpiGroupStaffQueryDTO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.server.consumer.FlowConsumer;
import cn.fw.morax.server.task.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : DebugController
 * @description : 绩效通用调试控制器
 * @date : 2022-04-11 10:45
 * @ignore
 */
@Slf4j
@RestController
@Authorization(AuthType.NONE)
@RequiredArgsConstructor
@Validated
@RequestMapping("/debug/task/kpi")
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class KpiTaskController {
    private final SalaryPoolFundShopTask salaryPoolFundShopTask;
    private final KpiCalcMonthlyTask kpiCalcMonthlyTask;
    private final KpiStarSpecialRuleTask kpiStarSpecialRuleTask;
    private final KpiGroupUserTask kpiGroupUserTask;
    private final KpiReportRatioTask kpiReportRatioTask;
    private final SalaryReportOldTask salaryReportOldTask;
    private final SalaryGroupTask salaryGroupTask;
    private final KpiGroupTask kpiGroupTask;
    private final EhrRpcService ehrRpcService;
    private final FlowConsumer flowConsumer;
    private final KpiCalcTask kpiCalcTask;
    private final EvalUserReportTask evalUserReportTask;
    private final KpiReportTask kpiReportTask;


    @GetMapping("/current-month/kpi-config")
    @ControllerMethod("当月绩效组定时器")
    public Message<Void> processCurMonthEffectKpis() {
        kpiGroupTask.processCurMonthEffectKpis();
        return success();
    }

    @GetMapping("/current-month/salary-config")
    @ControllerMethod("当月薪酬组定时器")
    public Message<Void> processCurMonthEffectSalaryGroup() {
        salaryGroupTask.processCurMonthEffectSalaryGroup();
        return success();
    }

    @GetMapping("/kpi-users")
    @ControllerMethod("当天绩效组人员定时器")
    public Message<Void> processKpiUser() {
        kpiGroupUserTask.processKpiUser();
        return success();
    }

    @GetMapping("/kpi-report")
    @ControllerMethod("绩效报表定时任务")
    public Message<Void> kpiReport(Date date) {
        if (PublicUtil.isEmpty(date)) {
            kpiReportRatioTask.kpiReport(DateUtil.date2LocalDate(date).minusDays(1));
        } else {
            kpiReportRatioTask.kpiReport(DateUtil.date2LocalDate(date));
        }
        return success();
    }

    @GetMapping("/salary-report")
    @ControllerMethod("薪酬报表定时任务")
    public Message<Void> salaryReport(Date date) {
        if (PublicUtil.isEmpty(date)) {
            salaryReportOldTask.salaryReport(DateUtil.date2LocalDate(date).minusDays(1));
        } else {
            salaryReportOldTask.salaryReport(DateUtil.date2LocalDate(date));
        }
        return success();
    }

    @GetMapping("/salary-user")
    @ControllerMethod("当天薪酬组人员定时器")
    public Message<Void> salaryGroupUser() {
        salaryGroupTask.asyncSalaryGroupUser();
        return success();
    }

    @GetMapping("/calculate-kpi")
    @ControllerMethod("计算绩效得分")
    public Message<Void> calculateKpi() {
        kpiCalcTask.calculateKpi();
        return success();
    }

    @GetMapping("/calculate-kpi-rate")
    @ControllerMethod("计算绩效得分率")
    public Message<Void> calcuScoringRate() {
        kpiCalcTask.calcuScoringRate();
        return success();
    }

    @GetMapping("/calculate-star")
    @ControllerMethod("计算绩效星级")
    public Message<Void> calcuStarLevel() {
        kpiCalcTask.calcuStarLevel();
        return success();
    }

    @GetMapping("/cache-day-assess")
    @ControllerMethod("缓存每天人员质量考评")
    public Message<Void> cacheDayKpiAssess() {
        kpiCalcTask.cacheKpiAssess();
        return success();
    }

    @GetMapping("/cache/kpi-group")
    @ControllerMethod("需要计算绩效得分的绩效组写入缓存")
    public Message<Void> cacheCalculableKpiGroup() {
        kpiCalcTask.cacheCalculableKpiGroup();
        return success();
    }

    @GetMapping("/cache/kpi-rank")
    @ControllerMethod("需要计算绩效排名的绩效组写入缓存")
    public Message<Void> cacheCalculableKpiGroupRank() {
        kpiCalcTask.cacheCalculableKpiGroupRank();
        return success();
    }

    /**
     * 实时查询绩效组人员
     * @param dto
     * @return
     */
    @PostMapping("/post-shop/staffs")
    @ControllerMethod("实时查询绩效组人员")
    public Message<List<PerformanceStaffDTO>> queryShopPostRealTimeStaffs(@Validated @RequestBody KpiGroupStaffQueryDTO dto) {
        return success(ehrRpcService.queryKpiStaff(dto.getPostId(), dto.getShopIds()));
    }

    /**
     * 审批消息
     * @param result
     * @return
     */
    @PostMapping("/flow")
    @ControllerMethod("审批消息")
    public Message<Void> approvalResult(@Validated @RequestBody ApprovalResult result) {
        flowConsumer.onMessage(result);
        return success();
    }

    /**
     * 社保公积金门店赋值
     *
     * @return
     */
    @PostMapping("/fund")
    @ControllerMethod("社保公积金门店赋值")
    public Message<Void> dealSalaryPoolFundShop() {
        salaryPoolFundShopTask.dealSalaryPoolFundShop();
        return success();
    }

    /**
     * 发送业务上报mq通知
     *
     * @return
     */
    @GetMapping("/kpi-mp")
    @ControllerMethod("发送业务上报mq通知")
    public Message<Void> sendKpiNotice() {
//        kpiCalcTask.sendNotice();
        return success();
    }

    /**
     * 几月开始计算绩效的星级调整
     *
     * @return
     */
    @GetMapping("/kpi-adjustment")
    @ControllerMethod("几月开始计算绩效的星级调整")
    public Message<Void> processKpiStarSpecialRule() {
        kpiStarSpecialRuleTask.processKpiStarSpecialRule();
        return success();
    }

    /**
     * 缓存人员质量评测绩效组
     *
     * @return
     */
    @GetMapping("/assess-kpi-group")
    @ControllerMethod("缓存人员质量评测绩效组")
    public Message<Void> cacheAssessFinalCalcKpiGroup() {
        kpiCalcMonthlyTask.cacheAssessFinalCalcKpiGroup();
        return success();
    }

    /**
     * 缓存人员质量评测绩效组排名
     *
     * @return
     */
    @GetMapping("/assess-kpi-group-rank")
    @ControllerMethod("缓存人员质量评测绩效组排名")
    public Message<Void> cacheAssessFinalCalcKpiGroupRank() {
        kpiCalcMonthlyTask.cacheAssessFinalCalcKpiGroupRank();
        return success();
    }
    /**
     * 备份当月人员质量考核数据
     *
     * @return
     */
    @GetMapping("/backup-assess")
    @ControllerMethod("备份当月人员质量考核数据")
    public Message<Void> backKpiAssesses() {
        kpiCalcMonthlyTask.backKpiAssesses();
        return success();
    }
    /**
     * 缓存人员质量评测
     *
     * @return
     */
    @GetMapping("/cache-assess")
    @ControllerMethod("缓存人员质量评测")
    public Message<Void> cacheKpiAssess() {
        kpiCalcMonthlyTask.cacheKpiAssess();
        return success();
    }
    /**
     * 计算人员质量评测
     *
     * @return
     */
    @GetMapping("/calc-assess")
    @ControllerMethod("计算人员质量评测")
    public Message<Void> calcKpiAssess() {
        kpiCalcMonthlyTask.calcAssessKpi();
        return success();
    }

    /**
     * 推送评测奖惩数据
     *
     * @return
     */
    @GetMapping("/push-assess")
    @ControllerMethod("推送评测奖惩数据")
    public Message<Void> pushAssessPunish() {
        kpiCalcMonthlyTask.pushAssessPunish();
        return success();
    }
    /**
     * 最终计算缓存上月绩效组
     *
     * @return
     */
    @GetMapping("/final-kpi-group")
    @ControllerMethod("最终计算缓存上月绩效组")
    public Message<Void> cacheFinalCalcKpiGroup() {
        kpiCalcMonthlyTask.cacheFinalCalcKpiGroup();
        return success();
    }
    /**
     * 最终计算缓存上月绩效排名组
     *
     * @return
     */
    @GetMapping("/final-kpi-group-rank")
    @ControllerMethod("最终计算缓存上月绩效排名组")
    public Message<Void> cacheFinalCalcKpiGroupRank() {
        kpiCalcMonthlyTask.cacheFinalCalcKpiGroupRank();
        return success();
    }
    /**
     * 考评人员报表定时任务
     *
     * @return
     */
    @GetMapping("/eval-user-report")
    @ControllerMethod("考评人员报表定时任务")
    public Message<Void> evalUserReportTask() {
        evalUserReportTask.evalUserReportTask();
        return success();
    }
//    /**
//     * 考评门店报表定时任务
//     *
//     * @return
//     */
//    @GetMapping("/eval-shop-report")
//    @ControllerMethod("考评门店报表定时任务")
//    public Message<Void> evalShopReportTask() {
//        evalShopReportTask.evalShopReportTask();
//        return success();
//    }

    /**
     * KpiReportTask 绩效报表定时任务 debug
     *
     * @return
     */
    @GetMapping("/kpi-report-run")
    @ControllerMethod("绩效报表定时任务")
    public Message<Void> kpiReportTaskRun(Date date) {
        if (PublicUtil.isEmpty(date)) {
            kpiReportTask.kpiReport(LocalDate.now().minusDays(1));
        } else {
            kpiReportTask.kpiReport(DateUtil.date2LocalDate(date));
        }
        return success();
    }

    /**
     * 处理绩效旧组合指标
     *
     * @return
     */
    @GetMapping("/kpi-combine")
    @ControllerMethod("处理绩效旧组合指标")
    public Message<Void> kpiProcessCombine() {
        kpiGroupTask.processCombine();
        return success();
    }

    /**
     * 处理薪酬旧组合指标
     *
     * @return
     */
    @GetMapping("/salary-combine")
    @ControllerMethod("处理薪酬旧组合指标")
    public Message<Void> salaryProcessCombine() {
        salaryGroupTask.processCombine();
        return success();
    }

}
