package cn.fw.morax.server.controller.erp;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.salary.SalaryGeneralSettinDTO;
import cn.fw.morax.domain.vo.salary.SalaryGeneralSettinVO;
import cn.fw.morax.service.biz.salary.SalaryGeneralSettinBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 薪酬通用配置
 *
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryCommonSettingController
 * @description : 薪酬通用配置
 * @date : 2022-05-12 09:42
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@RequestMapping("/erp/salary/config")
public class SalaryCommonSettingController {
    private final SalaryGeneralSettinBizService salaryGeneralSettinBizService;

    /**
     * 查看薪酬配置
     *
     * @param currentUser
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/detail")
    @ControllerMethod("查看薪酬系统配置")
    public Message<SalaryGeneralSettinVO> getSystemConfig(@CurrentUser LoginAuthBean currentUser) {
        return success(salaryGeneralSettinBizService.getSystemConfig(currentUser.getGroupId()));
    }

    /**
     * 薪酬配置保存
     *
     * @param dto
     * @return
     */
    @PostMapping("/save")
    @ControllerMethod("薪酬系统配置保存")
    public Message<Void> systemConfig(@Validated @RequestBody SalaryGeneralSettinDTO dto) {
        salaryGeneralSettinBizService.systemConfig(dto);
        return success();
    }
}
