package cn.fw.morax.server.controller.erp;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.SettingDraftDTO;
import cn.fw.morax.domain.dto.eval.EvalDTO;
import cn.fw.morax.domain.dto.eval.EvalGroupRankDTO;
import cn.fw.morax.domain.dto.kpi.*;
import cn.fw.morax.domain.dto.query.*;
import cn.fw.morax.domain.dto.salary.SalaryGroupDTO;
import cn.fw.morax.domain.enums.ApplicableTypeEnum;
import cn.fw.morax.domain.enums.SettingDraftTypeEnum;
import cn.fw.morax.domain.vo.SettingDraftVO;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.service.biz.SettingDraftBizService;
import cn.fw.morax.service.biz.kpi.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 草稿配置
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@RequestMapping("/erp/setting-draft")
@IgnoreAuth
public class SettingDraftController {

    private final SettingDraftBizService settingDraftBizService;

    /**
     * 绩效组列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/kpi-page")
    @ControllerMethod("绩效组列表")
    public Message<AppPage<KpiGroupDTO>> kpiGroupPage(@Validated SettingDraftQueryDTO dto) {
        return success(settingDraftBizService.kpiGroupPage(dto));
    }

    /**
     * 薪酬组列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/salary-page")
    @ControllerMethod("薪酬组列表")
    public Message<AppPage<SalaryGroupDTO>> salaryGroupPage(@Validated SettingDraftQueryDTO dto) {
        return success(settingDraftBizService.salaryGroupPage(dto));
    }

    /**
     * 考评列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/eval-page")
    @ControllerMethod("绩效组列表")
    public Message<AppPage<EvalDTO>> evalGroupPage(@Validated SettingDraftQueryDTO dto) {
        return success(settingDraftBizService.evalGroupPage(dto));
    }

    /**
     * 绩效排名组列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/kpi-rank-page")
    @ControllerMethod("绩效排名组列表")
    public Message<AppPage<KpiGroupRankDTO>> kpiGroupRankPage(@Validated SettingDraftQueryDTO dto) {
        return success(settingDraftBizService.kpiGroupRankPage(dto));
    }

    /**
     * 考评排名组列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/eval-rank-page")
    @ControllerMethod("考评排名组列表")
    public Message<AppPage<EvalGroupRankDTO>> evalGroupRankPage(@Validated SettingDraftQueryDTO dto) {
        return success(settingDraftBizService.evalGroupRankPage(dto));
    }

    /**
     * 绩效组草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/kpi-detail")
    @ControllerMethod("绩效组草稿详情")
    public Message<KpiGroupDTO> draftKpiGroupDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftKpiGroupDetail(id));
    }

    /**
     * 薪酬组草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/salary-detail")
    @ControllerMethod("绩效组草稿详情")
    public Message<SalaryGroupDTO> draftSalaryGroupDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftSalaryGroupDetail(id));
    }

    /**
     * 考评组草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/eval-detail")
    @ControllerMethod("考评组草稿详情")
    public Message<EvalDTO> draftEvalDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftEvalDetail(id));
    }

    /**
     * 绩效排名组草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/kpi-rank-detail")
    @ControllerMethod("绩效排名组草稿详情")
    public Message<KpiGroupRankDTO> draftKpiGroupRankDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftKpiGroupRankDetail(id));
    }

    /**
     * 考评排名组草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/eval-rank-detail")
    @ControllerMethod("考评排名组草稿详情")
    public Message<EvalGroupRankDTO> draftEvalGroupRankDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftEvalGroupRankDetail(id));
    }



    /**
     * 删除草稿，审批不通过、
     *
     * @return
     */
    @GetMapping("/del")
    @ControllerMethod("删除草稿")
    public Message<Void> delDraft(@NotNull(message = "草稿id不能为空") Long id,
                                     @CurrentUser LoginAuthBean currentUser) {
        settingDraftBizService.delDraft(id, currentUser);
        return success();
    }

    /**
     * 撤销审批
     *
     * @return
     */
    @GetMapping("/cancel")
    @ControllerMethod("撤销审批")
    public Message<Void> cancelDraft(@NotNull(message = "草稿id不能为空") Long id,
                                     @CurrentUser LoginAuthBean currentUser) {
        settingDraftBizService.cancelDraft(id, currentUser);
        return success();
    }

}
