package cn.fw.morax.server.runner;

import cn.fw.morax.common.utils.EventBusUtil;
import cn.fw.morax.domain.db.salary.SalaryGeneralSettin;
import cn.fw.morax.rpc.oop.OopRpcService;
import cn.fw.morax.rpc.oop.dto.GroupDTO;
import cn.fw.morax.service.component.*;
import cn.fw.morax.service.data.salary.SalaryGeneralSettinService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 必要配置项检查
 *
 * @author : kurisu
 * @version : 1.0
 * @className : CheckCommandLineRunner
 * @description : 必要配置项检查
 * @date : 2022-05-09 15:22
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class CheckCommandLineRunner implements CommandLineRunner {
    private final SalaryGeneralSettinService salaryGeneralSettinService;
    private final EvalIndicatorSaveObserver evalIndicatorSaveObserver;
    private final KpiStarAdjustmentObserver kpiStarAdjustmentObserver;
    private final SalaryGroupChangeObserver salaryGroupChangeObserver;
    private final KpiGroupChangeObserver kpiGroupChangeObserver;
    private final CalcSalaryObserver calcSalaryObserver;
    private final PaySalaryObserver paySalaryObserver;
    private final OopRpcService oopRpcService;


    @Override
    public void run(String... args) throws Exception {
        registerObserver();
        checkSetting();
    }

    /**
     * 注册监听器
     */
    public void registerObserver() {
        EventBusUtil.asyncRegister(kpiStarAdjustmentObserver);
        EventBusUtil.asyncRegister(paySalaryObserver);
        EventBusUtil.asyncRegister(kpiGroupChangeObserver);
        EventBusUtil.asyncRegister(salaryGroupChangeObserver);
        EventBusUtil.asyncRegister(evalIndicatorSaveObserver);
        EventBusUtil.asyncRegister(calcSalaryObserver);
    }

    /**
     * 检查配置
     */
    public void checkSetting() {
        List<GroupDTO> groupDTOList = oopRpcService.allGroups();
        if (CollectionUtils.isEmpty(groupDTOList)) {
            return;
        }
        for (GroupDTO groupData : groupDTOList) {
            queryAndInit(groupData);
        }
    }

    public void queryAndInit(GroupDTO group) {
        int count = salaryGeneralSettinService.count(Wrappers.<SalaryGeneralSettin>lambdaQuery()
                .eq(SalaryGeneralSettin::getGroupId, group.getId())
                .eq(SalaryGeneralSettin::getYn, Boolean.TRUE)
        );
        if (count < 1) {
            try {
                salaryGeneralSettinService.initData(group.getId());
            } catch (Exception e) {
                log.error("初始化基础设置失败：", e);
            }
        }
    }
}
