//package cn.fw.morax.server.task;
//
//import cn.fw.common.cache.locker.DistributedLocker;
//import cn.fw.morax.common.constant.Constant;
//import cn.fw.morax.common.constant.TimeTaskConstant;
//import cn.fw.morax.common.utils.PublicUtil;
//import cn.fw.morax.domain.db.eval.*;
//import cn.fw.morax.domain.db.kpi.IndicatorUserValue;
//import cn.fw.morax.domain.enums.*;
//import cn.fw.morax.service.biz.CommonService;
//import cn.fw.morax.service.data.eval.*;
//import cn.fw.morax.service.data.kpi.IndicatorUserValueService;
//import com.baomidou.mybatisplus.core.toolkit.Wrappers;
//import com.google.common.collect.Maps;
//import lombok.Getter;
//import lombok.RequiredArgsConstructor;
//import lombok.extern.slf4j.Slf4j;
//import org.redisson.api.RLock;
//import org.springframework.beans.factory.annotation.Value;
//import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
//import org.springframework.scheduling.annotation.Scheduled;
//import org.springframework.stereotype.Component;
//import org.springframework.transaction.PlatformTransactionManager;
//import org.springframework.transaction.TransactionDefinition;
//import org.springframework.transaction.TransactionStatus;
//import org.springframework.transaction.annotation.Transactional;
//
//import java.math.BigDecimal;
//import java.time.LocalDate;
//import java.time.YearMonth;
//import java.time.temporal.TemporalAdjusters;
//import java.util.*;
//import java.util.concurrent.atomic.AtomicInteger;
//import java.util.concurrent.locks.Lock;
//import java.util.function.Function;
//import java.util.stream.Collectors;
//
///**
// * @author : jiangchao
// * @className : KpiReportRatioTask
// * @description : 绩效报表定时器
// * @date : 2022-04-07 15:29
// */
//@Component
//@Slf4j
//@RequiredArgsConstructor
//@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
//public class EvalShopReportTask {
//
//    private final EvalGroupIndicatorTargetHitLogService evalGroupIndicatorTargetHitLogService;
//    private final EvalGroupIndicatorHitLogService evalGroupIndicatorHitLogService;
//    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
//    private final EvalShopRankCodeReportService evalShopRankCodeReportService;
//    private final EvalShopValueReportService evalShopValueReportService;
//    private final EvalGroupIndicatorService evalGroupIndicatorService;
//    private final IndicatorUserValueService indicatorUserValueService;
//    private final EvalIndicatorValueService evalIndicatorValueService;
//    private final PlatformTransactionManager platformTransactionManager;
//    private final EvalShopRankReportService evalShopRankReportService;
//    private final EvalShopReportService evalShopReportService;
//    private final TransactionDefinition transactionDefinition;
//    private final EvalShopPoolService evalShopPoolService;
//    private final EvalGroupRankService evalGroupRankService;
//    private final EvalGroupService evalGroupService;
//    private final DistributedLocker distributedLocker;
//    private final CommonService commonService;
//
//    @Value("${spring.cache.custom.global-prefix}:eval:shop:report")
//    @Getter
//    private String evalShopReportDistKey;
//
//    /**
//     * 考评报表定时任务
//     *
//     */
//    @Scheduled(cron = TimeTaskConstant.EVAL_REPORT)
//    @Transactional(rollbackFor = Exception.class)
//    public void evalShopReportTask() {
//        this.evalShopReport(LocalDate.now().minusDays(1));
//    }
//
//    /**
//     * 绩效报表定时任务
//     * 1. 遍历考评排名组
//     * 2. 创建考评排名组报表
//     * 3. 创建考评排名组编码
//     * 4. 统计数据
//     *
//     * @param date
//     */
//    public void evalShopReport(LocalDate date) {
//        Lock lock = distributedLocker.lock(getEvalShopReportDistKey());
//        if (! ((RLock) lock).isLocked()) {
//            return;
//        }
//        TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
//        try {
//            log.info("定时任务【考评人员报表数据抽取】开始执行");
//            Map<Long, List<EvalGroupRank>> groupKpiGroupMap = queryShopEffectRank(date);
//            for (Map.Entry<Long, List<EvalGroupRank>> entry : groupKpiGroupMap.entrySet()) {
//                for (EvalGroupRank rank : entry.getValue()) {
//                    //清除当月数据（当月只有一条数据）
//                    cleanRankCurMonthData(rank.getId(), date);
//                    //抽取排名组数据
//                    createRankReport(rank, entry.getKey(), date);
//                }
//            }
//            platformTransactionManager.commit(transactionStatus);
//        } catch (Exception e){
//            platformTransactionManager.rollback(transactionStatus);
//            log.error("考评人员报表数据抽取失败， " + e.getMessage(), e);
//        } finally {
//            lock.unlock();
//        }
//    }
//
//    /**
//     * 计算排名组报表数据
//     * @param rank
//     * @param groupId
//     * @param date
//     */
//    public void createRankReport(EvalGroupRank rank, Long groupId, LocalDate date) {
//        List<EvalGroup> evalGroups = evalGroupService.getByRankId(rank.getId());
//        List<Long> evalGroupIds = evalGroups.stream().map(EvalGroup::getId).collect(Collectors.toList());
//        if (PublicUtil.isEmpty(evalGroupIds)) {
//            return;
//        }
//        List<EvalShopPool> pools = evalShopPoolService.list(Wrappers.<EvalShopPool>lambdaQuery()
//                .in(EvalShopPool::getEvalGroupId, evalGroupIds)
//                .eq(EvalShopPool::getMonthly, YearMonth.from(date))
//                .eq(EvalShopPool::getYn, Boolean.TRUE)
//        );
//        if (PublicUtil.isEmpty(pools)) {
//            return;
//        }
//        List<EvalGroupIndicator> evalGroupIndicators = evalGroupIndicatorService.list(Wrappers.<EvalGroupIndicator>lambdaQuery()
//                .in(EvalGroupIndicator::getEvalGroupId, evalGroupIds)
//                .eq(EvalGroupIndicator::getYn, Boolean.TRUE)
//                .orderByAsc(EvalGroupIndicator::getId)
//        );
//
//        //排名组
//        EvalShopRankReport reportRank = this.saveRankReport(rank, evalGroupIds, evalGroups, date, groupId);
//        final Long evalShopRankReportId = reportRank.getId();
//        //排名组指标编码、人员、门店
//        this.saveRankCodeReports(evalGroupIndicators, evalShopRankReportId, date);
//        //门店维度
//        calcShop(pools, evalGroupIndicators, evalShopRankReportId, date);
//
//    }
//
//    public List<EvalShopValueReport> calcShop(
//                                              List<EvalShopPool> pools,
//                                              List<EvalGroupIndicator> evalGroupIndicators,
//                                              Long evalShopRankReportId,
//                                              LocalDate date) {
//        List<EvalShopValueReport> reportShopValues = new ArrayList<>();
//        List<Long> shopIds = pools.stream().map(EvalShopPool::getShopId).distinct().collect(Collectors.toList());
//        List<Long> shopPoolIds = pools.stream().map(EvalShopPool::getId).collect(Collectors.toList());
//        List<Long> evalGroupIndicatorIds = evalGroupIndicators.stream().map(EvalGroupIndicator::getId).collect(Collectors.toList());
//        List<EvalGroupIndicatorHitLog> hitLogs = evalGroupIndicatorHitLogService.getHitLogs(evalGroupIndicatorIds, shopPoolIds, EvalScopeEnum.SHOP, date);
//        Map<Long, List<EvalGroupIndicatorHitLog>> indicatorHitLogMap = hitLogs.stream().collect(Collectors.groupingBy(EvalGroupIndicatorHitLog::getEvalGroupIndicatorId));
//
//        for (EvalShopPool pool : pools) {
//            EvalShopReport shopReport = this.convertDim(pool, date, evalShopRankReportId);
//            evalShopReportService.save(shopReport);
//
//            for (EvalGroupIndicator groupIndicator : evalGroupIndicators) {
//                if (! pool.getEvalGroupId().equals(groupIndicator.getEvalGroupId())) {
//                    continue;
//                }
//                final IndicatorCodeTypeEnum codeType = groupIndicator.getCodeType();
//                final Long evalGroupIndicatorId = groupIndicator.getId();
//                List<EvalGroupIndicatorParam> params = evalGroupIndicatorParamService.list(Wrappers.<EvalGroupIndicatorParam>lambdaQuery()
//                        .eq(EvalGroupIndicatorParam::getEvalGroupIndicatorId, evalGroupIndicatorId)
//                        .eq(EvalGroupIndicatorParam::getYn, Boolean.TRUE)
//                );
//
//                List<EvalGroupIndicatorHitLog> userHitLogs = indicatorHitLogMap.getOrDefault(evalGroupIndicatorId, new ArrayList<>());
//                Map<Long, EvalGroupIndicatorHitLog> poolHitLogMap = userHitLogs.stream()
//                        .collect(Collectors.toMap(EvalGroupIndicatorHitLog::getPoolId, Function.identity(), (v1, v2) -> v1));
//                BigDecimal hitValue = this.getHitValue(poolHitLogMap, shopReport.getEvalShopPoolId(), groupIndicator.getScoreWay());
//                reportShopValues.add(convertDB(shopReport.getId(), groupIndicator, hitValue, date, params));
//                //是否有子指标
//                if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(codeType)) {
//                    for (EvalGroupIndicatorParam param : params) {
//                        if (TargetTypeEnum.NO.equals(param.getTargetType())) {
//                            Map<Long, BigDecimal> userValueMap = queryShopOriginValue(param, shopIds, date);
//                            BigDecimal value = userValueMap.getOrDefault(shopReport.getShopId(), BigDecimal.ZERO);
//                            reportShopValues.add(convertParamDB(shopReport.getId(), param, value, date));
//                        } else {
//                            Map<Long, EvalGroupIndicatorTargetHitLog> hitLogMap = queryShopTargetValue(param.getId(), shopPoolIds, date);
//                            BigDecimal value = Optional.ofNullable(hitLogMap.get(shopReport.getEvalShopPoolId()))
//                                    .map(EvalGroupIndicatorTargetHitLog::getReachValue).orElse(BigDecimal.ZERO).multiply(Constant.ONE_HUNDRED);
//                            reportShopValues.add(convertParamDB(shopReport.getId(), param, value, date));
//                        }
//                    }
//                }
//            }
//        }
//
//
//        if (PublicUtil.isNotEmpty(reportShopValues)) {
//            evalShopValueReportService.saveBatch(reportShopValues);
//        }
//        return reportShopValues;
//    }
//
//    public BigDecimal getHitValue(Map<Long, EvalGroupIndicatorHitLog> poolHitLogMap, Long evalShopPoolId, ScoreWayEnum scoreWay) {
//        Optional<EvalGroupIndicatorHitLog> hitLog = Optional.ofNullable(poolHitLogMap.get(evalShopPoolId));
//        BigDecimal value = (ScoreWayEnum.LADDER.equals(scoreWay)) ?
//                hitLog.map(EvalGroupIndicatorHitLog::getHitLadderValue).orElse(BigDecimal.ZERO) :
//                hitLog.map(EvalGroupIndicatorHitLog::getHitCommissionValue).orElse(BigDecimal.ZERO);
//        return value;
//    }
//
//    public List<EvalShopReport> saveShopReports(List<EvalShopPool> pools, LocalDate date, Long evalShopRankReportId) {
//        List<EvalShopReport> evalShopReports = new ArrayList<>();
//        for (EvalShopPool pool : pools) {
//            EvalShopReport report = new EvalShopReport();
//            report.setDimension(ReportDimensionEnum.SHOP);
//            report.setEvalShopRankReportId(evalShopRankReportId);
//            report.setShopId(pool.getShopId());
//            report.setShopName(pool.getShopName());
//            report.setDataDate(date);
//            report.setEvalShopPoolId(pool.getId());
//            evalShopReports.add(report);
//        }
//        evalShopReportService.saveBatch(evalShopReports);
//        return evalShopReports;
//    }
//
//    public EvalShopReport convertDim(EvalShopPool pool, LocalDate date, Long evalShopRankReportId) {
//        EvalShopReport report = new EvalShopReport();
//        report.setDimension(ReportDimensionEnum.SHOP);
//        report.setEvalShopRankReportId(evalShopRankReportId);
//        report.setShopId(pool.getShopId());
//        report.setShopName(pool.getShopName());
//        report.setDataDate(date);
//        report.setEvalShopPoolId(pool.getId());
//        return report;
//    }
//
//    public Map<Long, BigDecimal> queryShopOriginValue(EvalGroupIndicatorParam param, List<Long> shopIds, LocalDate dataDate) {
//        final String code = param.getCode();
//        final IndicatorCodeTypeEnum codeType = param.getCodeType();
//        Map<Long, BigDecimal> referValueMap = Maps.newHashMapWithExpectedSize(shopIds.size());
//        //无目标
//        if (IndicatorCodeTypeEnum.INDICATOR.equals(codeType)) {
//            List<IndicatorUserValue> indicatorShopValues = indicatorUserValueService.list(Wrappers.<IndicatorUserValue>lambdaQuery()
//                            .eq(IndicatorUserValue::getIndicatorCode, code)
//                            .eq(IndicatorUserValue::getDataDate, dataDate)
//                            .in(IndicatorUserValue::getShopId, shopIds)
//                            .eq(IndicatorUserValue::getDimensionType, DimensionTypeEnum.SHOP)
//                            .eq(IndicatorUserValue::getYn, Boolean.TRUE)
//            );
//            for (IndicatorUserValue userValue : indicatorShopValues) {
//                referValueMap.put(userValue.getShopId(), commonService.queryIndicatorValueVO(code, userValue,
//                        param.getTargetType(), param.getDataType()));
//            }
//            return referValueMap;
//        }
//        for (Long shopId : shopIds) {
//            EvalIndicatorValue evalIndicatorValue = evalIndicatorValueService.queryLastValue(shopId,
//                    DimensionTypeEnum.SHOP, code, dataDate);
//            referValueMap.put(shopId, commonService.queryEvalIndicatorValueVO(code, evalIndicatorValue,
//                    param.getTargetType(), param.getDataType()));
//        }
//        return referValueMap;
//    }
//
//    public Map<Long, EvalGroupIndicatorTargetHitLog> queryShopTargetValue(Long referId, List<Long> poolIds, LocalDate dataDate) {
//        List<EvalGroupIndicatorTargetHitLog> targetHitLogs = evalGroupIndicatorTargetHitLogService.list(Wrappers.<EvalGroupIndicatorTargetHitLog>lambdaQuery()
//                .eq(EvalGroupIndicatorTargetHitLog::getReferId, referId)
//                .eq(EvalGroupIndicatorTargetHitLog::getTargetType, IndicatorTypeEnum.EXAMINE)
//                .eq(EvalGroupIndicatorTargetHitLog::getDataDate, dataDate)
//                .in(EvalGroupIndicatorTargetHitLog::getPoolId, poolIds)
//                .eq(EvalGroupIndicatorTargetHitLog::getScopeType, EvalScopeEnum.SHOP)
//                .eq(EvalGroupIndicatorTargetHitLog::getYn, Boolean.TRUE)
//        );
//        return targetHitLogs.stream()
//                .collect(Collectors.toMap(EvalGroupIndicatorTargetHitLog::getPoolId, Function.identity(), (v1, v2) -> v1));
//    }
//
//    public void cleanTodayData(LocalDate date) {
//        evalShopRankReportService.remove(Wrappers.<EvalShopRankReport>lambdaUpdate()
//                .eq(EvalShopRankReport::getDataDate, date)
//        );
//        evalShopRankCodeReportService.remove(Wrappers.<EvalShopRankCodeReport>lambdaUpdate()
//                .eq(EvalShopRankCodeReport::getDataDate, date)
//        );
//        evalShopValueReportService.remove(Wrappers.<EvalShopValueReport>lambdaUpdate()
//                .eq(EvalShopValueReport::getDataDate, date)
//        );
//        evalShopReportService.remove(Wrappers.<EvalShopReport>lambdaUpdate()
//                .eq(EvalShopReport::getDataDate, date)
//        );
//    }
//
//    public void cleanRankCurMonthData(Long rankId, LocalDate endDate) {
//        LocalDate startDate = endDate.with(TemporalAdjusters.firstDayOfMonth());
//        List<EvalShopRankReport> rankReports = evalShopRankReportService.list(Wrappers.<EvalShopRankReport>lambdaQuery()
//                .eq(EvalShopRankReport::getRankId, rankId)
//                .ge(EvalShopRankReport::getDataDate, startDate)
//                .le(EvalShopRankReport::getDataDate, endDate)
//        );
//        if (PublicUtil.isEmpty(rankReports)) {
//            return;
//        }
//        List<Long> rankReportIds = rankReports.stream().map(EvalShopRankReport::getId).collect(Collectors.toList());
//        evalShopRankReportService.removeByIds(rankReportIds);
//        evalShopRankCodeReportService.remove(Wrappers.<EvalShopRankCodeReport>lambdaUpdate()
//                .in(EvalShopRankCodeReport::getEvalShopRankReportId, rankReportIds)
//        );
//
//        List<EvalShopReport> userReports = evalShopReportService.list(Wrappers.<EvalShopReport>lambdaQuery()
//                .in(EvalShopReport::getEvalShopRankReportId, rankReportIds)
//        );
//        List<Long> userReportIds = userReports.stream().map(EvalShopReport::getId).collect(Collectors.toList());
//        if (PublicUtil.isEmpty(userReportIds)) {
//            return;
//        }
//        evalShopReportService.removeByIds(userReportIds);
//        evalShopValueReportService.remove(Wrappers.<EvalShopValueReport>lambdaUpdate()
//                .in(EvalShopValueReport::getEvalShopReportId, userReportIds)
//        );
//    }
//
//    public List<EvalShopRankCodeReport> saveRankCodeReports(List<EvalGroupIndicator> evalGroupIndicators, Long reportRankId, LocalDate date) {
//        Set<String> codes = new HashSet<>();
//        List<EvalShopRankCodeReport> rankCodes = new ArrayList<>();
//        AtomicInteger order = new AtomicInteger();
//        for (EvalGroupIndicator groupIndicator : evalGroupIndicators) {
//            //一个编码只能展示一次
//            if (! codes.add(groupIndicator.getCode())) {
//                continue;
//            }
//            rankCodes.add(this.convertRankCodeDB(groupIndicator, date, reportRankId, order));
//
//            //组合指标
//            if (PublicUtil.isNotEmpty(groupIndicator.getCodeType()) && IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(groupIndicator.getCodeType())) {
//                //子指标
//                List<EvalGroupIndicatorParam> params = evalGroupIndicatorParamService.list(Wrappers.<EvalGroupIndicatorParam>lambdaQuery()
//                        .eq(EvalGroupIndicatorParam::getEvalGroupIndicatorId, groupIndicator.getId())
//                        .eq(EvalGroupIndicatorParam::getYn, Boolean.TRUE)
//                );
//                for (EvalGroupIndicatorParam indicatorParam : params) {
//                    rankCodes.add(this.convertParamToRankCodeDB(indicatorParam, date, reportRankId, order, groupIndicator));
//                }
//            }
//        }
//        evalShopRankCodeReportService.saveBatch(rankCodes);
//        return rankCodes;
//    }
//
//    public Map<Long, List<EvalGroupRank>> queryShopEffectRank(LocalDate date) {
//        List<EvalGroupRank> evalGroupRanks = evalGroupRankService.list(Wrappers.<EvalGroupRank>lambdaQuery()
//                .ge(EvalGroupRank::getOverTime, date)
//                .eq(EvalGroupRank::getScopeType, EvalScopeEnum.SHOP)
//                .eq(EvalGroupRank::getYn, Boolean.TRUE)
//        );
//        return evalGroupRanks.stream().collect(Collectors.groupingBy(EvalGroupRank::getGroupId));
//    }
//
//    public EvalShopRankReport saveRankReport(EvalGroupRank rank, List<Long> evalGroupIds, List<EvalGroup> evalGroups, LocalDate date, Long groupId) {
//        List<Long> shopIds = evalGroups.stream().map(EvalGroup::getShopIds).distinct().collect(ArrayList::new, List::addAll, List::addAll);
//        EvalShopRankReport reportRank = new EvalShopRankReport();
//        reportRank.setRankId(rank.getId());
//        reportRank.setRankName(rank.getName());
//        reportRank.setEvalGroupIds(evalGroupIds);
//        reportRank.setShopIds(shopIds);
//        reportRank.setDataDate(date);
//        reportRank.setMonthly(YearMonth.from(date));
//        reportRank.setGroupId(groupId);
//        evalShopRankReportService.save(reportRank);
//        return reportRank;
//    }
//
//    public EvalShopValueReport convertDB(Long evalShopReportId, EvalGroupIndicator groupIndicator, BigDecimal value,
//                                         LocalDate date, List<EvalGroupIndicatorParam> params) {
//        EvalShopValueReport userValue = new EvalShopValueReport();
//        userValue.setEvalShopReportId(evalShopReportId);
//        userValue.setCode(groupIndicator.getCode());
//        userValue.setCodeType(groupIndicator.getCodeType());
//        userValue.setValue(Optional.ofNullable(value).orElse(BigDecimal.ZERO));
//        userValue.setDataDate(date);
//        if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(groupIndicator.getCodeType())) {
//            userValue.setValue(userValue.getValue().multiply(Constant.ONE_HUNDRED));
//        } else {
//            Optional<EvalGroupIndicatorParam> paramOptional = params.stream().findFirst();
//            if (paramOptional.isPresent()) {
//                DataTypeEnum dataType = paramOptional.get().getDataType();
//                if (DataTypeEnum.RATIO.equals(dataType)) {
//                    userValue.setValue(userValue.getValue().multiply(Constant.ONE_HUNDRED));
//                }
//            }
//        }
//        return userValue;
//    }
//
//    public EvalShopValueReport convertParamDB(Long evalShopReportId, EvalGroupIndicatorParam param, BigDecimal value,
//                                              LocalDate date) {
//        EvalShopValueReport userValue = new EvalShopValueReport();
//        userValue.setEvalShopReportId(evalShopReportId);
//        userValue.setCode(param.getCode());
//        userValue.setCodeType(param.getCodeType());
//        userValue.setValue(Optional.ofNullable(value).orElse(BigDecimal.ZERO));
//        userValue.setDataDate(date);
//        return userValue;
//    }
//
//    public EvalShopRankCodeReport convertRankCodeDB(EvalGroupIndicator groupIndicator, LocalDate date, Long reportRankId,
//                                                    AtomicInteger order) {
//        EvalShopRankCodeReport rankCode = new EvalShopRankCodeReport();
//        rankCode.setEvalShopRankReportId(reportRankId);
//        rankCode.setEvalGroupIndicatorId(groupIndicator.getId());
//        rankCode.setCode(groupIndicator.getCode());
//        rankCode.setCodeType(groupIndicator.getCodeType());
////        String indicatorName = (ScoreWayEnum.LADDER.equals(groupIndicator.getScoreWay())) ?
////                groupIndicator.getLadderParamAlias() : groupIndicator.getCommissionParamAlias();
////        if (PublicUtil.isEmpty(indicatorName)) {
////            indicatorName = groupIndicator.getName();
////        }
////        rankCode.setIndicatorName(indicatorName);
//        rankCode.setIndicatorName(groupIndicator.getName());
//        rankCode.setDataDate(date);
//        rankCode.setOrderNum(order.incrementAndGet());
//        return rankCode;
//    }
//
//    public EvalShopRankCodeReport convertParamToRankCodeDB(EvalGroupIndicatorParam indicatorParam, LocalDate date, Long reportRankId,
//                                                           AtomicInteger order, EvalGroupIndicator groupIndicator) {
//        EvalShopRankCodeReport rankCode = new EvalShopRankCodeReport();
//        rankCode.setEvalShopRankReportId(reportRankId);
//        rankCode.setEvalGroupIndicatorId(indicatorParam.getEvalGroupIndicatorId());
//        rankCode.setCode(indicatorParam.getCode());
//        rankCode.setCodeType(indicatorParam.getCodeType());
//        rankCode.setIndicatorName(indicatorParam.getName());
//        if (PublicUtil.isNotEmpty(groupIndicator.getCode())) {
//            rankCode.setParentCode(groupIndicator.getCode());
//        }
//        rankCode.setDataDate(date);
//        rankCode.setOrderNum(order.incrementAndGet());
//        return rankCode;
//    }
//
//
//}
