package cn.fw.morax.service.biz.calculator.eval.kpi;

import cn.fw.morax.domain.bo.eval.EvalGroupIndicatorBO;
import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.db.eval.EvalGroupIndicator;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorParam;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorPrecondition;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : NormalCalculator
 * @description : 通用计算器
 * @date : 2022-04-18 11:46
 */
@Component
@Slf4j
public class EvalNormalCalculator extends EvalKpiBaseCalculator {
    @Override
    public ScoreWayEnum getCalMethod() {
        return ScoreWayEnum.NORMAL;
    }

    @Override
    public BigDecimal calculate(EvalGroupUserShop userShop, EvalGroupIndicator bo) {
        final BigDecimal baseScore = BigDecimal.valueOf(bo.getBaseScore());
        final Long evalGroupIndicatorId = bo.getId();
        List<EvalGroupIndicatorPrecondition> preconditions = queryPrecondition(evalGroupIndicatorId);
        BigDecimal coefficient = calcCoefficient(userShop, preconditions);
        List<EvalGroupIndicatorParam> params = queryParam(evalGroupIndicatorId);
        BigDecimal finalIndValue = calculateFinalIndValue(params, userShop, evalGroupIndicatorId);

        saveParamHitLog(evalGroupIndicatorId, userShop, hitLog -> {
            hitLog.setHitLadderValue(BigDecimal.ZERO);
            hitLog.setHitCommissionValue(finalIndValue);
        });
        BigDecimal multiply = baseScore.multiply(finalIndValue).multiply(coefficient);
        return multiply.divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
    }
}
