package cn.fw.morax.service.biz.calculator.kpi;

import cn.fw.morax.domain.bo.kpi.KpiIndicatorBO;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorLadders;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorParam;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorPrecondition;
import cn.fw.morax.domain.db.kpi.KpiGroupUser;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import cn.fw.morax.service.data.kpi.KpiGroupIndicatorLaddersService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : LadderCalculator
 * @description : 阶梯计算器
 * @date : 2022-04-18 11:47
 */
@Component
@Slf4j
public class LadderCalculator extends KpiAbsBaseCalculator {
    private final KpiGroupIndicatorLaddersService kpiGroupIndicatorLaddersService;

    @Autowired
    public LadderCalculator(final KpiGroupIndicatorLaddersService kpiGroupIndicatorLaddersService) {
        this.kpiGroupIndicatorLaddersService = kpiGroupIndicatorLaddersService;
    }

    @Override
    public ScoreWayEnum getCalMethod() {
        return ScoreWayEnum.LADDER;
    }

    @Override
    public BigDecimal calculate(KpiGroupUser user, KpiIndicatorBO bo) {
        final BigDecimal baseScore = BigDecimal.valueOf(bo.getBaseScore());
        final Long kpiGroupIndicatorId = bo.getKpiGroupIndicatorId();
        List<KpiGroupIndicatorPrecondition> list = queryPrecondition(kpiGroupIndicatorId, bo.getKpiGroupId());
        BigDecimal coefficient = calcCoefficient(user, list);
        return calcLadderScore(kpiGroupIndicatorId, bo.getKpiGroupId(), baseScore, coefficient, user);
    }

    /**
     * 通过配置条件计算绩效分
     *
     * @param kpiGroupIndicatorId
     * @param kpiGroupId
     * @param baseScore
     * @param coefficient
     * @param user
     * @return
     */
    private BigDecimal calcLadderScore(Long kpiGroupIndicatorId, Long kpiGroupId, BigDecimal baseScore, BigDecimal coefficient, KpiGroupUser user) {
        List<KpiGroupIndicatorParam> paramList = queryParam(kpiGroupIndicatorId, kpiGroupId);
        BigDecimal finalIndValue = calculateFinalIndValue(paramList, user, kpiGroupIndicatorId);
        BigDecimal score = ladderScore(kpiGroupIndicatorId, finalIndValue, user);
        if (baseScore.compareTo(score) < 0) {
            score = baseScore;
        }
        return score.multiply(coefficient).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
    }

    private BigDecimal ladderScore(Long kpiGroupIndicatorId, BigDecimal rate, KpiGroupUser user) {
        KpiGroupIndicatorLadders ladder = kpiGroupIndicatorLaddersService.getOne(Wrappers.<KpiGroupIndicatorLadders>lambdaQuery()
                        .eq(KpiGroupIndicatorLadders::getKpiGroupIndicatorId, kpiGroupIndicatorId)
                        .eq(KpiGroupIndicatorLadders::getYn, Boolean.TRUE)
                        .gt(KpiGroupIndicatorLadders::getUpper, rate)
                        .le(KpiGroupIndicatorLadders::getLower, rate),
                Boolean.FALSE
        );
        if (Objects.isNull(ladder)) {
            return BigDecimal.ZERO;
        }
        saveParamHitLog(kpiGroupIndicatorId, ladder.getId(), rate, user);
        BigDecimal standardScore = BigDecimal.valueOf(ladder.getStandardScore());
        return standardScore.multiply(rate);
    }
}
