package cn.fw.morax.service.biz.eval;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.SettingDraft;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.dto.eval.*;
import cn.fw.morax.domain.dto.query.EvalGroupQueryDTO;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.domain.vo.eval.*;
import cn.fw.morax.service.biz.ApprovalBizService;
import cn.fw.morax.service.data.SettingDraftService;
import cn.fw.morax.service.data.eval.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@Service
@Slf4j
@RequiredArgsConstructor
public class EvalGroupBizService {

    private final EvalGroupIndicatorPreconditionLaddersService evalGroupIndicatorPreconditionLaddersService;
    private final EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    private final EvalGroupRewardPreconditionService evalGroupRewardPreconditionService;
    private final EvalGroupIndicatorLaddersService evalGroupIndicatorLaddersService;
    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    private final EvalGroupRewardLaddersService evalGroupRewardLaddersService;
    private final EvalGroupRewardParamService evalGroupRewardParamService;
    private final EvalGroupIndicatorService evalGroupIndicatorService;
    private final EvalGroupRankStageService evalGroupRankStageService;
    private final EvalGroupRewardService evalGroupRewardService;
    private final EvalGroupRewardDimService evalGroupDimService;
    private final EvalGroupDataService evalGroupDataService;
    private final EvalGroupRankService evalGroupRankService;
    private final EvalUserPoolService evalUserPoolService;
    private final EvalShopPoolService evalShopPoolService;
    private final EvalGroupService evalGroupService;
    private final EvalService evalService;
    private final SettingDraftService settingDraftService;
    private final ApprovalBizService approvalBizService;

    /**
     * 装换为目标对象
     *
     * @return
     */
    public void setTargetVos(EvalGroupVO evalGroupVO) {

        if (PublicUtil.isNotEmpty(evalGroupVO.getIndicators())) {
            for (EvalGroupIndicatorVO indicatorDTO : evalGroupVO.getIndicators()) {
                List<EvalGroupIndicatorParamVO> ladderParams = Optional.ofNullable(indicatorDTO.getLadderParams()).orElse(new ArrayList<>());
                List<EvalGroupIndicatorParamVO> commissionParams = Optional.ofNullable(indicatorDTO.getCommissionParams()).orElse(new ArrayList<>());
                List<EvalGroupIndicatorPreconditionVO> conds = Optional.ofNullable(indicatorDTO.getConds()).orElse(new ArrayList<>());

                List<EvalGroupIndicatorTargetVO> targetVOS = new ArrayList<>();
                for (EvalGroupIndicatorParamVO paramDTO : ladderParams) {
                    if (PublicUtil.isNotEmpty(paramDTO.getTargetType()) && (! TargetTypeEnum.NO.equals(paramDTO.getTargetType()))) {
                        EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramDTO, EvalGroupIndicatorTargetVO.class);
                        targetVOS.add(targetVO);
                    }
                }

                for (EvalGroupIndicatorParamVO paramDTO : commissionParams) {
                    if (PublicUtil.isNotEmpty(paramDTO.getTargetType()) && (! TargetTypeEnum.NO.equals(paramDTO.getTargetType()))) {
                        EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramDTO, EvalGroupIndicatorTargetVO.class);
                        targetVOS.add(targetVO);
                    }
                }

                for (EvalGroupIndicatorPreconditionVO preconditionDTO : conds) {
                    if (PublicUtil.isNotEmpty(preconditionDTO.getTargetType()) && (! TargetTypeEnum.NO.equals(preconditionDTO.getTargetType()))) {
                        EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(preconditionDTO, EvalGroupIndicatorTargetVO.class);
                        targetVOS.add(targetVO);
                    }
                }

                indicatorDTO.setTargets(targetVOS);
            }
        }

//        if (PublicUtil.isNotEmpty(evalGroupVO.getRewards())) {
//            for (EvalGroupRewardVO rewardDTO : evalGroupVO.getRewards()) {
//                List<EvalGroupRewardParamVO> ladderParams = Optional.ofNullable(rewardDTO.getLadderParams()).orElse(new ArrayList<>());
//                List<EvalGroupRewardParamVO> commissionParams = Optional.ofNullable(rewardDTO.getCommissionParams()).orElse(new ArrayList<>());
//                List<EvalGroupRewardPreconditionVO> conds = Optional.ofNullable(rewardDTO.getConds()).orElse(new ArrayList<>());
//
//                List<EvalGroupIndicatorTargetVO> targetVOS = new ArrayList<>();
//                for (EvalGroupRewardParamVO paramDTO : ladderParams) {
//                    if (PublicUtil.isNotEmpty(paramDTO.getTargetType()) && (! TargetTypeEnum.NO.equals(paramDTO.getTargetType()))) {
//                        EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramDTO, EvalGroupIndicatorTargetVO.class);
//                        targetVOS.add(targetVO);
//                    }
//                }
//
//                for (EvalGroupRewardParamVO paramDTO : commissionParams) {
//                    if (PublicUtil.isNotEmpty(paramDTO.getTargetType()) && (! TargetTypeEnum.NO.equals(paramDTO.getTargetType()))) {
//                        EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramDTO, EvalGroupIndicatorTargetVO.class);
//                        targetVOS.add(targetVO);
//                    }
//                }
//
//                for (EvalGroupRewardPreconditionVO preconditionDTO : conds) {
//                    if (PublicUtil.isNotEmpty(preconditionDTO.getTargetType()) && (! TargetTypeEnum.NO.equals(preconditionDTO.getTargetType()))) {
//                        EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(preconditionDTO, EvalGroupIndicatorTargetVO.class);
//                        targetVOS.add(targetVO);
//                    }
//                }
//                rewardDTO.setTargets(targetVOS);
//            }
//        }
    }


    /**
     * 检查时间
     *
     * @param dto
     */
    public void checkTime(EvalGroupDTO dto) {
//        BV.isTrue(dto.getBeginTime().isBefore(dto.getOverTime()), "考评组【" + dto.getName()+ "】生效时间必须在结束时间之前");

//        if (PublicUtil.isNotEmpty(dto.getId())) {
//            List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
//                    .eq(EvalGroup::getEvalId, dto.getId())
//                    .eq(EvalGroup::getYn, Boolean.TRUE)
//            );
//            LocalDate now = LocalDate.now();
//            Optional<EvalGroup> optionalEvalGroup = evalGroups.stream()
//                    .filter(evalGroup -> EvalScopeEnum.SHOP.equals(evalGroup.getScopeType()) && now.equals(evalGroup.getOverTime())).findFirst();
//            if (optionalEvalGroup.isPresent()) {
//                throw new BusinessException("考评组 " + optionalEvalGroup.get().getName() + "已结束,不能编辑");
//            }
//        }
    }



    /**
     * 分页查询
     * @param dto
     * @return
     */
    public AppPage<EvalVO> evalPage(EvalGroupQueryDTO dto) {
        PageData<Eval> pageData = evalService.page(new PageData<Eval>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<Eval>lambdaQuery()
                        .eq(Eval::getGroupId, dto.getGroupId())
                        .eq(PublicUtil.isNotEmpty(dto.getStatus()), Eval::getStatus, dto.getStatus())
                        .eq(Eval::getYn, Boolean.TRUE)
                        .last("ORDER BY FIELD(`status`,3,1,2,4)")
        );

//        Map<Long, String> approvalRecordMap = new HashMap<>();
//        if (PublicUtil.isNotEmpty(pageData.getRecords())) {
//            List<Long> evalIds = pageData.getRecords().stream().map(Eval::getId).collect(Collectors.toList());
//            List<ApprovalRecord> approvalRecords = approvalRecordService.list(Wrappers.<ApprovalRecord>lambdaQuery()
//                    .in(ApprovalRecord::getDataId, kpiGroupIds)
//                    .eq(ApprovalRecord::getApprovalType, ApprovalTypeEnum.KPI)
//                    .eq(ApprovalRecord::getYn, Boolean.TRUE));
//
//            approvalRecordMap = approvalRecords.stream().collect(Collectors.toMap(ApprovalRecord::getDataId, ApprovalRecord::getApprovalNo));
//        }
//        Map<Long, String> finalApprovalRecordMap = approvalRecordMap;
        return PublicUtil.toPage(pageData, eval -> {
            EvalVO evalVO = PublicUtil.copy(eval, EvalVO.class);
//            kpiGroupVo.setApprovalNo(finalApprovalRecordMap.getOrDefault(kpiGroup.getId(), ""));
//            kpiGroupVo.setBeginTime(kpiGroup.getBeginTime());
//            kpiGroupVo.setOverTime(kpiGroup.getOverTime());
            return evalVO;
        });
    }


    /**
     * 考评组排名组合详情
     *
     * @param id
     * @return
     */
    public EvalGroupRankVO evalGroupRankDetail(Long id) {
        EvalGroupRank evalGroupRank = evalGroupRankService.getById(id);
        BV.notNull(evalGroupRank, "考评排名组为空，请重试");
        EvalGroupRankVO evalGroupRankVO = PublicUtil.copy(evalGroupRank, EvalGroupRankVO.class);

        List<EvalGroupRankStage> stages = evalGroupRankStageService.list(Wrappers.<EvalGroupRankStage>lambdaQuery()
                .in(EvalGroupRankStage::getEvalGroupRankId, evalGroupRankVO.getId())
                .eq(EvalGroupRankStage::getYn, Boolean.TRUE)
        );
        List<EvalGroupRankStageVO> stageVOS = PublicUtil.copyList(stages, EvalGroupRankStageVO.class);
        evalGroupRankVO.setStages(stageVOS);

        List<EvalGroup> evalGroups = evalGroupService.getEvalGroups(evalGroupRankVO.getId());
        List<EvalGroupVO> evalGroupVOS = PublicUtil.copyList(evalGroups, EvalGroupVO.class);
        Map<Long, List<EvalGroupVO>> evalGroupStageMap = evalGroupVOS.stream().collect(Collectors.groupingBy(EvalGroupVO::getEvalGroupRankStageId));
        for (EvalGroupRankStageVO stageVO : stageVOS) {
            stageVO.setEvalGroups(evalGroupStageMap.getOrDefault(stageVO.getId(), new ArrayList<>()));
            for (EvalGroupVO evalGroupVO : stageVO.getEvalGroups()) {
                this.setEvalGroupIndicator(evalGroupVO);
                this.setEvalGroupReward(evalGroupVO, EvalScopeEnum.STAFF);
                this.setEvalGroupReward(evalGroupVO, EvalScopeEnum.SHOP);
                this.setTargetVos(evalGroupVO);
            }
        }
        return evalGroupRankVO;
    }
    /**
     * 考评排名组详情
     *
     * @param
     * @return
     */
    public List<EvalGroupVO> rankEvalGroupDetail(EvalGroupRankQueryDto dto) {
        List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
                .in(EvalGroup::getId, dto.getEvalGroupIds())
                .eq(EvalGroup::getYn, Boolean.TRUE)
        );
        List<EvalGroupVO> evalGroupVOs = PublicUtil.copyList(evalGroups, EvalGroupVO.class);
        for (EvalGroupVO evalGroupVO : evalGroupVOs) {
            this.setEvalGroupIndicator(evalGroupVO);
            this.setEvalGroupReward(evalGroupVO, EvalScopeEnum.SHOP);
            this.setEvalGroupReward(evalGroupVO, EvalScopeEnum.STAFF);
            this.setTargetVos(evalGroupVO);
        }
        return evalGroupVOs;
    }


    /**
     * 设置奖励
     *
     * @param evalGroupVO
     */
    public void setEvalGroupReward(EvalGroupVO evalGroupVO, EvalScopeEnum scopeType) {
        List<EvalGroupRewardDimVO> rewardDimVOS = new ArrayList<>();
        if (EvalScopeEnum.SHOP.equals(scopeType)) {
            evalGroupVO.setShopRewardDims(rewardDimVOS);
        } else {
            evalGroupVO.setStaffRewardDims(rewardDimVOS);
        }

        Long evalGroupId = evalGroupVO.getId();
        List<EvalGroupRewardDim> dims = evalGroupDimService.list(Wrappers.<EvalGroupRewardDim>lambdaQuery()
                .in(EvalGroupRewardDim::getEvalGroupId, evalGroupVO.getId())
                .eq(EvalGroupRewardDim::getType, scopeType)
                .eq(EvalGroupRewardDim::getYn, Boolean.TRUE)
        );

        for (EvalGroupRewardDim dim : dims) {
            EvalGroupRewardDimVO dimVO = PublicUtil.copy(dim, EvalGroupRewardDimVO.class);
            rewardDimVOS.add(dimVO);
            Long dimId = dim.getId();
            List<EvalGroupReward> rewards = evalGroupRewardService.list(Wrappers.<EvalGroupReward>lambdaQuery()
                    .eq(EvalGroupReward::getEvalGroupDimId, dimId)
                    .eq(EvalGroupReward::getYn, Boolean.TRUE)
            );
            if (PublicUtil.isNotEmpty(rewards)) {
                List<EvalGroupRewardVO> rewardVOS = PublicUtil.copyList(rewards, EvalGroupRewardVO.class);
                dimVO.setRewards(rewardVOS);

                Map<Long, List<EvalGroupRewardParamVO>> rewardParamMap = this.getEvalGroupRewardParamVO(evalGroupId);
                Map<Long, List<EvalGroupRewardPreconditionVO>> rewardCondMap = this.getEvalGroupRewardPreconditionVO(evalGroupId);

                for (EvalGroupRewardVO reward : rewardVOS) {
                    Long rewardId = reward.getId();

                    List<EvalGroupRewardParamVO> rewardParamVOS = rewardParamMap.getOrDefault(rewardId, new ArrayList<>());
                    Map<ParamTypeEnum, List<EvalGroupRewardParamVO>> paramTypeMap = rewardParamVOS.stream()
                            .collect(Collectors.groupingBy(EvalGroupRewardParamVO::getParamType));
                    reward.setCommissionParams(paramTypeMap.getOrDefault(ParamTypeEnum.COMMISSION, new ArrayList<>()));
                    reward.setLadderParams(paramTypeMap.getOrDefault(ParamTypeEnum.LADDER, new ArrayList<>()));
                    reward.setConds(rewardCondMap.getOrDefault(rewardId, new ArrayList<>()));

                    List<EvalGroupRewardLadders> rewardLadders = evalGroupRewardLaddersService.list(Wrappers.<EvalGroupRewardLadders>lambdaQuery()
                            .eq(EvalGroupRewardLadders::getEvalGroupRewardId, rewardId)
                            .eq(EvalGroupRewardLadders::getYn, Boolean.TRUE)
                            .last("order by rank_order_type ASC, lower ASC")
                    );
                    List<EvalGroupRewardLaddersVO> rewardLaddersVOS = new ArrayList<>();
                    for (EvalGroupRewardLadders rewardLadder : rewardLadders) {
                        EvalGroupRewardLaddersVO rewardLaddersVO = PublicUtil.copy(rewardLadder, EvalGroupRewardLaddersVO.class);
                        rewardLaddersVO.processPercent(reward.getCalMethod(), reward.getRankType(), reward.getLaddersType());
                        rewardLaddersVOS.add(rewardLaddersVO);
                    }
                    reward.setLadders(rewardLaddersVOS);

                    setRewardTargetVos(reward);
                }
            }
        }
    }

    /**
     * 装换为目标对象
     *
     * @return
     */
    public void setRewardTargetVos(EvalGroupRewardVO reward) {

        List<EvalGroupRewardParamVO> ladderParams = reward.getLadderParams();
        List<EvalGroupRewardParamVO> commissionParams = reward.getCommissionParams();
        List<EvalGroupRewardPreconditionVO> conds = reward.getConds();

        List<EvalGroupIndicatorTargetVO> targetVOS = new ArrayList<>();
        for (EvalGroupRewardParamVO paramVO : ladderParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupRewardParamVO paramVO : commissionParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupRewardPreconditionVO preconditionVO : conds) {
            if (PublicUtil.isNotEmpty(preconditionVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(preconditionVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        reward.setTargets(targetVOS);
    }
    /**
     * 装换为目标对象
     *
     * @return
     */
    public void setIndicatorTargetVos(EvalGroupIndicatorVO indicatorVO) {

        List<EvalGroupIndicatorParamVO> ladderParams = indicatorVO.getLadderParams();
        List<EvalGroupIndicatorParamVO> commissionParams = indicatorVO.getCommissionParams();
        List<EvalGroupIndicatorPreconditionVO> conds = indicatorVO.getConds();

        List<EvalGroupIndicatorTargetVO> targetVOS = new ArrayList<>();
        for (EvalGroupIndicatorParamVO paramVO : ladderParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupIndicatorParamVO paramVO : commissionParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupIndicatorPreconditionVO preconditionVO : conds) {
            if (PublicUtil.isNotEmpty(preconditionVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(preconditionVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        indicatorVO.setTargets(targetVOS);
    }


    /**
     * 考评奖惩条件
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupRewardPreconditionVO>> getEvalGroupRewardPreconditionVO(Long evalGroupId) {
        List<EvalGroupRewardPrecondition> rewardPreconditions = evalGroupRewardPreconditionService.list(Wrappers.<EvalGroupRewardPrecondition>lambdaQuery()
                .eq(EvalGroupRewardPrecondition::getEvalGroupId, evalGroupId)
                .eq(EvalGroupRewardPrecondition::getYn, Boolean.TRUE)
        );

        List<EvalGroupRewardPreconditionVO> rewardPreconditionVOS = new ArrayList<>();
        for (EvalGroupRewardPrecondition precondition : rewardPreconditions) {
            EvalGroupRewardPreconditionVO preconditionVO = PublicUtil.copy(precondition, EvalGroupRewardPreconditionVO.class);
            preconditionVO.convertToPercent();
            rewardPreconditionVOS.add(preconditionVO);
        }

        Map<Long, List<EvalGroupRewardPreconditionVO>> rewardCondMap = rewardPreconditionVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupRewardPreconditionVO::getEvalGroupRewardId));

        return rewardCondMap;
    }

    /**
     * 考评奖惩参数
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupRewardParamVO>> getEvalGroupRewardParamVO(Long evalGroupId) {
        List<EvalGroupRewardParam> rewardParams = evalGroupRewardParamService.list(Wrappers.<EvalGroupRewardParam>lambdaQuery()
                .eq(EvalGroupRewardParam::getEvalGroupId, evalGroupId)
                .eq(EvalGroupRewardParam::getYn, Boolean.TRUE)
        );

        List<EvalGroupRewardParamVO> rewardParamVOS = new ArrayList<>();
        for (EvalGroupRewardParam rewardParam : rewardParams) {
            EvalGroupRewardParamVO paramVO = PublicUtil.copy(rewardParam, EvalGroupRewardParamVO.class);
            paramVO.convertToPercent();
            rewardParamVOS.add(paramVO);
        }

        Map<Long, List<EvalGroupRewardParamVO>> rewardParamMap = rewardParamVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupRewardParamVO::getEvalGroupRewardId));

        return rewardParamMap;
    }

    /**
     * 设置考评指标
     *
     * @param evalGroupVO
     */
    public void setEvalGroupIndicator(EvalGroupVO evalGroupVO) {
        Long evalGroupId = evalGroupVO.getId();
        List<EvalGroupIndicator> indicators = evalGroupIndicatorService.list(Wrappers.<EvalGroupIndicator>lambdaQuery()
                .eq(EvalGroupIndicator::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicator::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isNotEmpty(indicators)) {
            List<EvalGroupIndicatorVO> indicatorVOS = PublicUtil.copyList(indicators, EvalGroupIndicatorVO.class);
            evalGroupVO.setIndicators(indicatorVOS);

            Map<Long, List<EvalGroupIndicatorParamVO>> paramMap = this.getEvalGroupIndicatorParamVO(evalGroupId);
            Map<Long, List<EvalGroupIndicatorPreconditionVO>> condMap = this.getEvalGroupIndicatorCondVO(evalGroupId);

            for (EvalGroupIndicatorVO indicatorVO : indicatorVOS) {
                Long kpiGroupIndicatorId = indicatorVO.getId();

                //参数
                List<EvalGroupIndicatorParamVO> indicatorParamVOS = paramMap.getOrDefault(kpiGroupIndicatorId, new ArrayList<>());
                Map<ParamTypeEnum, List<EvalGroupIndicatorParamVO>> paramTypeMap = indicatorParamVOS.stream()
                        .collect(Collectors.groupingBy(EvalGroupIndicatorParamVO::getParamType));
                indicatorVO.setCommissionParams(paramTypeMap.getOrDefault(ParamTypeEnum.COMMISSION, new ArrayList<>()));
                indicatorVO.setLadderParams(paramTypeMap.getOrDefault(ParamTypeEnum.LADDER, new ArrayList<>()));

                //条件
                List<EvalGroupIndicatorPreconditionVO> preconditionVOS = condMap.getOrDefault(kpiGroupIndicatorId, new ArrayList<>());
                for (EvalGroupIndicatorPreconditionVO cond : preconditionVOS) {
                    List<EvalGroupIndicatorPreconditionLadders> condLadders = evalGroupIndicatorPreconditionLaddersService.list(Wrappers.<EvalGroupIndicatorPreconditionLadders>lambdaQuery()
                                    .eq(EvalGroupIndicatorPreconditionLadders::getPreconditionId, cond.getId())
                                    .eq(EvalGroupIndicatorPreconditionLadders::getYn, Boolean.TRUE));

                    List<EvalGroupIndicatorPreconditionLaddersVO> condLaddersVOS = new ArrayList<>();
                    for (EvalGroupIndicatorPreconditionLadders condLadder : condLadders) {
                        EvalGroupIndicatorPreconditionLaddersVO condLaddersVO = PublicUtil.copy(condLadder, EvalGroupIndicatorPreconditionLaddersVO.class);
                        condLaddersVO.convertLaddersToPercent(cond.getTargetType(), cond.getDataType());
                        condLaddersVOS.add(condLaddersVO);
                    }
                    cond.setLadders(condLaddersVOS);
                }
                Collections.sort(preconditionVOS, Comparator.comparingInt(EvalGroupIndicatorPreconditionVO::getSort));
                indicatorVO.setConds(preconditionVOS);

                //设置阶梯
                if (ScoreWayEnum.LADDER.equals(indicatorVO.getScoreWay())) {
                    List<EvalGroupIndicatorLadders> ladders = evalGroupIndicatorLaddersService.list(Wrappers.<EvalGroupIndicatorLadders>lambdaQuery()
                            .in(EvalGroupIndicatorLadders::getEvalGroupIndicatorId, kpiGroupIndicatorId)
                            .eq(EvalGroupIndicatorLadders::getYn, Boolean.TRUE)
                            .orderByAsc(EvalGroupIndicatorLadders::getId)
                    );
                    List<EvalGroupIndicatorLaddersVO> laddersVOS = PublicUtil.copyList(ladders, EvalGroupIndicatorLaddersVO.class);
                    laddersVOS.stream().forEach(l -> l.convertLadderToPercent(indicatorVO.getLaddersType()));
                    indicatorVO.setLadders(laddersVOS);
                }

                setIndicatorTargetVos(indicatorVO);
            }
        }
    }

    /**
     * 考评参数
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupIndicatorParamVO>> getEvalGroupIndicatorParamVO(Long evalGroupId) {
        List<EvalGroupIndicatorParam> params = evalGroupIndicatorParamService.list(Wrappers.<EvalGroupIndicatorParam>lambdaQuery()
                .eq(EvalGroupIndicatorParam::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicatorParam::getYn, Boolean.TRUE)
        );

        List<EvalGroupIndicatorParamVO> paramVOS = new ArrayList<>();
        for (EvalGroupIndicatorParam param : params) {
            EvalGroupIndicatorParamVO paramVO = PublicUtil.copy(param, EvalGroupIndicatorParamVO.class);
            paramVO.processPercent();
            paramVOS.add(paramVO);
        }

        Map<Long, List<EvalGroupIndicatorParamVO>> indicatorParamMap = paramVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupIndicatorParamVO::getEvalGroupIndicatorId));

        return indicatorParamMap;
    }

    /**
     * 获取指标条件
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupIndicatorPreconditionVO>> getEvalGroupIndicatorCondVO(Long evalGroupId) {
        List<EvalGroupIndicatorPrecondition> preconditions = evalGroupIndicatorPreconditionService.list(Wrappers.<EvalGroupIndicatorPrecondition>lambdaQuery()
                .eq(EvalGroupIndicatorPrecondition::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicatorPrecondition::getYn, Boolean.TRUE)
        );

        List<EvalGroupIndicatorPreconditionVO> preconditionVOS = new ArrayList<>();
        for (EvalGroupIndicatorPrecondition precondition : preconditions) {
            EvalGroupIndicatorPreconditionVO preconditionVO = PublicUtil.copy(precondition, EvalGroupIndicatorPreconditionVO.class);
            preconditionVO.processPercent();
            preconditionVOS.add(preconditionVO);
        }

        Map<Long, List<EvalGroupIndicatorPreconditionVO>> indicatorCondMap = preconditionVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupIndicatorPreconditionVO::getEvalGroupIndicatorId));

        return indicatorCondMap;
    }


    /**
     * 审批考评组
     *
     * @param approvalRecord
     */
    @Transactional(rollbackFor = Exception.class)
    public void approvalEvalConfig(ApprovalRecord approvalRecord, SettingDraft settingDraft) {
        log.info("收到岗位最低收入补偿审批信息：{}", JSON.toJSONString(approvalRecord));
        EvalGroupRankDTO evalGroupRankDto = JSON.parseObject(settingDraft.getContent(), EvalGroupRankDTO.class);
        EvalGroupRank evalGroupRank = evalGroupDataService.saveEvalGroupRank(evalGroupRankDto);
        for (EvalGroupRankStageDTO evalGroupRankStageDTO : evalGroupRankDto.getStages()) {
            EvalGroupRankStage evalGroupRankStage = evalGroupDataService.saveEvalStage(evalGroupRankStageDTO, evalGroupRank);
            for (EvalGroupDTO evalGroupDTO : evalGroupRankStageDTO.getEvalGroups()) {
                EvalGroup evalGroup = evalGroupDataService.saveEvalGroup(evalGroupDTO, evalGroupRank, evalGroupRankStage);
                evalGroupDataService.saveEvalGroupIndicator(evalGroup, evalGroupDTO.getIndicators());
                evalGroupDataService.saveEvalGroupReward(evalGroup, evalGroupDTO.getStaffRewardDims(), EvalScopeEnum.STAFF);
                evalGroupDataService.saveEvalGroupReward(evalGroup, evalGroupDTO.getShopRewardDims(), EvalScopeEnum.SHOP);
            }
        }
        settingDraftService.update(Wrappers.<SettingDraft>lambdaUpdate()
                .set(SettingDraft::getUnionId, evalGroupRank.getId())
                .set(SettingDraft::getUpdateTime, new Date())
                .eq(SettingDraft::getId, settingDraft.getId())
        );
    }

//    @Transactional(rollbackFor = Exception.class)
//    public void updateRankOverTime(EvalGroupDTO evalGroupDTO) {
//        if (PublicUtil.isNotEmpty(evalGroupDTO.getEgc())) {
//            EvalGroupRank evalGroupRank = evalGroupRankService.getOne(Wrappers.<EvalGroupRank>lambdaQuery()
//                    .eq(EvalGroupRank::getYn, Boolean.TRUE)
//                    .apply(MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, egcs)", evalGroupDTO.getEgc()))
//            , Boolean.FALSE);
//            if (PublicUtil.isNotEmpty(evalGroupRank) && evalGroupDTO.getOverTime().isAfter(evalGroupRank.getOverTime())) {
//                evalGroupRank.setOverTime(evalGroupDTO.getOverTime());
//                evalGroupRankService.updateById(evalGroupRank);
//            }
//        }
//    }

    /**
     * 禁用考评
     *
     * @param id
     * @param currentUser
     */
    public void disableEval(Long id, LoginAuthBean currentUser) {
        Eval eval = evalService.getById(id);
        BV.notNull(eval, "考评配置不存在，请重试");

        Boolean statusCheck = SettingStatusEnum.BE_EFFECTIVE.equals(eval.getStatus()) ||
                SettingStatusEnum.EFFECTIVE.equals(eval.getStatus());
        BV.isTrue(statusCheck, "只有生效中、待生效考评配置能禁用");

        log.info("禁用绩效组，id：{},操作人：{}", id, currentUser);

        evalService.update(Wrappers.<Eval>lambdaUpdate()
                .set(Eval::getUpdateTime, new Date())
                .set(Eval::getYn, Boolean.FALSE)
                .eq(Eval::getId, eval.getId())
        );

        if (SettingStatusEnum.EFFECTIVE.equals(eval.getStatus())) {
            evalUserPoolService.update(Wrappers.<EvalUserPool>lambdaUpdate()
                    .set(EvalUserPool::getUpdateTime, new Date())
                    .set(EvalUserPool::getYn, Boolean.FALSE)
                    .eq(EvalUserPool::getId, id)
                    .eq(EvalUserPool::getYn, Boolean.TRUE)
            );
            evalShopPoolService.update(Wrappers.<EvalShopPool>lambdaUpdate()
                    .set(EvalShopPool::getUpdateTime, new Date())
                    .set(EvalShopPool::getYn, Boolean.FALSE)
                    .eq(EvalShopPool::getId, id)
                    .eq(EvalShopPool::getYn, Boolean.TRUE)
            );
        }

    }

}
