package cn.fw.morax.service.biz.eval;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.dto.eval.*;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.service.data.SettingDraftService;
import cn.fw.morax.service.data.eval.*;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class EvalGroupDataService {

    private final EvalGroupIndicatorPreconditionLaddersService evalGroupIndicatorPreconditionLaddersService;
    private final EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    private final EvalGroupRewardPreconditionService evalGroupRewardPreconditionService;
    private final EvalGroupIndicatorLaddersService evalGroupIndicatorLaddersService;
    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    private final EvalGroupRewardLaddersService evalGroupRewardLaddersService;
    private final EvalGroupRewardParamService evalGroupRewardParamService;
    private final EvalGroupIndicatorService evalGroupIndicatorService;
    private final EvalGroupRewardService evalGroupRewardService;
    private final EvalGroupRewardDimService evalGroupDimService;
    private final EvalGroupRankService evalGroupRankService;
    private final EvalUserPoolService evalUserPoolService;
    private final EvalShopPoolService evalShopPoolService;
    private final EvalGroupService evalGroupService;
    private final EvalGroupRankStageService evalGroupRankStageService;
    private final EvalService evalService;
    private final SettingDraftService settingDraftService;

    /**
     * 保存考评组
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public EvalGroupRank saveEvalGroupRank(EvalGroupRankDTO dto) {
        EvalGroupRank evalGroupRank = PublicUtil.copy(dto, EvalGroupRank.class);
        evalGroupRank.setId(null);
        evalGroupRank.setStatus(SettingStatusEnum.EFFECTIVE);

        evalGroupRankService.save(evalGroupRank);
        //之前的考评 删除
        if (PublicUtil.isNotEmpty(dto.getId())) {
            evalGroupRankService.update(Wrappers.<EvalGroupRank>lambdaUpdate()
                    .eq(EvalGroupRank::getId, dto.getId())
                    .set(EvalGroupRank::getStatus, SettingStatusEnum.INEFFECTIVE)
                    .set(EvalGroupRank::getUpdateTime, new Date())
                    .set(EvalGroupRank::getYn, Boolean.FALSE)
            );
            evalGroupRankStageService.update(Wrappers.<EvalGroupRankStage>lambdaUpdate()
                    .eq(EvalGroupRankStage::getId, dto.getId())
                    .set(EvalGroupRankStage::getUpdateTime, new Date())
                    .set(EvalGroupRankStage::getYn, Boolean.FALSE)
            );
            List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
                    .eq(EvalGroup::getEvalGroupRankId, dto.getId())
                    .eq(EvalGroup::getYn, Boolean.TRUE)
            );
            List<Long> evalGroupIds = evalGroups.stream().map(EvalGroup::getId).collect(Collectors.toList());
            evalGroupService.update(Wrappers.<EvalGroup>lambdaUpdate()
                    .in(EvalGroup::getId, evalGroupIds)
                    .set(EvalGroup::getUpdateTime, new Date())
                    .set(EvalGroup::getYn, Boolean.FALSE)
            );
            evalUserPoolService.update(Wrappers.<EvalUserPool>lambdaUpdate()
                    .in(EvalUserPool::getEvalGroupId, evalGroupIds)
                    .set(EvalUserPool::getYn, Boolean.FALSE)
                    .set(EvalUserPool::getUpdateTime, new Date())
            );
            evalShopPoolService.update(Wrappers.<EvalShopPool>lambdaUpdate()
                    .in(EvalShopPool::getEvalGroupId, evalGroupIds)
                    .set(EvalShopPool::getYn, Boolean.FALSE)
                    .set(EvalShopPool::getUpdateTime, new Date())
            );
        }
        return evalGroupRank;
    }

    /**
     * 保存考评阶段
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public EvalGroupRankStage saveEvalStage(EvalGroupRankStageDTO dto, EvalGroupRank evalGroupRank) {
        EvalGroupRankStage evalGroupRankStage = PublicUtil.copy(dto, EvalGroupRankStage.class);
        evalGroupRankStage.setEvalGroupRankId(evalGroupRank.getId());
        evalGroupRankStageService.save(evalGroupRankStage);
        return evalGroupRankStage;
    }

    /**
     * 保存考评组
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public EvalGroup saveEvalGroup(EvalGroupDTO dto, EvalGroupRank evalGroupRank, EvalGroupRankStage evalGroupRankStage) {
        EvalGroup evalGroup = PublicUtil.copy(dto, EvalGroup.class);
        evalGroup.setIndicatorNum(dto.getIndicators().size());
        evalGroup.setEgc(PublicUtil.getUUID());
        evalGroup.setPostIds(dto.getPostIds());
        evalGroup.setPostNames(dto.getPostNames());
        evalGroup.setGroupId(evalGroupRank.getGroupId());
        evalGroup.setEvalGroupRankId(evalGroupRank.getId());
        evalGroup.setEvalGroupRankStageId(evalGroupRankStage.getId());
//        evalGroup.setStatus(SettingStatusEnum.APPROVING);
        evalGroupService.save(evalGroup);
        return evalGroup;
    }

    /**
     * 保存考评组指标
     *
     * @param evalGroup
     * @param indicatorDTOS
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveEvalGroupIndicator(EvalGroup evalGroup, List<EvalGroupIndicatorDTO> indicatorDTOS) {
        LinkedList<EvalGroupIndicatorParam> indicatorParams = Lists.newLinkedList();
        LinkedList<EvalGroupIndicatorPreconditionLadders> preconditionLadders = Lists.newLinkedList();
        LinkedList<EvalGroupIndicatorLadders> indicatorLadders = Lists.newLinkedList();
        Long evalGroupId = evalGroup.getId();
        Long indicatorId = null;
        for (EvalGroupIndicatorDTO indicatorDto : indicatorDTOS) {
            EvalGroupIndicator indicator = PublicUtil.copy(indicatorDto, EvalGroupIndicator.class);
            indicator.setEvalGroupId(evalGroupId);

            evalGroupIndicatorService.save(indicator);
            indicatorId = indicator.getId();

            //参数
            for (EvalGroupIndicatorParamDTO paramDTO : Optional.ofNullable(indicatorDto.getCommissionParams()).orElse(new ArrayList<>())) {
                EvalGroupIndicatorParam param = PublicUtil.copy(paramDTO, EvalGroupIndicatorParam.class);
                param.setEvalGroupId(evalGroupId);
                param.setEvalGroupIndicatorId(indicatorId);
                param.setParamType(ParamTypeEnum.COMMISSION);
                indicatorParams.add(param);
            }

            for (EvalGroupIndicatorParamDTO paramDTO : Optional.ofNullable(indicatorDto.getLadderParams()).orElse(new ArrayList<>())) {
                EvalGroupIndicatorParam param = PublicUtil.copy(paramDTO, EvalGroupIndicatorParam.class);
                param.setEvalGroupId(evalGroupId);
                param.setEvalGroupIndicatorId(indicatorId);
                param.setParamType(ParamTypeEnum.LADDER);
                indicatorParams.add(param);
            }

            for (EvalGroupIndicatorPreconditionDTO preconditionDTO : Optional.ofNullable(indicatorDto.getConds()).orElse(new ArrayList<>())) {
                EvalGroupIndicatorPrecondition cond = PublicUtil.copy(preconditionDTO, EvalGroupIndicatorPrecondition.class);
                cond.setEvalGroupId(evalGroupId);
                cond.setEvalGroupIndicatorId(indicatorId);
                //转换小数
                cond.dividePercent();
                evalGroupIndicatorPreconditionService.save(cond);
                for (EvalGroupIndicatorPreconditionLaddersDTO preconditionLaddersDTO : preconditionDTO.getLadders()) {
                    EvalGroupIndicatorPreconditionLadders condLadder = PublicUtil.copy(preconditionLaddersDTO, EvalGroupIndicatorPreconditionLadders.class);
                    condLadder.setPreconditionId(cond.getId());

                    condLadder.dividePercent(cond.getTargetType(), preconditionDTO.getDataType());
                    preconditionLadders.add(condLadder);
                }
            }

            if (ScoreWayEnum.LADDER.equals(indicatorDto.getScoreWay())) {
                for (EvalGroupIndicatorLaddersDTO laddersDTO : Optional.ofNullable(indicatorDto.getLadders()).orElse(new ArrayList<>())) {
                    EvalGroupIndicatorLadders ladder = PublicUtil.copy(laddersDTO, EvalGroupIndicatorLadders.class);
                    ladder.setEvalGroupIndicatorId(indicatorId);
                    //转换小数
                    ladder.dividePercent(indicatorDto.getLaddersType());
                    indicatorLadders.add(ladder);
                }
            }
        }

        //处理百分数
        for (EvalGroupIndicatorParam param : indicatorParams) {
            param.dividePercent();
        }

        if (PublicUtil.isNotEmpty(indicatorParams)) {
            evalGroupIndicatorParamService.saveBatch(indicatorParams);
        }
        if (PublicUtil.isNotEmpty(preconditionLadders)) {
            evalGroupIndicatorPreconditionLaddersService.saveBatch(preconditionLadders);
        }
        if (PublicUtil.isNotEmpty(indicatorLadders)) {
            evalGroupIndicatorLaddersService.saveBatch(indicatorLadders);
        }
    }

    /**
     * 保存考评组奖励
     *
     * @param evalGroup
     * @param dims
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveEvalGroupReward(EvalGroup evalGroup, List<EvalGroupRewardDimDTO> dims, EvalScopeEnum type) {
        Long evalGroupId = evalGroup.getId();
        Long evalGroupRankStageId = evalGroup.getEvalGroupRankStageId();
        LinkedList<EvalGroupRewardParam> rewardParams = Lists.newLinkedList();
        LinkedList<EvalGroupRewardLadders> rewardLadders = Lists.newLinkedList();
        LinkedList<EvalGroupRewardPrecondition> rewardPreconditions = Lists.newLinkedList();
        for (EvalGroupRewardDimDTO dimDTO : dims) {
            EvalGroupRewardDim dim = PublicUtil.copy(dimDTO, EvalGroupRewardDim.class);
            dim.setType(type);
            dim.setEvalGroupId(evalGroupId);
            dim.setEvalGroupRankStageId(evalGroupRankStageId);
            evalGroupDimService.save(dim);
            final Long dimId = dim.getId();
            for (EvalGroupRewardDTO rewardDTO : dimDTO.getRewards()) {
                EvalGroupReward reward = PublicUtil.copy(rewardDTO, EvalGroupReward.class);
                reward.setEvalGroupId(evalGroupId);
                reward.setEvalGroupDimId(dimId);
                reward.setEvalGroupRankStageId(evalGroupRankStageId);
                evalGroupRewardService.save(reward);
                Long rewardId = reward.getId();
                //没有配置计算方式
                if (PublicUtil.isEmpty(rewardDTO.getCalMethod())) {
                    continue;
                }

                for (EvalGroupRewardParamDTO rewardParamDTO : Optional.ofNullable(rewardDTO.getLadderParams()).orElse(new ArrayList<>())) {
                    EvalGroupRewardParam rewardParam = PublicUtil.copy(rewardParamDTO, EvalGroupRewardParam.class);
                    rewardParam.setEvalGroupId(evalGroupId);
                    rewardParam.setEvalGroupRewardId(rewardId);
                    rewardParam.setParamType(ParamTypeEnum.LADDER);
                    rewardParams.add(rewardParam);
                }

                for (EvalGroupRewardParamDTO rewardParamDTO : Optional.ofNullable(rewardDTO.getCommissionParams()).orElse(new ArrayList<>())) {
                    EvalGroupRewardParam rewardParam = PublicUtil.copy(rewardParamDTO, EvalGroupRewardParam.class);
                    rewardParam.setEvalGroupId(evalGroupId);
                    rewardParam.setEvalGroupRewardId(rewardId);
                    rewardParam.setParamType(ParamTypeEnum.COMMISSION);
                    rewardParams.add(rewardParam);
                }

                for (EvalGroupRewardLaddersDTO rewardLaddersDTO : Optional.ofNullable(rewardDTO.getLadders()).orElse(new ArrayList<>())) {
                    EvalGroupRewardLadders rewardLadder = this.convertRewardLaddersDB(rewardDTO, rewardLaddersDTO);
                    rewardLadder.setEvalGroupId(evalGroupId);
                    rewardLadder.setEvalGroupRewardId(rewardId);
                    rewardLadders.add(rewardLadder);
                }

                for (EvalGroupRewardPreconditionDTO rewardPreconditionDTO : Optional.ofNullable(rewardDTO.getConds()).orElse(new ArrayList<>())) {
                    EvalGroupRewardPrecondition rewardPrecondition = PublicUtil.copy(rewardPreconditionDTO, EvalGroupRewardPrecondition.class);
                    rewardPrecondition.setEvalGroupId(evalGroupId);
                    rewardPrecondition.setEvalGroupRewardId(rewardId);
                    rewardPreconditions.add(rewardPrecondition);
                }
            }
        }

        //处理百分数
        for (EvalGroupRewardParam param : rewardParams) {
            param.dividePercent();
        }
        for (EvalGroupRewardPrecondition precondition : rewardPreconditions) {
            precondition.dividePercent();
        }

        if (PublicUtil.isNotEmpty(rewardParams)) {
            evalGroupRewardParamService.saveBatch(rewardParams);
        }
        if (PublicUtil.isNotEmpty(rewardLadders)) {
            evalGroupRewardLaddersService.saveBatch(rewardLadders);
        }
        if (PublicUtil.isNotEmpty(rewardPreconditions)) {
            evalGroupRewardPreconditionService.saveBatch(rewardPreconditions);
        }
    }


    /**
     * 转换
     *
     * @param rewardDTO
     * @param rewardLaddersDTO
     * @return
     */
    public EvalGroupRewardLadders convertRewardLaddersDB(EvalGroupRewardDTO rewardDTO, EvalGroupRewardLaddersDTO rewardLaddersDTO) {
        EvalGroupRewardLadders rewardLadder = PublicUtil.copy(rewardLaddersDTO, EvalGroupRewardLadders.class);
        if (EvalRewardCalMethodEnum.RANK.equals(rewardDTO.getCalMethod()) && RankTypeEnum.NUMBER.equals(rewardDTO.getRankType())) {
            return rewardLadder;
        }
        if (DataTypeEnum.RATIO.equals(rewardDTO.getLaddersType())) {
            rewardLadder.setUpper(rewardLadder.getUpper().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
            rewardLadder.setLower(rewardLadder.getLower().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
        }
        return rewardLadder;
    }

//    public EvalGroupIndicator convertDB(EvalGroupIndicatorDTO indicatorDto, Long evalGroupId) {
//        EvalGroupIndicator indicator = new EvalGroupIndicator();
//        indicator.setEvalGroupId(evalGroupId);
//        indicator.setName(indicatorDto.getName());
//        indicator.setBaseScore(indicatorDto.getBaseScore());
//        indicator.setScoreWay(indicatorDto.getScoreWay());
//
//        if (PublicUtil.isNotEmpty(indicatorDto.getLaddersType())) {
//            indicator.setLaddersType(indicatorDto.getLaddersType());
//        }
//
//        if (PublicUtil.isNotEmpty(indicatorDto.getLadderParamAlias())) {
//            indicator.setLadderParamAlias(indicatorDto.getLadderParamAlias());
//        }
//        if (PublicUtil.isNotEmpty(indicatorDto.getCommissionParamAlias())) {
//            indicator.setCommissionParamAlias(indicatorDto.getCommissionParamAlias());
//        }
//        if (PublicUtil.isNotEmpty(indicatorDto.getPreconditionAlias())) {
//            indicator.setPreconditionAlias(indicatorDto.getPreconditionAlias());
//        }
//
//        if (PublicUtil.isNotEmpty(indicatorDto.getLadderCode())) {
//            indicator.setLadderCode(indicatorDto.getLadderCode());
//        }
//        if (PublicUtil.isNotEmpty(indicatorDto.getLadderCodeType())) {
//            indicator.setLadderCodeType(indicatorDto.getLadderCodeType());
//        }
//        if (PublicUtil.isNotEmpty(indicatorDto.getCommissionCode())) {
//            indicator.setCommissionCode(indicatorDto.getCommissionCode());
//        }
//        if (PublicUtil.isNotEmpty(indicatorDto.getCommissionCodeType())) {
//            indicator.setCommissionCodeType(indicatorDto.getCommissionCodeType());
//        }
//
//        indicator.setYn(Boolean.TRUE);
//        return indicator;
//    }

}
