package cn.fw.morax.service.biz.kpi;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.ehr.sdk.api.enums.RoleManagerEnum;
import cn.fw.ehr.sdk.api.result.PostRoleVo;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.kpi.ShopIndicatorBO;
import cn.fw.morax.domain.bo.kpi.ShopIndicatorStageBO;
import cn.fw.morax.domain.bo.kpi.UserIndicatorBO;
import cn.fw.morax.domain.bo.kpi.UserIndicatorStageBO;
import cn.fw.morax.domain.db.eval.EvalGroup;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorParam;
import cn.fw.morax.domain.db.eval.EvalGroupIndicatorPrecondition;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.db.salary.SalaryGroupProjectParam;
import cn.fw.morax.domain.db.salary.SalaryGroupProjectPrecondition;
import cn.fw.morax.domain.dto.kpi.IndicatorsDTO;
import cn.fw.morax.domain.dto.kpi.KpiConfigPostIndicatorsDTO;
import cn.fw.morax.domain.dto.kpi.UserIndicatorDTO;
import cn.fw.morax.domain.dto.query.IndicatorPostQueryDTO;
import cn.fw.morax.domain.dto.query.IndicatorQueryDTO;
import cn.fw.morax.domain.dto.query.QueryConditionIndicatorsListDTO;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.domain.vo.kpi.IndicatorUserStageValueVO;
import cn.fw.morax.domain.vo.kpi.IndicatorsVO;
import cn.fw.morax.domain.vo.kpi.QueryConditionIndicatorsListVO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PostInfoDTO;
import cn.fw.morax.rpc.ehr.dto.PostRoleDTO;
import cn.fw.morax.sdk.dto.CustomList;
import cn.fw.morax.sdk.dto.common.*;
import cn.fw.morax.service.data.CombineIndicatorParamService;
import cn.fw.morax.service.data.eval.EvalGroupIndicatorParamService;
import cn.fw.morax.service.data.eval.EvalGroupIndicatorPreconditionService;
import cn.fw.morax.service.data.eval.EvalGroupService;
import cn.fw.morax.service.data.kpi.IndicatorUserStageValueService;
import cn.fw.morax.service.data.kpi.IndicatorUserValueService;
import cn.fw.morax.service.data.kpi.IndicatorsService;
import cn.fw.morax.service.data.salary.SalaryGroupProjectParamService;
import cn.fw.morax.service.data.salary.SalaryGroupProjectPreconditionService;
import cn.fw.morax.service.data.salary.SalaryGroupService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class IndicatorBizService {

    private final SalaryGroupProjectPreconditionService salaryGroupProjectPreconditionService;
    private final SalaryGroupProjectParamService salaryGroupProjectParamService;
    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    private final EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    private final EvalGroupService evalGroupService;
    private final SalaryGroupService salaryGroupService;
    private final KpiGroupIndicatorBizService kpiGroupIndicatorBizService;
    private final IndicatorsService indicatorsService;
    private final CombineIndicatorParamService combineIndicatorParamService;
    private final EhrRpcService ehrRpcService;
    private final StringRedisTemplate stringRedisTemplate;
    private final IndicatorUserValueService indicatorUserValueService;
    private final IndicatorUserStageValueService indicatorUserStageValueService;

    /**
     * 指标数据上报redis-key
     */
    @Value("${spring.cache.custom.global-prefix}:indicator-value")
    @Getter
    private String indicatorKeyPref;
    @Value("${spring.cache.custom.global-prefix}:indicator-shop-value")
    @Getter
    private String indicatorShopKeyPref;
    @Value("${spring.cache.custom.global-prefix}:indicator:save:")
    @Getter
    private String savePrefix;
    @Value("${spring.cache.custom.global-prefix}:indicator:enable:")
    @Getter
    private String enablePrefix;
    @Value("${spring.cache.custom.global-prefix}:indicator-stage-value")
    @Getter
    private String indicatorStageKeyPref;
    @Value("${spring.cache.custom.global-prefix}:indicator-shop-stage-value")
    @Getter
    private String indicatorShopStageKeyPref;

    /**
     * 分页查询
     *
     * @param dto
     * @return
     */
    public AppPage<IndicatorsVO> indicatorsPage(IndicatorQueryDTO dto) {
        if (PublicUtil.isNotEmpty(dto.getRoleCode())) {
            List<ApplicableRoleEnum> roleTypes = new ArrayList<ApplicableRoleEnum>() {{
                add(ApplicableRoleEnum.ALL);
                add(ApplicableRoleEnum.MANAGEMENT_ROLE);
            }};
            dto.setAllRoleTypes(roleTypes);
        }
        Long total = indicatorsService.indicatorsPageTotal(dto);
        return PublicUtil.<IndicatorsVO, IndicatorQueryDTO>queryPage(dto, total, new Function<IndicatorQueryDTO, List<IndicatorsVO>>() {
            @Override
            public List<IndicatorsVO> apply(IndicatorQueryDTO basePageQuery) {
                return indicatorsService.indicatorsPage(dto);
            }
        });

    }

    /**
     * 指标详情
     *
     * @param id
     * @return
     */
    public IndicatorsVO indicatorDetail(Long id) {
        Indicators indicators = indicatorsService.getById(id);
        BV.notNull(indicators, "指标不存在，请重试");
        IndicatorsVO indicatorsVO = PublicUtil.copy(indicators, IndicatorsVO.class);
//        List<ApplicableTypeEnum> applicableTypes = indicators.getApplyTypes().stream()
//                .map(applyType -> ApplicableTypeEnum.ofValue(applyType)).collect(Collectors.toList());
//        indicatorsVO.setApplyTypes(applicableTypes);
        return indicatorsVO;
    }

    /**
     * 保存指标
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getIndicatorName()", message = "保存中，请勿重复操作")
    public void saveIndicator(IndicatorsDTO dto) {
        this.checkIndicatorRole(dto);
        this.checkIndicatorName(dto.getId(), dto.getIndicatorName());
        Indicators indicators = dto.convertToPo();
        if (PublicUtil.isEmpty(dto.getId())) {
            String indicatorCode = PublicUtil.getUUID();
            this.checkIndicatorCode(indicatorCode);
            indicators.setIndicatorCode(indicatorCode);
            indicators.setInnerIndicator(Boolean.FALSE);
        } else {
            Indicators originIndicators = indicatorsService.getById(dto.getId());
            BV.isFalse(originIndicators.getInnerIndicator(), "内部指标不能编辑，请重试");
            BV.notNull(originIndicators, "指标不存在，请重试");
            indicators.setIndicatorCode(originIndicators.getIndicatorCode());
            indicators.setId(originIndicators.getId());
            indicators.setInnerIndicator(originIndicators.getInnerIndicator());
            indicators.setEnable(originIndicators.getEnable());
        }
        indicatorsService.saveOrUpdate(indicators);
    }


    /**
     * 指标库禁用启用
     *
     * @param indicatorId
     * @param enable
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getEnablePrefix()", key = "#indicatorId", message = "修改中，请勿重复操作")
    public void enableIndicator(Long indicatorId, Boolean enable) {
        Indicators indicators = indicatorsService.getById(indicatorId);
        BV.notNull(indicators, "指标不存在，请重试");
        BV.isFalse(enable.equals(indicators.getEnable()), "指标已经是" + ((enable) ? "启用" : "禁用") + "状态，请重试");
        String indicatorCode = indicators.getIndicatorCode();
        //禁用情况：正在使用的指标不能禁用，并提示哪些地方在用
        if (!enable) {
            BV.isFalse(indicators.getInnerIndicator(), "内部指标不能禁用，请重试");

            StringBuilder errorStr = new StringBuilder();
            List<KpiGroup> kpiGroups = kpiGroupIndicatorBizService.getKpiGroupByIndicatorCode(indicatorCode);
            Set<String> kpiGroupNames = kpiGroups.stream().map(KpiGroup::getName).collect(Collectors.toSet());
            if (PublicUtil.isNotEmpty(kpiGroupNames)) {
                errorStr.append("绩效组:[").append(String.join(",", kpiGroupNames)).append("]");
            }
            List<SalaryGroup> salaryGroups = this.getSalaryGroupByIndicatorCode(indicatorCode);
            Set<String> salaryGroupNames = salaryGroups.stream().map(SalaryGroup::getName).collect(Collectors.toSet());
            if (PublicUtil.isNotEmpty(salaryGroupNames)) {
                if (errorStr.length() > 0) {
                    errorStr.append(",");
                }
                errorStr.append("薪酬组:[").append(String.join(",", salaryGroupNames)).append("]");
            }
//            List<EvalGroup> evalGroups = this.getEvalGroupByIndicatorCode(indicatorCode);
//            Set<String> evalGroupNames = evalGroups.stream().map(EvalGroup::getName).collect(Collectors.toSet());
//            if (PublicUtil.isNotEmpty(evalGroups)) {
//                if (errorStr.length() > 0) {
//                    errorStr.append(",");
//                }
//                errorStr.append("考评组:[").append(String.join(",", salaryGroupNames)).append("]");
//                errorStr.append("考评组");
//            }

            if (errorStr.length() > 0) {
                errorStr.append("正在使用此绩效指标，不能禁用");
                throw new BusinessException(errorStr.toString());
            }
        }
        indicatorsService.update(Wrappers.<Indicators>lambdaUpdate()
                .set(Indicators::getEnable, enable)
                .eq(Indicators::getId, indicatorId)
        );
    }

    /**
     * 手动上报指标值
     *
     * @param dto
     */
    @DisLock(key = "#dto.indicatorCode+':'+ #dto.userId", message = "请勿重复操作")
    public void reportData(UserIndicatorDTO dto) {
        UserIndicatorBO bo = new UserIndicatorBO();
        bo.setIndicatorCode(dto.getIndicatorCode());
        bo.setUserId(dto.getUserId());
        bo.setDataDate(dto.getDataDate());
        bo.setGroupId(dto.getGroupId());
        bo.setValueType(dto.getValueType());
        BigDecimal value = dto.getValue();
        Map<String, BigDecimal> map = new HashMap<>(1);
        map.put(dto.getIndicatorCode(), value);
        bo.setValue(map);
        saveUserIndicator(bo);
    }

    /**
     * 收集业务系统上报的数据
     * 先直接放到redis增加系统吞吐量
     *
     * @param userIndicatorList
     */
    public void reportData(CustomList<IndicatorValueReq> userIndicatorList) {
        String[] jsonArr = userIndicatorList.stream().map(r -> {
            UserIndicatorBO bo = transferBo(r);
            return JSONObject.toJSONString(bo);
        }).toArray(String[]::new);
        BoundListOperations<String, String> opsList = stringRedisTemplate.boundListOps(getIndicatorKeyPref());
        opsList.rightPushAll(jsonArr);
    }

    /**
     * 收集业务系统上报的数据
     * 先直接放到redis增加系统吞吐量
     *
     * @param userIndicatorList
     */
    public void reportStageData(CustomList<IndicatorStageValueReq> userIndicatorList) {
        String[] jsonArr = userIndicatorList.stream().map(r -> {
            UserIndicatorStageBO bo = transferStageBo(r);
            return JSONObject.toJSONString(bo);
        }).toArray(String[]::new);
        BoundListOperations<String, String> opsList = stringRedisTemplate.boundListOps(getIndicatorStageKeyPref());
        opsList.rightPushAll(jsonArr);
    }

    /**
     * 从缓存取池上报的数据并进行持久化
     */
    public void persistenceUserData() {
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        ListOperations<String, String> opsForList = stringRedisTemplate.opsForList();
        List<String> overflowsOrFailList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = opsForList.leftPop(getIndicatorKeyPref())) != null) {
            final UserIndicatorBO bo = JSONObject.parseObject(jsonStr, UserIndicatorBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                //保存业务系统绩效指标值
                final String finalJsonStr = jsonStr;
                threadPool.execute(() -> {
                    try {
                        saveUserIndicator(bo);
                    } catch (Exception e) {
                        log.error("保存业务系统绩效指标值失败：{}", bo, e);
                        overflowsOrFailList.add(finalJsonStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsOrFailList.add(jsonStr);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsOrFailList)) {
            opsForList.rightPushAll(getIndicatorKeyPref(), overflowsOrFailList);
        }
    }

    /**
     * 从缓存取池上报的数据并进行持久化
     */
    public void persistenceUserStageData() {
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        ListOperations<String, String> opsForList = stringRedisTemplate.opsForList();
        List<String> overflowsOrFailList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = opsForList.leftPop(getIndicatorStageKeyPref())) != null) {
            final UserIndicatorStageBO bo = JSONObject.parseObject(jsonStr, UserIndicatorStageBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                //保存业务系统绩效指标值
                final String finalJsonStr = jsonStr;
                threadPool.execute(() -> {
                    try {
                        saveUserStageIndicator(bo);
                    } catch (Exception e) {
                        log.error("保存业务系统绩效阶段指标值失败：{}", bo, e);
                        overflowsOrFailList.add(finalJsonStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsOrFailList.add(jsonStr);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsOrFailList)) {
            opsForList.rightPushAll(getIndicatorKeyPref(), overflowsOrFailList);
        }
    }

    /**
     * 持久化业务系统上报的指标数据
     *
     * @param bo
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveUserIndicator(UserIndicatorBO bo) {
        final LocalDate dataDate = DateUtil.date2LocalDate(bo.getDataDate());
        Indicators indicator = indicatorsService.queryByCode(bo.getIndicatorCode(), Boolean.FALSE);
        if (Objects.isNull(indicator)) {
            return;
        }
        IndicatorUserValue value = new IndicatorUserValue();
        value.setUserId(bo.getUserId());
        value.setShopId(-1L);
        value.setDimensionType(DimensionTypeEnum.STAFF);
        value.setIndicatorCode(indicator.getIndicatorCode());
        value.setIndicatorName(indicator.getIndicatorName());
        IndicatorValueTypeEnum valueTypeEnum = IndicatorValueTypeEnum.ofValue(bo.getValueType());
        value.setValueType(valueTypeEnum);
        Map<String, BigDecimal> valueMap = bo.getValue();
        value.setIndicatorValue(JSONObject.toJSONString(valueMap));
        value.setDataDate(dataDate);
        value.setGroupId(bo.getGroupId());
        value.setYn(Boolean.TRUE);
        indicatorUserValueService.remove(Wrappers.<IndicatorUserValue>lambdaQuery()
                .eq(IndicatorUserValue::getUserId, value.getUserId())
                .eq(IndicatorUserValue::getIndicatorCode, value.getIndicatorCode())
                .eq(IndicatorUserValue::getDataDate, dataDate)
                .eq(IndicatorUserValue::getGroupId, value.getGroupId())
        );
        indicatorUserValueService.save(value);
    }

    /**
     * 持久化业务系统上报的指标数据
     *
     * @param bo
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveUserStageIndicator(UserIndicatorStageBO bo) {
        final LocalDate beginDate = DateUtil.date2LocalDate(bo.getBeginDate());
        final LocalDate endDate = DateUtil.date2LocalDate(bo.getEndDate());
        Indicators indicator = indicatorsService.queryByCode(bo.getIndicatorCode(), Boolean.FALSE);
        if (Objects.isNull(indicator)) {
            return;
        }
        IndicatorUserStageValue value = new IndicatorUserStageValue();
        value.setUserId(bo.getUserId());
        value.setShopId(-1L);
        value.setDimensionType(DimensionTypeEnum.STAFF);
        value.setIndicatorCode(indicator.getIndicatorCode());
        value.setIndicatorName(indicator.getIndicatorName());
        IndicatorValueTypeEnum valueTypeEnum = IndicatorValueTypeEnum.ofValue(bo.getValueType());
        value.setValueType(valueTypeEnum);
        Map<String, BigDecimal> valueMap = bo.getValue();
        value.setIndicatorValue(JSONObject.toJSONString(valueMap));
        value.setBeginDate(beginDate);
        value.setEndDate(endDate);
        value.setGroupId(bo.getGroupId());
        value.setYn(Boolean.TRUE);
        indicatorUserStageValueService.remove(Wrappers.<IndicatorUserStageValue>lambdaQuery()
                .eq(IndicatorUserStageValue::getUserId, value.getUserId())
                .eq(IndicatorUserStageValue::getIndicatorCode, value.getIndicatorCode())
                .eq(IndicatorUserStageValue::getBeginDate, beginDate)
                .eq(IndicatorUserStageValue::getEndDate, beginDate)
                .eq(IndicatorUserStageValue::getGroupId, value.getGroupId())
        );
        indicatorUserStageValueService.save(value);
    }

    /**
     * 收集业务系统上报的数据
     * 先直接放到redis增加系统吞吐量
     *
     * @param shopIndicatorList
     */
    public void reportShopData(CustomList<IndicatorShopValueReq> shopIndicatorList) {
        String[] jsonArr = shopIndicatorList.stream().map(r -> {
            ShopIndicatorBO bo = transferShopBo(r);
            return JSONObject.toJSONString(bo);
        }).toArray(String[]::new);
        BoundListOperations<String, String> opsList = stringRedisTemplate.boundListOps(getIndicatorShopKeyPref());
        opsList.rightPushAll(jsonArr);
    }

    /**
     * 收集业务系统上报的数据
     * 先直接放到redis增加系统吞吐量
     *
     * @param shopIndicatorList
     */
    public void reportStageShopData(CustomList<IndicatorShopStageValueReq> shopIndicatorList) {
        String[] jsonArr = shopIndicatorList.stream().map(r -> {
            ShopIndicatorStageBO bo = transferShopBo(r);
            return JSONObject.toJSONString(bo);
        }).toArray(String[]::new);
        BoundListOperations<String, String> opsList = stringRedisTemplate.boundListOps(getIndicatorShopStageKeyPref());
        opsList.rightPushAll(jsonArr);
    }

    /**
     * 从缓存取池上报的数据并进行持久化
     */
    public void persistenceShopData() {
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        ListOperations<String, String> opsForList = stringRedisTemplate.opsForList();
        List<String> overflowsOrFailList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = opsForList.leftPop(getIndicatorShopKeyPref())) != null) {
            final ShopIndicatorBO bo = JSONObject.parseObject(jsonStr, ShopIndicatorBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                //保存业务系统绩效指标值
                final String finalJsonStr = jsonStr;
                threadPool.execute(() -> {
                    try {
                        saveShopIndicator(bo);
                    } catch (Exception e) {
                        log.error("保存业务系统绩效指标值失败：{}", bo, e);
                        overflowsOrFailList.add(finalJsonStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsOrFailList.add(jsonStr);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsOrFailList)) {
            opsForList.rightPushAll(getIndicatorShopKeyPref(), overflowsOrFailList);
        }
    }

    /**
     * 持久化业务系统上报的指标数据
     *
     * @param bo
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveShopIndicator(ShopIndicatorBO bo) {
        final LocalDate dataDate = DateUtil.date2LocalDate(bo.getDataDate());
        Indicators indicator = indicatorsService.queryByCode(bo.getIndicatorCode(), Boolean.FALSE);
        if (Objects.isNull(indicator)) {
            return;
        }
        IndicatorUserValue value = new IndicatorUserValue();
        value.setUserId(-1L);
        value.setShopId(bo.getShopId());
        value.setDimensionType(DimensionTypeEnum.SHOP);
        value.setIndicatorCode(indicator.getIndicatorCode());
        value.setIndicatorName(indicator.getIndicatorName());
        IndicatorValueTypeEnum valueTypeEnum = IndicatorValueTypeEnum.ofValue(bo.getValueType());
        value.setValueType(valueTypeEnum);
        Map<String, BigDecimal> valueMap = bo.getValue();
        value.setIndicatorValue(JSONObject.toJSONString(valueMap));
        value.setDataDate(dataDate);
        value.setGroupId(bo.getGroupId());
        value.setYn(Boolean.TRUE);
        indicatorUserValueService.remove(Wrappers.<IndicatorUserValue>lambdaQuery()
                .eq(IndicatorUserValue::getShopId, value.getShopId())
                .eq(IndicatorUserValue::getIndicatorCode, value.getIndicatorCode())
                .eq(IndicatorUserValue::getDataDate, dataDate)
                .eq(IndicatorUserValue::getGroupId, value.getGroupId())
        );
        indicatorUserValueService.save(value);
    }

    /**
     * 从缓存取池上报的数据并进行持久化
     */
    public void persistenceShopStageData() {
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        ListOperations<String, String> opsForList = stringRedisTemplate.opsForList();
        List<String> overflowsOrFailList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = opsForList.leftPop(getIndicatorShopStageKeyPref())) != null) {
            final ShopIndicatorStageBO bo = JSONObject.parseObject(jsonStr, ShopIndicatorStageBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                //保存业务系统绩效指标值
                final String finalJsonStr = jsonStr;
                threadPool.execute(() -> {
                    try {
                        saveShopStageIndicator(bo);
                    } catch (Exception e) {
                        log.error("保存业务系统绩效指标值失败：{}", bo, e);
                        overflowsOrFailList.add(finalJsonStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsOrFailList.add(jsonStr);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsOrFailList)) {
            opsForList.rightPushAll(getIndicatorShopKeyPref(), overflowsOrFailList);
        }
    }

    /**
     * 持久化业务系统上报的指标数据
     *
     * @param bo
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveShopStageIndicator(ShopIndicatorStageBO bo) {
        final LocalDate beginDate = DateUtil.date2LocalDate(bo.getBeginDate());
        final LocalDate endDate = DateUtil.date2LocalDate(bo.getEndDate());
        Indicators indicator = indicatorsService.queryByCode(bo.getIndicatorCode(), Boolean.FALSE);
        if (Objects.isNull(indicator)) {
            return;
        }
        IndicatorUserStageValue value = new IndicatorUserStageValue();
        value.setUserId(-1L);
        value.setShopId(bo.getShopId());
        value.setDimensionType(DimensionTypeEnum.SHOP);
        value.setIndicatorCode(indicator.getIndicatorCode());
        value.setIndicatorName(indicator.getIndicatorName());
        IndicatorValueTypeEnum valueTypeEnum = IndicatorValueTypeEnum.ofValue(bo.getValueType());
        value.setValueType(valueTypeEnum);
        Map<String, BigDecimal> valueMap = bo.getValue();
        value.setIndicatorValue(JSONObject.toJSONString(valueMap));
        value.setBeginDate(beginDate);
        value.setEndDate(endDate);
        value.setGroupId(bo.getGroupId());
        value.setYn(Boolean.TRUE);
        indicatorUserStageValueService.remove(Wrappers.<IndicatorUserStageValue>lambdaQuery()
                .eq(IndicatorUserStageValue::getShopId, value.getShopId())
                .eq(IndicatorUserStageValue::getIndicatorCode, value.getIndicatorCode())
                .eq(IndicatorUserStageValue::getBeginDate, beginDate)
                .eq(IndicatorUserStageValue::getEndDate, beginDate)
                .eq(IndicatorUserStageValue::getGroupId, value.getGroupId())
        );
        indicatorUserStageValueService.save(value);
    }

    /**
     * 检查指标角色配置
     *
     * @param dto
     */
    private void checkIndicatorRole(IndicatorsDTO dto) {
        Boolean portionNotSelectRole = ApplicableRoleEnum.PORTION.equals(dto.getRoleType()) &&
                PublicUtil.isEmpty(dto.getRoleNames());
        BV.isFalse(portionNotSelectRole, "自定义角色，请选择至少一个角色");
    }

    /**
     * 检查指标名称
     *
     * @param indicatorName
     */
    private void checkIndicatorName(Long id, String indicatorName) {
        long taskCount = indicatorsService.count(Wrappers.<Indicators>lambdaQuery()
                .eq(Indicators::getIndicatorName, indicatorName)
                .eq(Indicators::getYn, Boolean.TRUE)
                .ne(PublicUtil.isNotEmpty(id), Indicators::getId, id)
        );
        BV.isFalse(taskCount > 0, "指标名称已存在，请重新输入");
    }

    /**
     * 检查指标编码
     *
     * @param indicatorCode
     */
    private void checkIndicatorCode(String indicatorCode) {
        long taskCount = indicatorsService.count(Wrappers.<Indicators>lambdaQuery()
                .eq(Indicators::getIndicatorCode, indicatorCode)
        );
        if (taskCount > 0) {
            log.error("指标编码生成重复：{}", indicatorCode);
            throw new BusinessException("指标编码已存在，请重试");
        }
    }

    /**
     * 获取岗位下的绩效组指标
     *
     * @param indicatorsDTO
     * @return
     */
    public List<IndicatorsVO> getPostIndicators(KpiConfigPostIndicatorsDTO indicatorsDTO, LoginAuthBean currentUser) {
        List<Long> postIds = new ArrayList<Long>(){{
            add(indicatorsDTO.getPostId());
        }};
        List<PostInfoDTO> postInfoDTOS = ehrRpcService.getPostInfoList(postIds);
        List<String> roleCodes = postInfoDTOS.stream()
                .map(PostInfoDTO::getRoleList)
                .filter(Objects::nonNull)
                .flatMap(Collection::stream)
                .map(PostRoleDTO::getRoleCode)
                .distinct().collect(Collectors.toList());
        Boolean queryManagerRole = hasManagerRole(roleCodes);
        IndicatorPostQueryDTO dto = new IndicatorPostQueryDTO(roleCodes, queryManagerRole);
        dto.setApplicableType(ApplicableTypeEnum.ofValue(indicatorsDTO.getApplicableType()));
//        IndicatorPostQueryDTO dto = new IndicatorPostQueryDTO(new ArrayList<String>(){{add("FWGW");add("XSGW");}}, Boolean.TRUE);
        List<IndicatorsVO> indicatorsVOS = indicatorsService.getPostIndicators(dto);
        if (!CollectionUtils.isEmpty(indicatorsVOS)){
            for (IndicatorsVO indicatorsVO : indicatorsVOS) {
                indicatorsVO.setCodeType(IndicatorCodeTypeEnum.INDICATOR);
            }
        }else {
            indicatorsVOS = new ArrayList<>();
        }
        if (Objects.nonNull(indicatorsDTO.getWithCombination()) && indicatorsDTO.getWithCombination()){
            indicatorsVOS.addAll(combineIndicatorParamService.getEnableCombines(currentUser.getGroupId(),IndicatorCodeTypeEnum.INDICATOR,CombineIndicatorTypeEnum.INDICATOR.getValue(),null));
        }
        return indicatorsVOS;
    }

    /**
     * 是否拥有管理角色
     *
     * @return
     */
    private Boolean hasManagerRole(List<String> roleCodes) {
        if (PublicUtil.isEmpty(roleCodes)) {
            return Boolean.FALSE;
        }
        HashSet<String> roleCodeSet = new HashSet<>(roleCodes);
        return Arrays.stream(RoleManagerEnum.values())
                .anyMatch(roleManagerEnum -> roleCodeSet.contains(roleManagerEnum.getCode()));
    }

    /**
     * 获取正在使用指标的薪酬组
     *
     * @param indicatorCode
     * @return
     */
    public List<SalaryGroup> getSalaryGroupByIndicatorCode(String indicatorCode) {
        List<SalaryGroupProjectParam> salaryGroupProjectParams = salaryGroupProjectParamService.list(Wrappers.<SalaryGroupProjectParam>lambdaQuery()
                .eq(SalaryGroupProjectParam::getIndicatorCode, indicatorCode)
                .eq(SalaryGroupProjectParam::getYn, Boolean.TRUE)
        );
        List<SalaryGroupProjectPrecondition> preconditions = salaryGroupProjectPreconditionService.list(Wrappers.<SalaryGroupProjectPrecondition>lambdaQuery()
                .eq(SalaryGroupProjectPrecondition::getIndicatorCode, indicatorCode)
                .eq(SalaryGroupProjectPrecondition::getYn, Boolean.TRUE)
        );

        Set<Long> salaryGroupIds = new HashSet<>();
        if (PublicUtil.isNotEmpty(salaryGroupProjectParams)) {
            salaryGroupIds.addAll(salaryGroupProjectParams.stream().map(SalaryGroupProjectParam::getSalaryGroupId).collect(Collectors.toSet()));
        }
        if (PublicUtil.isNotEmpty(preconditions)) {
            salaryGroupIds.addAll(preconditions.stream().map(SalaryGroupProjectPrecondition::getSalaryGroupId).collect(Collectors.toSet()));
        }

        if (PublicUtil.isEmpty(salaryGroupIds)) {
            return new ArrayList<>();
        }

        List<SettingStatusEnum> status = new ArrayList<SettingStatusEnum>(){{
            add(SettingStatusEnum.APPROVING);
            add(SettingStatusEnum.BE_EFFECTIVE);
            add(SettingStatusEnum.EFFECTIVE);
        }};
        List<SalaryGroup> salaryGroups = salaryGroupService.list(Wrappers.<SalaryGroup>lambdaQuery()
                .in(SalaryGroup::getId, salaryGroupIds)
                .in(SalaryGroup::getStatus, status)
                .eq(SalaryGroup::getYn, Boolean.TRUE)
        );
        return salaryGroups;
    }

    /**
     * 获取正在使用指标的薪酬组
     *
     * @param indicatorCode
     * @return
     */
//    public List<EvalGroup> getEvalGroupByIndicatorCode(String indicatorCode) {
//        List<EvalGroupIndicatorParam> evalGroupIndicatorParams = evalGroupIndicatorParamService.list(Wrappers.<EvalGroupIndicatorParam>lambdaQuery()
//                .eq(EvalGroupIndicatorParam::getCode, indicatorCode)
//                .eq(EvalGroupIndicatorParam::getCodeType, IndicatorCodeTypeEnum.INDICATOR)
//                .eq(EvalGroupIndicatorParam::getYn, Boolean.TRUE)
//        );
//        List<EvalGroupIndicatorPrecondition> preconditions = evalGroupIndicatorPreconditionService.list(Wrappers.<EvalGroupIndicatorPrecondition>lambdaQuery()
//                .eq(EvalGroupIndicatorPrecondition::getCode, indicatorCode)
//                .eq(EvalGroupIndicatorPrecondition::getCodeType, IndicatorCodeTypeEnum.INDICATOR)
//                .eq(EvalGroupIndicatorPrecondition::getYn, Boolean.TRUE)
//        );
//
//        Set<Long> evalGroupIds = new HashSet<>();
//        if (PublicUtil.isNotEmpty(evalGroupIndicatorParams)) {
//            evalGroupIds.addAll(evalGroupIndicatorParams.stream().map(EvalGroupIndicatorParam::getEvalGroupId).collect(Collectors.toSet()));
//        }
//        if (PublicUtil.isNotEmpty(preconditions)) {
//            evalGroupIds.addAll(preconditions.stream().map(EvalGroupIndicatorPrecondition::getEvalGroupId).collect(Collectors.toSet()));
//        }
//
//        if (PublicUtil.isEmpty(evalGroupIds)) {
//            return new ArrayList<>();
//        }
//
////        List<SettingStatusEnum> status = new ArrayList<SettingStatusEnum>(){{
////            add(SettingStatusEnum.APPROVING);
////            add(SettingStatusEnum.BE_EFFECTIVE);
////            add(SettingStatusEnum.EFFECTIVE);
////        }};
//
//
//        LocalDate now = LocalDate.now();
//        List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
//                .in(EvalGroup::getId, evalGroupIds)
//                .ge(EvalGroup::getBeginTime, now)
//                .le(EvalGroup::getOverTime, now)
//                .eq(EvalGroup::getYn, Boolean.TRUE)
//        );
//        return evalGroups;
//    }

    /**
     * 转换数据
     *
     * @param req
     * @return
     */
    private UserIndicatorBO transferBo(IndicatorValueReq req) {
        UserIndicatorBO bo = new UserIndicatorBO();
        bo.setIndicatorCode(req.getIndicatorCode());
        bo.setUserId(req.getUserId());
        bo.setDataDate(req.getDataDate());
        bo.setGroupId(req.getGroupId());
        IndicatorValueTypeEnum typeEnum = IndicatorValueTypeEnum.ofValue(req.getValueType());
        bo.setValueType(req.getValueType());
        List<IndicatorValueDTO> valueList = req.getIndicatorValue();
        Map<String, BigDecimal> map = new HashMap<>(valueList.size());
        for (IndicatorValueDTO valueDTO : valueList) {
            String key = req.getIndicatorCode();
            if (IndicatorValueTypeEnum.CONDITION.equals(typeEnum)) {
                key = valueDTO.getKey();
            }
            map.put(key, valueDTO.getValue());
        }
        bo.setValue(map);
        return bo;
    }

    /**
     * 转换数据
     *
     * @param req
     * @return
     */
    private UserIndicatorStageBO transferStageBo(IndicatorStageValueReq req) {
        UserIndicatorStageBO bo = new UserIndicatorStageBO();
        bo.setIndicatorCode(req.getIndicatorCode());
        bo.setUserId(req.getUserId());
        bo.setBeginDate(req.getBeginDate());
        bo.setEndDate(req.getEndDate());
        bo.setGroupId(req.getGroupId());
        IndicatorValueTypeEnum typeEnum = IndicatorValueTypeEnum.ofValue(req.getValueType());
        bo.setValueType(req.getValueType());
        List<IndicatorValueDTO> valueList = req.getIndicatorValue();
        Map<String, BigDecimal> map = new HashMap<>(valueList.size());
        for (IndicatorValueDTO valueDTO : valueList) {
            String key = req.getIndicatorCode();
            if (IndicatorValueTypeEnum.CONDITION.equals(typeEnum)) {
                key = valueDTO.getKey();
            }
            map.put(key, valueDTO.getValue());
        }
        bo.setValue(map);
        return bo;
    }

    /**
     * 转换数据
     *
     * @param req
     * @return
     */
    private ShopIndicatorBO transferShopBo(IndicatorShopValueReq req) {
        ShopIndicatorBO bo = new ShopIndicatorBO();
        bo.setIndicatorCode(req.getIndicatorCode());
        bo.setShopId(req.getShopId());
        bo.setDataDate(req.getDataDate());
        bo.setGroupId(req.getGroupId());
        IndicatorValueTypeEnum typeEnum = IndicatorValueTypeEnum.ofValue(req.getValueType());
        bo.setValueType(req.getValueType());
        List<IndicatorValueDTO> valueList = req.getIndicatorValue();
        Map<String, BigDecimal> map = new HashMap<>(valueList.size());
        for (IndicatorValueDTO valueDTO : valueList) {
            String key = req.getIndicatorCode();
            if (IndicatorValueTypeEnum.CONDITION.equals(typeEnum)) {
                key = valueDTO.getKey();
            }
            map.put(key, valueDTO.getValue());
        }
        bo.setValue(map);
        return bo;
    }

    /**
     * 转换数据
     *
     * @param req
     * @return
     */
    private ShopIndicatorStageBO transferShopBo(IndicatorShopStageValueReq req) {
        ShopIndicatorStageBO bo = new ShopIndicatorStageBO();
        bo.setIndicatorCode(req.getIndicatorCode());
        bo.setShopId(req.getShopId());
        bo.setBeginDate(req.getBeginDate());
        bo.setEndDate(req.getEndDate());
        bo.setGroupId(req.getGroupId());
        IndicatorValueTypeEnum typeEnum = IndicatorValueTypeEnum.ofValue(req.getValueType());
        bo.setValueType(req.getValueType());
        List<IndicatorValueDTO> valueList = req.getIndicatorValue();
        Map<String, BigDecimal> map = new HashMap<>(valueList.size());
        for (IndicatorValueDTO valueDTO : valueList) {
            String key = req.getIndicatorCode();
            if (IndicatorValueTypeEnum.CONDITION.equals(typeEnum)) {
                key = valueDTO.getKey();
            }
            map.put(key, valueDTO.getValue());
        }
        bo.setValue(map);
        return bo;
    }


    /**
     * 获取带条件的绩效指标库
     *
     * @param dto
     * @return
     */
    public List<QueryConditionIndicatorsListVO> queryConditionIndicatorsList(QueryConditionIndicatorsListDTO dto) {
        if (PublicUtil.isNotEmpty(dto.getRoleCode())) {
            List<ApplicableRoleEnum> roleTypes = new ArrayList<ApplicableRoleEnum>() {{
                add(ApplicableRoleEnum.ALL);
                add(ApplicableRoleEnum.MANAGEMENT_ROLE);
            }};
            dto.setAllRoleTypes(roleTypes);
        }
        return indicatorsService.queryConditionIndicatorsList(dto);
    }
}
