package cn.fw.morax.service.biz.kpi;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.page.AppPage;
import cn.fw.ehr.sdk.api.enums.StaffStatusEnum;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.MessageFormatUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.dto.query.*;
import cn.fw.morax.domain.enums.KpiIgnoreCauseEnum;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.domain.vo.SelectorVO;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.service.biz.CommonService;
import cn.fw.morax.service.data.kpi.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.core.BoundValueOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class KpiGroupUserBizService {

    private final KpiPoolIndicatorValueService kpiPoolIndicatorValueService;
    private final KpiStarSpecialRuleService kpiStarSpecialRuleService;
    private final KpiStarLaddersService kpiStarLaddersService;
    private final KpiGroupRankService kpiGroupRankService;
    private final KpiGroupUserService kpiGroupUserService;
    private final StringRedisTemplate stringRedisTemplate;
    private final KpiGroupService kpiGroupService;
    private final KpiPoolService kpiPoolService;
    private final EhrRpcService ehrRpcService;
    private final CommonService commonService;

    @Value("${spring.cache.custom.global-prefix}:kpi:anticipated_date:")
    @Getter
    private String startKpiKey;

    @Value("${spring.cache.custom.global-prefix}:kpi:await_job:anticipated_date:")
    @Getter
    private String awaitJobAfterStartKpiKey;

    /**
     * 构建绩效组人员
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public List<KpiGroupUser> fetchBuildKpiUser(Long postId, List<KpiGroup> kpiGroups, LocalDate localDate){
        Set<Long> shopIds = new HashSet<>();
        Map<Long, KpiGroup> shopKpiGroupMap = new HashMap<>();
        for (KpiGroup kpiGroup : kpiGroups) {
            shopIds.addAll(kpiGroup.getShopIds());
            for (Long shopId : kpiGroup.getShopIds()) {
                shopKpiGroupMap.put(shopId, kpiGroup);
            }
        }

        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaff(postId, new ArrayList<>(shopIds));
        if (PublicUtil.isEmpty(staffs)) {
            return new ArrayList<>();
        }
        List<KpiGroupUser> result = Lists.newArrayListWithCapacity(staffs.size());
        List<KpiStarSpecialRule> specialRules = kpiStarSpecialRuleService.list(Wrappers.<KpiStarSpecialRule>lambdaQuery()
                .eq(KpiStarSpecialRule::getStatus, SettingStatusEnum.EFFECTIVE)
                .eq(KpiStarSpecialRule::getYn, Boolean.TRUE)
        );
        Map<Long, KpiStarSpecialRule> userRuleMap = specialRules.stream()
                .collect(Collectors.toMap(KpiStarSpecialRule::getUserId, Function.identity(), (v1, v2) -> v2));


        KpiGroup kpiGroup = null;
        for (PerformanceStaffDTO staff : staffs) {
            //检查员工数据是否正确
//            if (this.checkStaffInValid(staff)) {
//                continue;
//            }
            KpiGroupUser kpiUser = this.convertToGroupUser(staff, localDate);
            if (! shopKpiGroupMap.containsKey(staff.getShopId())) {
                log.error("员工门店没有绩效组配置，门店：{}，岗位：{}，员工：{}", staff.getShopId(), staff.getPostId(), staff.getId());
                continue;
            }
            kpiGroup = shopKpiGroupMap.get(staff.getShopId());
            kpiUser.setKpiGroupId(kpiGroup.getId());
            kpiUser.setKgc(kpiGroup.getKgc());
            //处理绩效考核数据
            this.handleStaffKpi(staff, kpiUser, userRuleMap);
            result.add(kpiUser);
        }
        return result;
    }
    /**
     * 预期开始考核时间
     *
     * @param postId
     * @param shopIds
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public List<KpiGroupUser> fetchRealTimeKpiUser(Long postId, List<Long> shopIds, String staffName){
        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaffByName(postId, shopIds, staffName);
        if (PublicUtil.isEmpty(staffs)) {
            return new ArrayList<>();
        }
        LocalDate now = LocalDate.now();
        List<KpiGroupUser> result = Lists.newArrayListWithCapacity(staffs.size());
        List<KpiStarSpecialRule> specialRules = kpiStarSpecialRuleService.list(Wrappers.<KpiStarSpecialRule>lambdaQuery()
                .eq(KpiStarSpecialRule::getStatus, SettingStatusEnum.EFFECTIVE)
                .eq(KpiStarSpecialRule::getYn, Boolean.TRUE)
        );
        Map<Long, KpiStarSpecialRule> userRuleMap = specialRules.stream()
                .collect(Collectors.toMap(KpiStarSpecialRule::getUserId, Function.identity(), (v1, v2) -> v2));

        KpiGroupUser kpiUser = null;
        for (PerformanceStaffDTO staff : staffs) {
            //检查员工数据是否正确
//            if (this.checkStaffInValid(staff)) {
//                continue;
//            }
            kpiUser = this.convertToGroupUser(staff, now);
            //处理绩效考核数据
            this.handleStaffKpi(staff, kpiUser, userRuleMap);
            result.add(kpiUser);
        }
        return result;
    }

    /**
     * 检查员工数据无效
     *
     * @param staff
     * @return
     */
    public Boolean checkStaffInValid(PerformanceStaffDTO staff) {
        if (StaffStatusEnum.PROBATION.getValue().equals(staff.getStaffStatus())) {
//            log.error("员工目前试用期，但是转正时间不为空，员工：{}", JSON.toJSONString(staff));
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }

    /**
     * 处理员工绩效考核数据
     * 试用期、转正两个月内、调岗两个月内、待岗两个月内、申请不进行绩效考核
     *
     * @param staff
     * @param kpiUser
     * @param userRuleMap
     */
    public void handleStaffKpi(PerformanceStaffDTO staff, KpiGroupUser kpiUser, Map<Long, KpiStarSpecialRule> userRuleMap) {
        LocalDate currentTime = LocalDate.now();
        KpiIgnoreCauseEnum ignoreCauseEnum = null;
        LocalDate startKpiDate = null;
        Long staffId = staff.getId();
        try {
            //试用期不计算绩效  绩效开始时间为试用期转正的两个月后
            if (StaffStatusEnum.PROBATION.getValue().equals(staff.getStaffStatus())) {
                startKpiDate = PublicUtil.getKpiDateOfProbationStaff(PublicUtil.isNotEmpty(staff.getEntryDate()) ? staff.getEntryDate() : LocalDate.now());
                ignoreCauseEnum = KpiIgnoreCauseEnum.PROBATION;
            }

            //转正两个月内不计算绩效  绩效开始时间为转正的两个月后
            if (StaffStatusEnum.REGULAR.getValue().equals(staff.getStaffStatus())) {
                LocalDate regularDate = PublicUtil.isNotEmpty(staff.getRegularDate()) ? staff.getRegularDate() : staff.getEntryDate();
                LocalDate newStartKpiDate = PublicUtil.getKpiDateOfRegularStaff(regularDate);
                if (this.checkNewStartKpiDateValid(startKpiDate, newStartKpiDate, currentTime)) {
                    ignoreCauseEnum = KpiIgnoreCauseEnum.TURN_POSITIVE_AFTER;
                    startKpiDate = newStartKpiDate;
                }
            }

            //绩效组变动不足完整2个月  绩效开始时间为获得岗位的两个月后
            LocalDate postChangeStartKpiDate = this.getStartKpiDate(staffId);
            if (PublicUtil.isNotEmpty(postChangeStartKpiDate)) {
                if (this.checkNewStartKpiDateValid(startKpiDate, postChangeStartKpiDate, currentTime)) {
                    ignoreCauseEnum = KpiIgnoreCauseEnum.TRANSFER_NEW_POSITION_AFTER;
                    startKpiDate = postChangeStartKpiDate;
                }
            }

            //主动申请不计算绩效  绩效开始时间为设置的申请开始绩效考核时间
            KpiStarSpecialRule specialRule = userRuleMap.get(staffId);
            //开始考核日期
            if (PublicUtil.isNotEmpty(specialRule) && PublicUtil.isNotEmpty(specialRule.getEarliestStartingMonthly())) {
                LocalDate newStartKpiDate = specialRule.getEarliestStartingMonthly();
                //当前时间在开始考核时间之之前    审批通过后，前面月份的绩效不纳入后续计算
                if (this.checkNewStartKpiDateValid(startKpiDate, newStartKpiDate, currentTime)) {
                    ignoreCauseEnum = KpiIgnoreCauseEnum.APPLY_NO_ASSESSMENT;
                    startKpiDate = newStartKpiDate;
                }
            }

            //待岗期间
            if (PublicUtil.isNotEmpty(staff.getAwaitJob()) && staff.getAwaitJob()) {
                ignoreCauseEnum = KpiIgnoreCauseEnum.AWAIT_JOB_DURATION;
                //@todo 人事系统待岗中人员没有待岗结束时间
                startKpiDate = null;
            }

            //待岗结束两个月期间
            LocalDate afterAwaitJobStartKpiDate = this.getAfterAwaitJobStartKpiDate(staffId);
            if (PublicUtil.isNotEmpty(afterAwaitJobStartKpiDate) ) {
                if (this.checkNewStartKpiDateValid(startKpiDate, afterAwaitJobStartKpiDate, currentTime)) {
                    ignoreCauseEnum = KpiIgnoreCauseEnum.AWAIT_JOB_AFTER;
                    startKpiDate = afterAwaitJobStartKpiDate;
                }
            }

            if (PublicUtil.isNotEmpty(ignoreCauseEnum)) {
                this.handleIgnoredKpiGroupUser(kpiUser, ignoreCauseEnum, startKpiDate);
                return;
            }

            //都未命中上面的条件就计算绩效，开始计算时间为转正两月后、获得新岗位两月后、申请开始考核时间任一
            kpiUser.setIgnored(Boolean.FALSE);
        }catch (Exception e) {
            kpiUser.setIgnored(Boolean.TRUE);
            log.error("处理绩效组人员数据异常，员工信息：{}", staff, e);
        }
    }

    /**
     * 判断新绩效生效日期 是否有效
     *
     * @param oldStartKpiDate
     * @param newStartKpiDate
     * @return
     */
    private Boolean checkNewStartKpiDateValid(LocalDate oldStartKpiDate, LocalDate newStartKpiDate, LocalDate currentTime) {
        //当前时间在生效日期之后，计算绩效
        if (currentTime.isAfter(newStartKpiDate)) {
            return Boolean.FALSE;
        }
        //旧生效时间为空  或者  新生效日期在旧生效日期之后
        if (PublicUtil.isEmpty(oldStartKpiDate) || newStartKpiDate.isAfter(oldStartKpiDate)) {
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }


    /**
     * 处理绩效忽略数据
     *
     * @param kpiUser
     * @param ignoreCauseEnum
     * @param anticipatedDate
     */
    private void handleIgnoredKpiGroupUser(KpiGroupUser kpiUser, KpiIgnoreCauseEnum ignoreCauseEnum, LocalDate anticipatedDate) {
        kpiUser.setIgnored(Boolean.TRUE);
        if (PublicUtil.isNotEmpty(anticipatedDate)) {
            kpiUser.setAnticipatedDate(anticipatedDate);
        }
        kpiUser.setIgnoreCause(ignoreCauseEnum);
        kpiUser.setIgnoreCauseDesc(ignoreCauseEnum.getName());
    }

    /**
     * 转换员工基础信息
     *
     * @param staff
     * @return
     */
    private KpiGroupUser convertToGroupUser(PerformanceStaffDTO staff, LocalDate now){
        KpiGroupUser kpiUser = KpiGroupUser.builder()
                .userId(staff.getId())
                .userName(staff.getName())
                .postId(staff.getPostId())
                .postName(staff.getPostName())
                .shopId(staff.getShopId())
                .shopName(staff.getShopName())
                .groupId(staff.getGroupId())
                .dataDate(now)
                .build();
        return kpiUser;
    }

    /**
     * 实时查询绩效组人员
     *
     * @param dto
     * @return
     */
    public List<KpiGroupUserVO> queryShopPostRealTimeStaffs(KpiGroupStaffQueryDTO dto) {
        List<KpiGroupUser> kpiGroupUsers = this.fetchRealTimeKpiUser(dto.getPostId(), dto.getShopIds(), dto.getStaffName());
        List<KpiGroupUserVO> kpiGroupUserVOS = PublicUtil.copyList(kpiGroupUsers, KpiGroupUserVO.class);
        return kpiGroupUserVOS;
    }

    /**
     * 实时查询绩效组人员数量
     *
     * @param postId
     * @param shopIds
     * @return
     */
    public Integer queryShopPostCurStaffNum(Long postId, List<Long> shopIds) {
        List<KpiGroupUser> kpiGroupUsers = this.fetchRealTimeKpiUser(postId, shopIds, null);
        return kpiGroupUsers.size();
    }

    /**
     * 查询绩效组人员
     *
     * @param dto
     * @return
     */
    public AppPage<KpiGroupUserVO> queryShopPostConfigStaffs(KpiGroupUserPageDTO dto) {
        KpiGroup kpiGroup = kpiGroupService.getById(dto.getId());
        BV.notNull(kpiGroup, "绩效组配置不存在");
        PageData<KpiGroupUser> pageData = kpiGroupUserService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<KpiGroupUser>lambdaQuery()
                        .eq(KpiGroupUser::getKpiGroupId, kpiGroup.getId())
                        .eq(KpiGroupUser::getDataDate, LocalDate.now())
                        .like(PublicUtil.isNotEmpty(dto.getStaffName()), KpiGroupUser::getUserName, dto.getStaffName())
                        .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );

        return PublicUtil.toPage(pageData, kpiGroupUser -> {
            return PublicUtil.copy(kpiGroupUser, KpiGroupUserVO.class);
        });
    }

    /**
     * 获取绩效池
     *
     * @param dto
     * @return
     */
    public AppPage<KpiPoolVO> queryKpiPool(KpiPoolQueryDTO dto) {
        if (PublicUtil.isNotEmpty(dto.getManagerUserId())) {
            dto.setManagerStaffList(commonService.queryManageStaffIds(dto.getManagerUserId(), dto.getMonthly()));
        } else {
            dto.setManagerStaffList(commonService.queryManageStaffIds(dto.getCurrentUserId(), dto.getMonthly()));
        }
        if (PublicUtil.isEmpty(dto.getManagerStaffList())) {
            return AppPage.empty(dto);
        }

        prepareParams(dto);
        AppPage<KpiPoolVO> page = AppPage.empty(dto);
        log.info("绩效池查询参数：{}", JSON.toJSONString(dto));
        long total = kpiPoolService.queryPagePoolTotal(dto);
        if (total > 0) {
            page.setTotal(total);
            List<KpiPoolVO> kpiPoolVOS = kpiPoolService.queryPagePool(dto);
            for (KpiPoolVO poolVO : kpiPoolVOS) {
                poolVO.setAverageKpiScoreRatio(poolVO.getAverageKpiScoreRatio().multiply(Constant.ONE_HUNDRED));
                poolVO.setKpiScoreRatio(poolVO.getKpiScoreRatio().multiply(Constant.ONE_HUNDRED));
            }
            page.setData(kpiPoolVOS);
        }
        return page;
    }


    /**
     * 绩效池详情
     *
     * @param kpiPoolId
     * @return
     */
    public KpiPoolDetailVO queryKpiPoolDetail(Long kpiPoolId) {
        KpiPool kpiPool = kpiPoolService.getById(kpiPoolId);
        BV.notNull(kpiPool, "绩效池数据不存在");
        Long kpiGroupId = kpiPool.getKpiGroupId();
//        LocalDate localDate = kpiPool.getMonthly();

        Long userId = kpiPool.getUserId();
        KpiPoolDetailVO kpiPoolDetailVo = PublicUtil.copy(kpiPool, KpiPoolDetailVO.class);

        List<KpiPool> kpiPools = kpiPoolService.list(Wrappers.<KpiPool>lambdaQuery()
                .ge(KpiPool::getMonthly, YearMonth.now().minusMonths(12L))
                .eq(KpiPool::getUserId, userId)
                .eq(KpiPool::getInclusion, Boolean.TRUE)
                .eq(KpiPool::getYn, Boolean.TRUE)
        );
        List<KpiPoolDetailVO> kpis = PublicUtil.copyList(kpiPools, KpiPoolDetailVO.class);
        kpiPoolDetailVo.setKpis(kpis);

        LocalDate now = LocalDate.now();
        List<KpiPoolIndicatorValueVO> kpiValueVos = kpiPoolIndicatorValueService.getStaffIndicatorValue(kpiPoolId, kpiGroupId, now);
        kpiPoolDetailVo.setKpiValues(kpiValueVos);
        return kpiPoolDetailVo;
    }

    /**
     * 查询薪酬池详情
     *
     * @param userId    用户id
     * @param yearMonth 月度
     * @return 薪酬池详情
     */
    public KpiPoolDetailVO queryPoolDetail(final Long userId, final YearMonth yearMonth) {
        List<KpiPool> list = userKpiPool(userId, yearMonth);
        BV.isNotEmpty(list, () -> "人员绩效数据不存在");
        KpiPool kpiPool = list.get(0);
        Long kpiGroupId = kpiPool.getKpiGroupId();
        Long kpiPoolId = kpiPool.getId();
        KpiPoolDetailVO kpiPoolDetailVo = PublicUtil.copy(kpiPool, KpiPoolDetailVO.class);

        List<KpiPool> kpiPools = kpiPoolService.list(Wrappers.<KpiPool>lambdaQuery()
                .ge(KpiPool::getMonthly, YearMonth.now().minusMonths(12L))
                .eq(KpiPool::getUserId, userId)
                .eq(KpiPool::getInclusion, Boolean.TRUE)
                .eq(KpiPool::getYn, Boolean.TRUE)
        );
        List<KpiPoolDetailVO> kpis = PublicUtil.copyList(kpiPools, KpiPoolDetailVO.class);
        kpiPoolDetailVo.setKpis(kpis);

        LocalDate now = LocalDate.now();
        List<KpiPoolIndicatorValueVO> kpiValueVos = kpiPoolIndicatorValueService.getStaffIndicatorValue(kpiPoolId, kpiGroupId, now);
        kpiPoolDetailVo.setKpiValues(kpiValueVos);
        return kpiPoolDetailVo;
    }

    /**
     * 参数赋值
     *
     * @param queryVO
     */
    private void prepareParams(KpiPoolQueryDTO queryVO) {
        if (Objects.isNull(queryVO.getMonthly())) {
            queryVO.setMonthly(YearMonth.now());
        }
        if (Objects.nonNull(queryVO.getOrder()) && StringUtils.isValid(queryVO.getOrderAtt())) {
            StringBuilder sb = new StringBuilder(" order by ");
            sb.append(StringUtils.toColumnName(queryVO.getOrderAtt()).toLowerCase());
            if (queryVO.getOrder() == 1) {
                sb.append(" asc ");
            } else {
                sb.append(" desc ");
            }
            queryVO.setOrderString(sb.toString());
        }
        if (PublicUtil.isNotEmpty(queryVO.getQueryStaffs())) {
            List<Long> queryStaffIds = Arrays.stream(queryVO.getQueryStaffs().split(",")).map(Long::parseLong).collect(Collectors.toList());
            queryVO.getManagerStaffList().retainAll(queryStaffIds);
        }
    }

    /**
     * 查询用户绩效池
     *
     * @param userId
     * @param yearMonth
     * @return
     */
    public Set<SelectorVO> queryUserKpiPool(Long userId, YearMonth yearMonth) {
        List<KpiPool> list = userKpiPool(userId, yearMonth);
        if (CollectionUtils.isEmpty(list)) {
            return new HashSet<>();
        }
        return list.stream().map(r -> SelectorVO.whit(r.getId(), r.getShopName(), r.getPostName())).collect(Collectors.toSet());
    }

    /**
     * 查询用户月度薪酬池列表
     *
     * @param userId    用户id
     * @param yearMonth 月度
     * @return 返回结果
     */
    @Cacheable(cacheNames = "kpi", key = "':user:pool:'+ #yearMonth.toString() +':'+ #userId", unless = "#result.size() < 1")
    public List<KpiPool> userKpiPool(final Long userId, final YearMonth yearMonth) {
        List<KpiPool> list = kpiPoolService.list(Wrappers.<KpiPool>lambdaQuery()
                .eq(KpiPool::getUserId, userId)
                .eq(KpiPool::getMonthly, yearMonth)
                .eq(KpiPool::getYn, Boolean.TRUE)
                .orderByDesc(KpiPool::getId)
        );
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list;
    }

    /**
     * 记录开始计算绩效时间
     *
     * @param userId
     */
    public LocalDate calcStartKpiDate(Long userId) {
        String key = getStartKpiKey() + userId;
        BoundValueOperations<String, String> strOps = stringRedisTemplate.boundValueOps(key);
        LocalDate currentDate = LocalDate.now();
        LocalDate startKpiDate = PublicUtil.getKpiDateOfTransferPostStaff(currentDate);
        long days = currentDate.until(startKpiDate, ChronoUnit.DAYS);
        //若定时器执行之前key失效，那么不能正确赋值开始计算考核时间，多加一天
        days = days + 1;
        strOps.set(startKpiDate.toString(), days, TimeUnit.DAYS);
        return startKpiDate;
    }

    /**
     * 获取开始计算绩效时间
     *
     * @param userId
     */
    public LocalDate getStartKpiDate(Long userId) {
        String key = getStartKpiKey() + userId;
        if (! stringRedisTemplate.hasKey(key)) {
            return null;
        }
        BoundValueOperations<String, String> strOps = stringRedisTemplate.boundValueOps(key);
        LocalDate startKpiDate = LocalDate.parse(strOps.get());
        LocalDate currentDate = LocalDate.now();
        //当前时间在开始计算绩效时间  之后，已经开始计算绩效
        if (currentDate.isAfter(startKpiDate)) {
            return null;
        }
        return startKpiDate;
    }

    /**
     * 记录待岗后开始计算绩效时间
     *
     * @param userId
     */
    public void calcAwaitJobAfterStartKpiDate(Long userId) {
        String key = getAwaitJobAfterStartKpiKey() + userId;
        BoundValueOperations<String, String> strOps = stringRedisTemplate.boundValueOps(key);
        LocalDate currentDate = LocalDate.now();
        LocalDate startKpiDate = PublicUtil.getKpiDateOfAwaitJobStaff(currentDate);
        long days = currentDate.until(startKpiDate, ChronoUnit.DAYS);
        //若定时器执行之前key失效，那么不能正确赋值开始计算考核时间，多加一天
        days = days + 1;
        strOps.set(startKpiDate.toString(), days, TimeUnit.DAYS);
    }

    /**
     * 检查员工是否在待岗保护期
     *
     * @param userId
     */
    public LocalDate getAfterAwaitJobStartKpiDate(Long userId) {
        String key = getAwaitJobAfterStartKpiKey() + userId;
        if (! stringRedisTemplate.hasKey(key)) {
            return null;
        }
        BoundValueOperations<String, String> strOps = stringRedisTemplate.boundValueOps(key);
        LocalDate startKpiDate = LocalDate.parse(strOps.get());
        LocalDate currentDate = LocalDate.now();
        //当前时间在开始计算绩效时间  之后，已经开始计算绩效
        if (currentDate.isAfter(startKpiDate)) {
            return null;
        }
        return startKpiDate;
    }

    /**
     * 查询绩效组不考核人员
     *
     * @param dto
     * @return
     */
    public AppPage<KpiGroupUserVO> queryKpiIgnoredStaffs(KpiGroupUserPageDTO dto) {
        PageData<KpiGroupUser> pageData = kpiGroupUserService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<KpiGroupUser>lambdaQuery()
                        .eq(KpiGroupUser::getKpiGroupId, dto.getId())
                        .eq(KpiGroupUser::getDataDate, LocalDate.now().plusDays(-1))
                        .eq(KpiGroupUser::getIgnored, Boolean.TRUE)
                        .like(PublicUtil.isNotEmpty(dto.getStaffName()), KpiGroupUser::getUserName, dto.getStaffName())
                        .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );

        return PublicUtil.toPage(pageData, kpiGroupUser -> {
            return PublicUtil.copy(kpiGroupUser, KpiGroupUserVO.class);
        });
    }


    /**
     * 查询绩效组排名分页列表
     *
     * @param dto
     * @return
     */
    public AppPage<KpiPoolVO> queryKpiPoolRank(KpiPoolRankQueryDTO dto) {
        List<String> kgcs;
        if (PublicUtil.isNotEmpty(dto.getKpiGroupRankId())) {
            KpiGroupRank kpiGroupRank = kpiGroupRankService.getById(dto.getKpiGroupRankId());
            BV.notNull(kpiGroupRank, "绩效排名组不存在");
            kgcs = kpiGroupRank.getKgcs();
        } else {
            KpiGroup kpiGroup = kpiGroupService.getById(dto.getKpiGroupId());
            KpiGroupRank kpiGroupRank = kpiGroupRankService.getOne(Wrappers.<KpiGroupRank>lambdaQuery()
                    .apply(MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, kgcs)", kpiGroup.getKgc()))
                    .eq(KpiGroupRank::getYn, Boolean.TRUE)
            );
            if (PublicUtil.isNotEmpty(kpiGroupRank)) {
                kgcs = kpiGroupRank.getKgcs();
            } else {
                kgcs = new ArrayList<String>(){{add(kpiGroup.getKgc());}};
            }
        }

        PageData<KpiPool> pageData = kpiPoolService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<KpiPool>lambdaQuery()
                        .in(KpiPool::getKgc, kgcs)
                        .eq(KpiPool::getMonthly, dto.getYearMonth())
                        .last("ORDER BY ISNULL(rank), rank ASC")
        );

        return PublicUtil.toPage(pageData, kpiPool -> {
            KpiPoolVO kpiPoolVO = PublicUtil.copy(kpiPool, KpiPoolVO.class);
            kpiPoolVO.setKpiScoreRatio(kpiPoolVO.getKpiScoreRatio().multiply(Constant.ONE_HUNDRED));
            kpiPoolVO.setAverageKpiScoreRatio(kpiPoolVO.getAverageKpiScoreRatio().multiply(Constant.ONE_HUNDRED));
            return kpiPoolVO;
        });
    }

    /**
     * 查询绩效配置
     *
     * @param kpiGroupId
     * @return
     */
    public List<KpiStarLaddersVO> queryKpiGroup(Long kpiGroupId) {
        KpiGroup kpiGroup = kpiGroupService.getById(kpiGroupId);
        BV.notNull(kpiGroup, "绩效配置不存在，请重试");
        List<KpiStarLadders> starLadders = kpiStarLaddersService.list(Wrappers.<KpiStarLadders>lambdaQuery()
                .eq(KpiStarLadders::getKpiGroupId, kpiGroupId)
                .eq(KpiStarLadders::getYn, Boolean.TRUE)
        );
        List<KpiStarLaddersVO> starLaddersVos = PublicUtil.copyList(starLadders, KpiStarLaddersVO.class);
        for (KpiStarLaddersVO starLaddersVO : starLaddersVos) {
            starLaddersVO.setUpper(starLaddersVO.getUpper().multiply(Constant.ONE_HUNDRED));
            starLaddersVO.setLower(starLaddersVO.getLower().multiply(Constant.ONE_HUNDRED));
        }
        return starLaddersVos;
    }

    /**
     * 查询绩效组人员
     *
     * @param kpiGroupId
     * @return
     */
    public List<KpiGroupUserVO> queryKpiGroupStaffs(Long kpiGroupId) {
        KpiGroup kpiGroup = kpiGroupService.getById(kpiGroupId);
        BV.notNull(kpiGroup, "绩效配置不存在，请重试");
        List<KpiGroupUser> kpiGroupUsers = this.fetchRealTimeKpiUser(kpiGroup.getPostId(), kpiGroup.getShopIds(), null);
        List<KpiGroupUserVO> kpiGroupUserVOS = PublicUtil.copyList(kpiGroupUsers, KpiGroupUserVO.class);
        return kpiGroupUserVOS;
    }


    /**
     * 查询绩效组排名所有人员
     *
     * @param kpiGroupIds
     * @return
     */
    public List<KpiGroupUserVO> queryRankStaffs(List<Long> kpiGroupIds) {
        List<KpiGroupUserVO> kpiGroupUserVOS = Lists.newArrayListWithCapacity(100);
        for (Long kpiGroupId : kpiGroupIds) {
            kpiGroupUserVOS.addAll(this.queryKpiGroupStaffs(kpiGroupId));
        }
        return kpiGroupUserVOS;
    }

    /**
     * 查询门店岗位所有人员
     *
     * @param
     * @return
     */
    public List<KpiGroupUserVO> queryShopsPostStaffs(List<ShopsPostQueryDTO> shopsPosts) {
        List<KpiGroupUserVO> allKpiGroupUserVOS = Lists.newArrayListWithCapacity(100);
        for (ShopsPostQueryDTO shopsPost : shopsPosts) {
            List<KpiGroupUser> kpiGroupUsers = this.fetchRealTimeKpiUser(shopsPost.getPostId(), shopsPost.getShopIds(), null);
            List<KpiGroupUserVO> kpiGroupUserVOS = PublicUtil.copyList(kpiGroupUsers, KpiGroupUserVO.class);
            allKpiGroupUserVOS.addAll(kpiGroupUserVOS);
        }
        return allKpiGroupUserVOS;
    }

}
