package cn.fw.morax.service.biz.salary;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.pojo.event.GroupSalaryCalcEvent;
import cn.fw.morax.common.utils.EventBusUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.SalaryClosure;
import cn.fw.morax.domain.db.salary.SalaryGeneralSettin;
import cn.fw.morax.domain.dto.salary.SalaryGeneralSettinDTO;
import cn.fw.morax.domain.vo.salary.SalaryGeneralSettinVO;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.service.data.salary.SalaryGeneralSettinService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.YearMonth;
import java.time.ZoneId;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class SalaryGeneralSettinBizService {

    private final SalaryGeneralSettinService salaryGeneralSettinService;

    @Value("${spring.cache.custom.global-prefix}:salary:system:save:")
    @Getter
    private String savePrefix;

    /**
     * 查询薪酬系统配置
     *
     * @param groupId
     * @return
     */
    public SalaryGeneralSettinVO getSystemConfig(Long groupId) {
        SalaryGeneralSettin salaryGeneralSettin = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaUpdate()
                .eq(SalaryGeneralSettin::getGroupId, groupId)
        );
        if (PublicUtil.isEmpty(salaryGeneralSettin)) {
            return new SalaryGeneralSettinVO();
        }
        SalaryGeneralSettinVO settingVo = PublicUtil.copy(salaryGeneralSettin, SalaryGeneralSettinVO.class);
        return settingVo;
    }


    /**
     * 薪酬系统配置保存
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getGroupId()", message = "保存中，请勿重复操作")
    public void systemConfig(SalaryGeneralSettinDTO dto) {
        this.checkSystemConfig(dto);
        SalaryGeneralSettin setting = PublicUtil.copy(dto, SalaryGeneralSettin.class);
        setting.setAppealDays(dto.getAppealDays());
        setting.setPayoffLimit(Optional.ofNullable(dto.getPayoffLimit()).orElse(0));
        salaryGeneralSettinService.save(setting);
    }

    /**
     * 检查配置
     *
     * @param dto
     */
    public void checkSystemConfig(SalaryGeneralSettinDTO dto) {
        BV.isTrue(dto.getAppealDays() < Constant.SALARY_APPEAL_MAX_DAYS, "申述日期最长为15天");
    }


}
