package cn.fw.morax.service.biz.salary;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.page.AppPage;
import cn.fw.ehr.sdk.api.enums.StaffStatusEnum;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.IndicatorUserValue;
import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.db.salary.SalaryGroupUser;
import cn.fw.morax.domain.dto.query.SalaryGroupStaffQueryDTO;
import cn.fw.morax.domain.dto.query.SalaryGroupUserPageDTO;
import cn.fw.morax.domain.enums.IndicatorValueTypeEnum;
import cn.fw.morax.domain.vo.salary.SalaryGroupUserVO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.service.data.kpi.IndicatorUserValueService;
import cn.fw.morax.service.data.salary.SalaryGroupService;
import cn.fw.morax.service.data.salary.SalaryGroupUserService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class SalaryGroupUserBizService {
    private final SalaryPoolCommonService salaryPoolCommonService;
    private final SalaryGroupUserService salaryGroupUserService;
    private final SalaryGroupService salaryGroupService;
    private final EhrRpcService ehrRpcService;
    private final IndicatorUserValueService indicatorUserValueService;
    @Value("${special-indicator-code.work-age}")
    @Getter
    private String workAgeCode;
//    @Value("${special-indicator-code.base-salary}")
//    @Getter
//    private String baseSalaryCode;
    @Value("${special-indicator-code.work-day-proportion}")
    @Getter
    private String workDayProportion;

    /**
     * 实时查询薪酬组人员
     *
     * @param dto
     * @return
     */
    public List<SalaryGroupUserVO> queryShopPostRealTimeStaffs(SalaryGroupStaffQueryDTO dto) {
        List<SalaryGroupUser> salaryGroupUsers = this.fetchRealTimeSalaryGroup(dto.getPostId(), dto.getShopIds(), dto.getStaffName());
        List<SalaryGroupUserVO> salaryGroupUserVOS = PublicUtil.copyList(salaryGroupUsers, SalaryGroupUserVO.class);
        return salaryGroupUserVOS;
    }


    /**
     * 查询薪酬组人员
     *
     * @param dto
     * @return
     */
    public AppPage<SalaryGroupUserVO> querySalaryGroupUsers(SalaryGroupUserPageDTO dto) {
        SalaryGroup salaryGroup = salaryGroupService.getById(dto.getId());
        BV.notNull(salaryGroup, "薪酬组配置不存在");
        PageData<SalaryGroupUser> pageData = salaryGroupUserService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<SalaryGroupUser>lambdaQuery()
                        .eq(SalaryGroupUser::getSalaryGroupId, dto.getId())
                        .eq(SalaryGroupUser::getDataDate, LocalDate.now())
                        .like(PublicUtil.isNotEmpty(dto.getStaffName()), SalaryGroupUser::getUserName, dto.getStaffName())
                        .eq(SalaryGroupUser::getYn, Boolean.TRUE)
        );
        return PublicUtil.toPage(pageData, salaryGroupUser -> PublicUtil.copy(salaryGroupUser, SalaryGroupUserVO.class));
    }


    /**
     * 同步薪酬组人员
     *
     * @param salaryGroupId
     */
    public void asyncSalaryGroupUser(Long salaryGroupId) {
        SalaryGroup salaryGroup = salaryGroupService.getById(salaryGroupId);
        BV.notNull(salaryGroup, () -> "薪酬组不存在");
        asyncSalaryGroupUser(salaryGroup, LocalDate.now());
    }


    /**
     * 同步薪酬组人员
     *
     * @param salaryGroup
     * @param date
     */
    @Transactional(rollbackFor = Exception.class)
    public void asyncSalaryGroupUser(SalaryGroup salaryGroup, LocalDate date) {
        if (!Boolean.TRUE.equals(salaryGroup.getYn())) {
            return;
        }
        List<Long> shopIds = salaryGroup.getShopIds();
        Long postId = salaryGroup.getPostId();
        String sgc = salaryGroup.getSgc();
        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaffByName(postId, shopIds, null);
        if (PublicUtil.isEmpty(staffs)) {
            return;
        }
        List<SalaryGroupUser> groupUsers = Lists.newArrayListWithCapacity(staffs.size());
        for (PerformanceStaffDTO staff : staffs) {
            groupUsers.add(this.convertStaffToGroupUserPO(staff, date, salaryGroup));
        }
        //保存薪酬组人员
        if (CollectionUtils.isNotEmpty(groupUsers)) {
            salaryGroupUserService.update(Wrappers.<SalaryGroupUser>lambdaUpdate()
                    .set(SalaryGroupUser::getYn, Boolean.FALSE)
                    .eq(SalaryGroupUser::getDataDate, date)
                    .eq(SalaryGroupUser::getSalaryGroupId, salaryGroup.getId())
            );
            salaryGroupUserService.saveBatch(groupUsers);
        }
        //保存指标数据
        for (PerformanceStaffDTO staff : staffs) {
            BigDecimal workDayProportion = salaryPoolCommonService.getWorkDayProportion(staff.getId(), sgc, date);
            saveRawData(staff, date, BigDecimal.valueOf(staff.getWorkAge()), workDayProportion);
//            saveRawData(staff, date, workDayProportion);
        }

    }

    /**
     * 存薪酬原始数据
     *
     * @param staff
     * @param date
     * @param indicatorValue
     */
//    private void saveRawData(PerformanceStaffDTO staff, LocalDate date, BigDecimal indicatorValue) {
//        String salaryCode = getBaseSalaryCode();
//        IndicatorUserValue rawData = new IndicatorUserValue();
//        rawData.setIndicatorCode(salaryCode);
//        Map<String, BigDecimal> map = new HashMap<>(1);
//        map.put(salaryCode, indicatorValue);
//        rawData.setIndicatorValue(JSONObject.toJSONString(map));
//        rawData.setValueType(IndicatorValueTypeEnum.RATIO);
//        rawData.setDataDate(date);
//        rawData.setUserId(staff.getId());
//        rawData.setGroupId(staff.getGroupId());
//        rawData.setYn(Boolean.TRUE);
//
//        indicatorUserValueService.remove(Wrappers.<IndicatorUserValue>lambdaQuery()
//                .eq(IndicatorUserValue::getUserId, staff.getId())
//                .eq(IndicatorUserValue::getIndicatorCode, salaryCode)
//                .eq(IndicatorUserValue::getDataDate, date)
//                .eq(IndicatorUserValue::getGroupId, staff.getGroupId())
//        );
//
//        indicatorUserValueService.save(rawData);
//    }

    private void saveRawData(PerformanceStaffDTO staff, LocalDate date, BigDecimal indicatorValue, BigDecimal workProportion) {
        String indicatorCode = getWorkAgeCode();
        IndicatorUserValue rawData = new IndicatorUserValue();
        rawData.setIndicatorCode(indicatorCode);
        Map<String, BigDecimal> map = new HashMap<>(2);
        map.put(indicatorCode, indicatorValue);
        map.put(getWorkDayProportion(), workProportion);
        rawData.setIndicatorValue(JSONObject.toJSONString(map));
        rawData.setValueType(IndicatorValueTypeEnum.WORK_AGE);
        rawData.setDataDate(date);
        rawData.setUserId(staff.getId());
        rawData.setGroupId(staff.getGroupId());
        rawData.setYn(Boolean.TRUE);

        indicatorUserValueService.remove(Wrappers.<IndicatorUserValue>lambdaQuery()
                .eq(IndicatorUserValue::getUserId, staff.getId())
                .eq(IndicatorUserValue::getIndicatorCode, indicatorCode)
                .eq(IndicatorUserValue::getDataDate, date)
                .eq(IndicatorUserValue::getGroupId, staff.getGroupId())
        );

        indicatorUserValueService.save(rawData);
    }


    /**
     * 查询薪酬组人员
     *
     * @param postId
     * @param shopIds
     * @return
     */
    public List<SalaryGroupUser> fetchRealTimeSalaryGroup(Long postId, List<Long> shopIds, String staffName) {
        final LocalDate now = LocalDate.now();
        return fetchRealTimeSalaryGroup(postId, shopIds, now, staffName);
    }

    /**
     * 查询薪酬组人员
     *
     * @param postId
     * @param shopIds
     * @param date
     * @return
     */
    public List<SalaryGroupUser> fetchRealTimeSalaryGroup(Long postId, List<Long> shopIds, LocalDate date, String staffName) {
        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaffByName(postId, shopIds, staffName);
        if (PublicUtil.isEmpty(staffs)) {
            return new ArrayList<>();
        }
        List<SalaryGroupUser> result = Lists.newArrayListWithCapacity(staffs.size());
        for (PerformanceStaffDTO staff : staffs) {
            result.add(this.convertStaffToGroupUser(staff, date));
        }
        return result;
    }

    /**
     * 转换员工基础信息
     *
     * @param staff
     * @return
     */
    private SalaryGroupUser convertStaffToGroupUser(PerformanceStaffDTO staff, LocalDate now) {
        return SalaryGroupUser.builder()
                .userId(staff.getId())
                .userName(staff.getName())
                .postId(staff.getPostId())
                .postName(staff.getPostName())
                .shopId(staff.getShopId())
                .shopName(staff.getShopName())
                .probationer(StaffStatusEnum.PROBATION.getValue().equals(staff.getStaffStatus()) ? Boolean.TRUE : Boolean.FALSE)
                .frozen(Boolean.FALSE) // TODO: 2022/5/5 维护冻结状态
                .groupId(staff.getGroupId())
                .dataDate(now)
                .build();
    }

    /**
     * 转换员工基础信息
     *
     * @param staff
     * @return
     */
    private SalaryGroupUser convertStaffToGroupUserPO(PerformanceStaffDTO staff, LocalDate now, SalaryGroup salaryGroup) {
        return SalaryGroupUser.builder()
                .sgc(salaryGroup.getSgc())
                .salaryGroupId(salaryGroup.getId())
                .userId(staff.getId())
                .userName(staff.getName())
                .postId(staff.getPostId())
                .postName(staff.getPostName())
                .shopId(staff.getShopId())
                .shopName(staff.getShopName())
                .probationer(StaffStatusEnum.PROBATION.getValue().equals(staff.getStaffStatus()) ? Boolean.TRUE : Boolean.FALSE)
                .frozen(Boolean.FALSE) // TODO: 2022/5/5 维护冻结状态
                .groupId(staff.getGroupId())
                .dataDate(now)
                .build();
    }

}
