package cn.fw.morax.service.component;

import cn.fw.morax.common.pojo.event.GroupSalaryCalcEvent;
import cn.fw.morax.domain.bo.salary.SalaryGroupCalculableBO;
import cn.fw.morax.service.data.salary.SalaryGroupService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.google.common.eventbus.Subscribe;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.util.Set;

@Component
@Slf4j
@RequiredArgsConstructor
public class CalcSalaryObserver {

    private final SalaryGroupService salaryGroupService;
    private final StringRedisTemplate stringRedisTemplate;

    @Value("${spring.cache.custom.global-prefix}:calculable:salary-group")
    @Getter
    private String calculableSalaryKey;


    /**
     * 计算集团最终薪酬
     *
     * @param event
     */
    @Subscribe
    public void calcSalary(GroupSalaryCalcEvent event) {
        log.info("计算集团最终薪酬[{}]", event);
        final Long groupId = event.getGroupId();
        final LocalDate dataDate = event.getDataDate();
        Set<Long> ids = salaryGroupService.querySalaryGroupId(dataDate, groupId);
        if (CollectionUtils.isEmpty(ids)) {
            return;
        }
        String[] array = ids.stream()
                .map(salaryGroupId -> new SalaryGroupCalculableBO(salaryGroupId, dataDate, Boolean.TRUE))
                .map(JSONObject::toJSONString)
                .toArray(String[]::new);
        stringRedisTemplate.opsForSet().add(getCalculableSalaryKey(), array);
    }

}