package cn.fw.morax.service.data.kpi.impl;

import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.dao.kpi.KpiGroupDao;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.dto.query.KpiGroupRepeatQueryDTO;
import cn.fw.morax.service.data.kpi.KpiGroupService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * @author kurisu
 */
@Service
@Slf4j
@AllArgsConstructor
public class KpiGroupServiceImpl extends ServiceImpl<KpiGroupDao, KpiGroup> implements KpiGroupService {

    @Override
    public List<Long> queryKgiGroupIdByIndCode(final String indicatorCode, final Long postId, final Long groupId, final Date dataDate) {
        return getBaseMapper().queryKgiGroupIdsByCode(indicatorCode, postId, groupId, dataDate);
    }

    @Override
    public List<KpiGroup> queryKgiGroupByIndCode(final String indicatorCode, final Long postId, final Long groupId, final Date dataDate) {
        return getBaseMapper().queryKgiGroupByCode(indicatorCode, postId, groupId, dataDate);
    }

    @Override
    public List<KpiGroup> queryRepeatKpiGroup(KpiGroupRepeatQueryDTO dto) {
        return getBaseMapper().queryRepeatKpiGroup(dto);
    }

    @Cacheable(cacheNames = "kpi_group", key = "#id", unless = "#result == null")
    @Override
    public KpiGroup getById(Serializable id) {
        return super.getById(id);
    }

    @CacheEvict(cacheNames = "kpi_group", key = "#entity.id", condition = "#result == true")
    @Override
    public boolean updateById(KpiGroup entity) {
        return super.updateById(entity);
    }

    @Override
    @Nullable
    public Set<Long> queryCalculableKpiGroupIds(@NonNull LocalDate localDate) {
        Date day = DateUtil.localDate2Date(localDate);
        return this.getBaseMapper().queryKgiGroupIdsByDay(day);
    }

    @Override
    public KpiGroup getAllDataById(Long id) {
        return this.getBaseMapper().getAllDataById(id);
    }

    @Override
    public KpiGroup getEffectGroup(Long shopId, Long postId, Date date) {
        return this.getBaseMapper().getEffectGroup(shopId, postId, date);
    }

    @Override
    public List<KpiGroup> getAllEffectGroups(Date date) {
        return this.getBaseMapper().getAllEffectGroups(date);
    }

    @Override
    public List<KpiGroup> getRepeatKpis(Long postId, List<Long> shopIds, Long preId, LocalDate beginTime) {
        return this.getBaseMapper().getRepeatKpis(postId, shopIds, preId, beginTime);
    }

    @Override
    public List<KpiGroup> queryKgiGroupsByDay(@NonNull LocalDate localDate) {
        Date day = DateUtil.localDate2Date(localDate);
        return this.getBaseMapper().queryKgiGroupsByDay(day);
    }

    @Override
    public List<KpiGroup> getKpis(Long postId, List<Long> shopIds, LocalDate day) {
        return this.getBaseMapper().getKpis(postId, shopIds, day);
    }
}
